@@ -12,7 +12,7 @@ DO_ZIP=${DO_ZIP:-0}
 DO_INSTALLER=${DO_INSTALLER:-1}
 DO_TESTS=${DO_TESTS:-1}
 DO_VERPATCH=${DO_VERPATCH:-1}
-DO_SERVICE=${DO_SERVICE:-0}
+DO_SERVICE=${DO_SERVICE:-1}
 CLIENT_ONLY=${CLIENT_ONLY:-0}
 RUN_INSTALLER=${RUN_INSTALLER:-1}
 DO_MSI=${DO_MSI:-0}
@@ -142,16 +142,32 @@ if [ "${DO_SERVICE}" == "1" ]; then
 	echo "* Compiling system service shim"
 	pushd "win32/service" > /dev/null
 	rm -f event_log.rc event_log.res MSG00409.bin Xpra-Service.exe
-	WINDOWS_KITS="C:\Program Files (x86)\\Windows Kits"
-	if [ -d "C:\\Program Files\\Windows Kits" ]; then
-		WINDOWS_KITS="C:\Program Files\\Windows Kits"
-	fi
-	MC="${WINDOWS_KITS}\\8.1\\bin\\x86\\mc.exe"
-	RC="${WINDOWS_KITS}\\8.1\\bin\\x86\\rc.exe"
-	LINK="C:\\Program Files\\Microsoft Visual Studio 14.0\\VC\\bin\\link.exe"
-	if  [ ! -e "${LINK}" ]; then
-		LINK="C:\\Program Files (x86)\\Microsoft Visual Studio 14.0\\VC\\bin\\link.exe"
-	fi
+	for KIT_DIR in "C:\Program Files\\Windows Kits" "C:\\Program Files (x86)\\Windows Kits"; do
+		for V in 8.1 10; do
+			if [ "${BITS}" == "64" ]; then
+				ARCH_DIRS="x64 x86"
+			else
+				ARCH_DIRS="x86"
+			fi
+			for B in $ARCH_DIRS; do
+				MC="${KIT_DIR}\\$V\\bin\\$B\\mc.exe"
+				RC="${KIT_DIR}\\$V\\bin\\$B\\rc.exe"
+				if [ -e "$MC" ]; then
+					echo "  using SDK $V $B found in:"
+					echo "  '$KIT_DIR'"
+					break 3
+				fi
+			done
+		done
+	done
+	for PF in "C:\\Program Files" "C:\\Program Files (x86)"; do
+		for VSV in 14.0 17.0 19.0; do
+			LINK="$PF\\Microsoft Visual Studio $VSV\\VC\\bin\\link.exe"
+			if  [ -e "${LINK}" ]; then
+				break 2
+			fi
+		done
+	done
 	#MC="C:\\Program Files\\Windows Kits\\8.1\\bin\\x64\\mc.exe"
 	#RC="C:\\Program Files\\Windows Kits\\8.1\\bin\\x64\\mc.exe"
 	EVENT_LOG_BUILD_LOG="event_log_build.log"
@@ -224,6 +240,10 @@ if [ "$?" != "0" ]; then
 	tail -n 20 "${CX_FREEZE_LOG}"
 	exit 1
 fi
+#fix case sensitive mess:
+mv ${DIST}/lib/girepository-1.0/Glib-2.0.typelib ${DIST}/lib/girepository-1.0/GLib-2.0.typelib.tmp
+mv ${DIST}/lib/girepository-1.0/GLib-2.0.typelib.tmp ${DIST}/lib/girepository-1.0/GLib-2.0.typelib
+
 #fixup cx_Logging, required by the service class before we can patch sys.path to find it:
 if [ -e "${DIST}/lib/cx_Logging.pyd" ]; then
 	mv "${DIST}/lib/cx_Logging.pyd" "${DIST}/"
@@ -353,11 +373,11 @@ fi
 if [ "${DO_VERPATCH}" == "1" ]; then
 	for exe in `ls dist/*exe | grep -v Plink.exe`; do
 		tool_name=`echo $exe | sed 's+dist/++g;s+Xpra_++g;s+Xpra-++g;s+_+ +g;s+-+ +g;s+\.exe++g'`
-		verpatch $exe				//s desc "Xpra $tool_name"		//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2017" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
+		verpatch $exe				//s desc "Xpra $tool_name"		//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2019" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
 	done
-	verpatch dist/Xpra_cmd.exe 		//s desc "Xpra command line"	//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2017" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
-	verpatch dist/Xpra-Proxy.exe	//s desc "Xpra Proxy Server"	//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2017" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
-	verpatch dist/Xpra.exe 			//s desc "Xpra" 				//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2017" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
+	verpatch dist/Xpra_cmd.exe 		//s desc "Xpra command line"	//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2019" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
+	verpatch dist/Xpra-Proxy.exe	//s desc "Xpra Proxy Server"	//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2019" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
+	verpatch dist/Xpra.exe 			//s desc "Xpra" 				//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2019" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
 fi
 
 ################################################################################
@@ -400,7 +420,7 @@ if [ "${DO_INSTALLER}" == "1" ]; then
 	if [ "$?" != "0" ]; then
 		echo "InnoSetup error - see ${INNOSETUP_LOG}:"
 		tail -n 20 "${INNOSETUP_LOG}"
-		rm "xpra.iss"
+		#rm "xpra.iss"
 		exit 1
 	fi
 	rm "xpra.iss"
