@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -9,9 +9,10 @@ import os
 import re
 
 from xpra.client.client_widget_base import ClientWidgetBase
-from xpra.os_util import bytestostr, PYTHON2, PYTHON3, OSX, WIN32
-from xpra.util import typedict, envbool, WORKSPACE_UNSET, WORKSPACE_NAMES
+from xpra.os_util import bytestostr, PYTHON2, PYTHON3, OSX, WIN32, is_Wayland
+from xpra.util import typedict, envbool, envint, WORKSPACE_UNSET, WORKSPACE_NAMES
 from xpra.log import Logger
+
 log = Logger("window")
 plog = Logger("paint")
 focuslog = Logger("focus")
@@ -21,18 +22,32 @@ keylog = Logger("keyboard")
 metalog = Logger("metadata")
 geomlog = Logger("geometry")
 iconlog = Logger("icon")
+alphalog = Logger("alpha")
 
 
 REPAINT_ALL = os.environ.get("XPRA_REPAINT_ALL", "")
 SIMULATE_MOUSE_DOWN = envbool("XPRA_SIMULATE_MOUSE_DOWN", True)
 PROPERTIES_DEBUG = [x.strip() for x in os.environ.get("XPRA_WINDOW_PROPERTIES_DEBUG", "").split(",")]
 AWT_DIALOG_WORKAROUND = envbool("XPRA_AWT_DIALOG_WORKAROUND", WIN32)
+SET_SIZE_CONSTRAINTS = envbool("XPRA_SET_SIZE_CONSTRAINTS", True)
+DEFAULT_GRAVITY = envint("XPRA_DEFAULT_GRAVITY", 0)
+OVERRIDE_GRAVITY = envint("XPRA_OVERRIDE_GRAVITY", 0)
 
 
 class ClientWindowBase(ClientWidgetBase):
 
-    def __init__(self, client, group_leader, watcher_pid, wid, wx, wy, ww, wh, bw, bh, metadata, override_redirect, client_properties, border, max_window_size, default_cursor_data, pixel_depth):
-        log("%s%s", type(self), (client, group_leader, watcher_pid, wid, wx, wy, ww, wh, bw, bh, metadata, override_redirect, client_properties, max_window_size, default_cursor_data, pixel_depth))
+    #(overriden in subclasses)
+    NAME_TO_HINT = {}
+
+    def __init__(self, client, group_leader, watcher_pid, wid,
+                 wx, wy, ww, wh, bw, bh,
+                 metadata, override_redirect, client_properties,
+                 border, max_window_size, default_cursor_data, pixel_depth):
+        log("%s%s", type(self),
+            (client, group_leader, watcher_pid, wid,
+             wx, wy, ww, wh, bw, bh,
+             metadata, override_redirect, client_properties,
+             border, max_window_size, default_cursor_data, pixel_depth))
         ClientWidgetBase.__init__(self, client, watcher_pid, wid, metadata.boolget("has-alpha"))
         self._override_redirect = override_redirect
         self.group_leader = group_leader
@@ -53,6 +68,7 @@ class ClientWindowBase(ClientWidgetBase):
         self._sticky = False
         self._iconified = False
         self._focused = False
+        self.window_gravity = OVERRIDE_GRAVITY or DEFAULT_GRAVITY
         self.border = border
         self.cursor_data = None
         self.default_cursor_data = default_cursor_data
@@ -81,13 +97,17 @@ class ClientWindowBase(ClientWidgetBase):
                 #client properties override application specified workspace value on init only:
                 metadata["workspace"] = int(workspace)
         self._window_workspace = WORKSPACE_UNSET        #will get set in set_metadata if present
-        self._desktop_workspace = self.get_desktop_workspace()
+        self._desktop_workspace = self.get_desktop_workspace()  #pylint: disable=assignment-from-none
         def wn(w):
             return WORKSPACE_NAMES.get(w, w)
-        workspacelog("init_window(..) workspace=%s, current workspace=%s", wn(self._window_workspace), wn(self._desktop_workspace))
+        workspacelog("init_window(..) workspace=%s, current workspace=%s",
+                     wn(self._window_workspace), wn(self._desktop_workspace))
         if self.max_window_size and b"size-constraints" not in metadata:
             #this ensures that we will set size-constraints and honour max_window_size:
             metadata[b"size-constraints"] = {}
+        #initialize gravity early:
+        sc = typedict(metadata.dictget("size-constraints", {}))
+        self.window_gravity = OVERRIDE_GRAVITY or sc.intget("gravity", DEFAULT_GRAVITY)
 
 
     def get_desktop_workspace(self):
@@ -105,6 +125,7 @@ class ClientWindowBase(ClientWidgetBase):
         self._backing = self.make_new_backing(backing_class, w, h, bw, bh)
         self._backing.border = self.border
         self._backing.default_cursor_data = self.default_cursor_data
+        self._backing.gravity = self.window_gravity
         return self._backing._backing
 
 
@@ -131,7 +152,7 @@ class ClientWindowBase(ClientWidgetBase):
                 #ie: "encodings.rgb_formats" -> "rgb_formats"
                 #ie: "encoding.full_csc_modes" -> "full_csc_modes"
                 ek = k.split(".", 1)[1]
-            except:
+            except IndexError:
                 ek = k
             dv = encoding_defaults.get(ek)
             if dv is not None and dv==v:
@@ -174,12 +195,23 @@ class ClientWindowBase(ClientWidgetBase):
 
     def update_metadata(self, metadata):
         metalog("update_metadata(%s)", metadata)
+        if self._client.readonly:
+            metadata.update(self._force_size_constraint(*self._size))
         self._metadata.update(metadata)
         try:
             self.set_metadata(metadata)
         except Exception:
             metalog.warn("failed to set window metadata to '%s'", metadata, exc_info=True)
 
+    def _force_size_constraint(self, *size):
+        return {
+            b"size-constraints" : {
+                b"maximum-size" : size,
+                b"minimum-size" : size,
+                b"base-size" : size,
+                }
+            }
+
     def set_metadata(self, metadata):
         metalog("set_metadata(%s)", metadata)
         debug_props = [x for x in PROPERTIES_DEBUG if x in metadata.keys()]
@@ -198,28 +230,30 @@ class ClientWindowBase(ClientWidgetBase):
                     #perform metadata variable substitutions:
                     #full of py3k unicode headaches that don't need to be
                     default_values = {
-                                      "title"           : "<untitled window>",
-                                      "client-machine"  : "<unknown machine>",
-                                      "windowid"        : str(self._id),
-                                      }
+                        "title"           : "<untitled window>",
+                        "client-machine"  : "<unknown machine>",
+                        "windowid"        : str(self._id),
+                        "server-machine"  : getattr(self._client, "_remote_hostname", None) or "<unknown machine>",
+                        "server-display"  : getattr(self._client, "_remote_display", None) or "<unknown display>",
+                        }
                     def metadata_replace(match):
                         atvar = match.group(0)          #ie: '@title@'
                         var = atvar[1:len(atvar)-1]     #ie: 'title'
+                        if not var:
+                            #atvar = "@@"
+                            return "@"
                         default_value = default_values.get(var, "<unknown %s>" % var)
-                        value = self._metadata.strget(var, default_value)
-                        if PYTHON2:
-                            value = value.decode("utf-8")
-                        return value
-                    title = re.sub("@[\w\-]*@", metadata_replace, title)
-                if PYTHON2:
-                    utf8_title = title.encode("utf-8")
-                else:
-                    utf8_title = title
+                        value = self._metadata.bytesget(var, default_value)
+                        try:
+                            return value.decode("utf-8")
+                        except UnicodeDecodeError:
+                            return str(value)
+                    title = re.sub(r"@[\w\-]*@", metadata_replace, title)
             except Exception as e:
                 log.error("Error parsing window title:")
                 log.error(" %s", e)
-                utf8_title = b""
-            self.set_title(utf8_title)
+                title = b""
+            self.set_title(title)
 
         if b"icon-title" in metadata:
             icon_title = metadata.strget("icon-title")
@@ -229,9 +263,10 @@ class ClientWindowBase(ClientWidgetBase):
             self.reset_icon()
 
         if b"size-constraints" in metadata:
-            self.size_constraints = typedict(metadata.dictget("size-constraints"))
-            self._set_initial_position = self.size_constraints.boolget("set-initial-position", self._set_initial_position)
-            self.set_size_constraints(self.size_constraints, self.max_window_size)
+            sc = typedict(metadata.dictget("size-constraints", {}))
+            self.size_constraints = sc
+            self._set_initial_position = sc.boolget("set-initial-position", self._set_initial_position)
+            self.set_size_constraints(sc, self.max_window_size)
 
         if b"set-initial-position" in metadata:
             #this should be redundant - but we keep it here for consistency
@@ -271,7 +306,7 @@ class ClientWindowBase(ClientWidgetBase):
         if b"has-alpha" in metadata:
             new_alpha = metadata.boolget("has-alpha")
             if new_alpha!=self._has_alpha:
-                l = metalog
+                l = alphalog
                 if PYTHON3 and not WIN32:
                     #win32 without opengl can't do transparency,
                     #so it triggers too many warnings
@@ -345,7 +380,7 @@ class ClientWindowBase(ClientWidgetBase):
             skip_pager = metadata.boolget("skip-pager")
             if self._skip_pager!=skip_pager:
                 self._skip_pager = skip_pager
-                self.set_skip_taskbar_hint(skip_pager)
+                self.set_skip_pager_hint(skip_pager)
 
         if b"workspace" in metadata:
             self.set_workspace(metadata.intget("workspace"))
@@ -354,22 +389,22 @@ class ClientWindowBase(ClientWidgetBase):
             self.set_bypass_compositor(metadata.intget("bypass-compositor"))
 
         if b"strut" in metadata:
-            self.set_strut(metadata.dictget("strut"))
+            self.set_strut(metadata.dictget("strut", {}))
 
         if b"fullscreen-monitors" in metadata:
             self.set_fullscreen_monitors(metadata.intlistget("fullscreen-monitors"))
 
         if b"shape" in metadata:
-            self.set_shape(metadata.dictget("shape"))
+            self.set_shape(metadata.dictget("shape", {}))
 
         if b"command" in metadata:
             self.set_command(metadata.strget("command"))
 
-        if b"menu" in metadata:
-            self.set_menu(metadata.dictget("menu"))
+        if b"x11-property" in metadata:
+            self.set_x11_property(*metadata.listget("x11-property"))
 
 
-    def set_menu(self, menu):
+    def set_x11_property(self, *x11_property):
         pass
 
     def set_command(self, command):
@@ -384,7 +419,7 @@ class ClientWindowBase(ClientWidgetBase):
     def set_bypass_compositor(self, v):
         pass        #see gtk client window base
 
-    def set_strut(self, d):
+    def set_strut(self, strut):
         pass        #see gtk client window base
 
     def set_fullscreen_monitors(self, fsm):
@@ -395,31 +430,43 @@ class ClientWindowBase(ClientWidgetBase):
 
 
     def set_size_constraints(self, size_constraints, max_window_size):
+        if not SET_SIZE_CONSTRAINTS:
+            return
         geomlog("set_size_constraints(%s, %s)", size_constraints, max_window_size)
         hints = typedict()
-        for (a, h1, h2) in [
+        client = self._client
+        for (a, h1, h2) in (
             (b"maximum-size", b"max_width", b"max_height"),
             (b"minimum-size", b"min_width", b"min_height"),
             (b"base-size", b"base_width", b"base_height"),
             (b"increment", b"width_inc", b"height_inc"),
-            ]:
-            if a in (b"base-size", b"increment"):
-                def closetoint(v):
-                    return abs(int(v)-v)<0.00001
-                int_scaling = closetoint(self._client.xscale) and closetoint(self._client.yscale)
-                if not int_scaling:
-                    #don't scale increment and base size constraints with non integer scaling values
-                    #as this would use rounding
-                    continue
+            ):
             v = size_constraints.intpair(a)
+            geomlog("intpair(%s)=%s", a, v)
             if v:
                 v1, v2 = v
-                hints[h1], hints[h2] = self._client.sp(v1, v2)
+                sv1 = client.sx(v1)
+                sv2 = client.sy(v2)
+                if a in (b"base-size", b"increment"):
+                    #rounding is not allowed for these values
+                    fsv1 = client.fsx(v1)
+                    fsv2 = client.fsy(v2)
+                    def closetoint(v):
+                        #tolerate some rounding error:
+                        #(ie: 2:3 scaling may not give an integer without a tiny bit of rounding)
+                        return abs(int(v)-v)<0.00001
+                    if not closetoint(fsv1) or not closetoint(fsv2):
+                        #the scaled value is not close to an int,
+                        #so we can't honour it:
+                        geomlog("cannot honour '%s' due to scaling, scaled values are not integers: %s, %s",
+                                a, fsv1, fsv2)
+                        continue
+                hints[h1], hints[h2] = sv1, sv2
         if not OSX:
-            for (a, h) in [
+            for (a, h) in (
                 (b"minimum-aspect-ratio", b"min_aspect"),
                 (b"maximum-aspect-ratio", b"max_aspect"),
-                ]:
+                ):
                 v = size_constraints.intpair(a)
                 if v:
                     v1, v2 = v
@@ -449,23 +496,27 @@ class ClientWindowBase(ClientWidgetBase):
             #not honouring "base" + "inc", but honouring just "min" instead:
             maxw = max(minw, maxw)
             maxh = max(minh, maxh)
-            geomlog("modified hints for max window size %s: %s (rw=%s, rh=%s) -> max=%sx%s", max_window_size, hints, rw, rh, maxw, maxh)
+            geomlog("modified hints for max window size %s: %s (rw=%s, rh=%s) -> max=%sx%s",
+                    max_window_size, hints, rw, rh, maxw, maxh)
             #ensure we don't have duplicates with bytes / strings,
             #and that keys are always "bytes":
             #(in practice this code should never fire, just here as a reminder)
             for x in ("max_width", "max_height"):
                 try:
                     del hints[x]
-                except:
+                except KeyError:
                     pass
-            hints[b"max_width"] = maxw
-            hints[b"max_height"] = maxh
+            #bug 2214: GTK3 on win32 gets confused if we specify a large max-size
+            # and it will mess up maximizing the window
+            if not WIN32 or PYTHON2 or (maxw<32000 or maxh<32000):
+                hints[b"max_width"] = maxw
+                hints[b"max_height"] = maxh
         try:
             geomlog("calling: %s(%s)", self.apply_geometry_hints, hints)
             #save them so the window hooks can use the last value used:
             self.geometry_hints = hints
             self.apply_geometry_hints(hints)
-        except:
+        except Exception:
             geomlog("set_size_constraints%s", (size_constraints, max_window_size), exc_info=True)
             geomlog.error("Error setting window hints:")
             for k,v in hints.items():
@@ -473,16 +524,19 @@ class ClientWindowBase(ClientWidgetBase):
             geomlog.error(" from size constraints:")
             for k,v in size_constraints.items():
                 geomlog.error(" %s=%s", k, v)
-        #TODO: handle gravity
-        #gravity = size_metadata.get("gravity")
+        self.window_gravity = OVERRIDE_GRAVITY or size_constraints.intget("gravity", DEFAULT_GRAVITY)
+        b = self._backing
+        if b:
+            b.gravity = self.window_gravity
+
 
     def set_window_type(self, window_types):
         hints = 0
         for window_type in window_types:
             #win32 workaround:
             if AWT_DIALOG_WORKAROUND and window_type=="DIALOG" and self._metadata.boolget("skip-taskbar"):
-                wm_class = self._metadata.strlistget("class-instance")
-                if wm_class and len(wm_class)==2 and wm_class[0].startswith("sun-awt-X11"):
+                wm_class = self._metadata.strlistget("class-instance", (None, None), 2, 2)
+                if wm_class and len(wm_class)==2 and wm_class[0] and wm_class[0].startswith("sun-awt-X11"):
                     #replace "DIALOG" with "NORMAL":
                     if "NORMAL" in window_types:
                         continue
@@ -576,7 +630,7 @@ class ClientWindowBase(ClientWidgetBase):
         if b:
             b.toggle()
             log("magic_key%s border=%s", args, b)
-            self.queue_draw(0, 0, *self._size)
+            self.queue_draw_area(0, 0, *self._size)
 
     def refresh_window(self, *args):
         log("refresh_window(%s) wid=%s", args, self._id)
@@ -588,7 +642,7 @@ class ClientWindowBase(ClientWidgetBase):
         #the "ClientWindow"
         self._client.send_refresh_all()
 
-    def draw_region(self, x, y, width, height, coding, img_data, rowstride, packet_sequence, options, callbacks):
+    def draw_region(self, x, y, width, height, coding, img_data, rowstride, _packet_sequence, options, callbacks):
         """ Note: this runs from the draw thread (not UI thread) """
         backing = self._backing
         if not backing:
@@ -597,20 +651,21 @@ class ClientWindowBase(ClientWidgetBase):
             fire_paint_callbacks(callbacks, -1, "no backing")
             return
         def after_draw_refresh(success, message=""):
-            plog("after_draw_refresh(%s, %s) %sx%s at %sx%s encoding=%s, options=%s", success, message, width, height, x, y, coding, options)
+            plog("after_draw_refresh(%s, %s) %sx%s at %sx%s encoding=%s, options=%s",
+                 success, message, width, height, x, y, coding, options)
             if success<=0:
                 return
             backing = self._backing
-            if backing and backing.draw_needs_refresh:
-                if REPAINT_ALL=="1" or self._client.xscale!=1 or self._client.yscale!=1:
+            if backing and (backing.draw_needs_refresh or REPAINT_ALL):
+                if REPAINT_ALL=="1" or self._client.xscale!=1 or self._client.yscale!=1 or is_Wayland():
                     rw, rh = self.get_size()
                     rx, ry = 0, 0
                 else:
                     rx, ry, rw, rh = self._client.srect(x, y, width, height)
-                if self.window_offset:
-                    rx += self.window_offset[0]
-                    ry += self.window_offset[1]
-                self.idle_add(self.queue_draw, rx, ry, rw, rh)
+                #if self.window_offset:
+                #    rx -= self.window_offset[0]
+                #    ry -= self.window_offset[1]
+                self.idle_add(self.queue_draw_area, rx, ry, rw, rh)
         #only register this callback if we actually need it:
         if backing.draw_needs_refresh:
             callbacks.append(after_draw_refresh)
@@ -629,7 +684,7 @@ class ClientWindowBase(ClientWidgetBase):
         #with normal windows, we just queue a draw request
         #and let the expose event paint the spinner
         w, h = self.get_size()
-        self.queue_draw(self, 0, 0, w, h)
+        self.queue_draw_area(0, 0, w, h)
 
     def can_have_spinner(self):
         if self._backing is None:
@@ -686,14 +741,14 @@ class ClientWindowBase(ClientWidgetBase):
             log.error(" this server does not support dbus-proxying")
             return
         rpc_args = [self._id]+args
-        return self._client.rpc_call("dbus", rpc_args, **kwargs)
+        self._client.rpc_call("dbus", rpc_args, **kwargs)
 
 
     def get_mouse_event_wid(self, _x, _y):
         #overriden in GTKClientWindowBase
         return self._id
 
-    def do_motion_notify_event(self, event):
+    def _do_motion_notify_event(self, event):
         if self._client.readonly or self._client.server_readonly or not self._client.server_pointer:
             return
         pointer, relative_pointer, modifiers, buttons = self._pointer_modifiers(event)
@@ -708,7 +763,7 @@ class ClientWindowBase(ClientWidgetBase):
     def _device_info(self, event):
         try:
             return event.device.get_name()
-        except:
+        except AttributeError:
             return ""
 
     def _button_action(self, button, event, depressed, *args):
@@ -716,7 +771,8 @@ class ClientWindowBase(ClientWidgetBase):
             return
         pointer, relative_pointer, modifiers, buttons = self._pointer_modifiers(event)
         wid = self.get_mouse_event_wid(*pointer)
-        mouselog("_button_action(%s, %s, %s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s", button, event, depressed, wid, self._client._focused, self._id, self._device_info(event), pointer, modifiers, buttons)
+        mouselog("_button_action(%s, %s, %s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s",
+                 button, event, depressed, wid, self._client._focused, self._id, self._device_info(event), pointer, modifiers, buttons)
         #map wheel buttons via translation table to support inverted axes:
         server_button = button
         if button>3:
