@@ -4,6 +4,8 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+#pylint: disable=no-member
+
 """ client_launcher.py
 
 This is a simple GUI for starting the xpra client.
@@ -29,7 +31,8 @@ from xpra.gtk_common.gtk_util import (
     FILE_CHOOSER_ACTION_SAVE, FILE_CHOOSER_ACTION_OPEN,
     )
 from xpra.util import DEFAULT_PORT, csv, repr_ellipsized
-from xpra.os_util import thread, WIN32, OSX, PYTHON3
+from xpra.os_util import WIN32, OSX, PYTHON3
+from xpra.make_thread import start_thread
 from xpra.client.gtk_base.gtk_tray_menu_base import (
     make_min_auto_menu, make_encodingsmenu,
     MIN_QUALITY_OPTIONS, QUALITY_OPTIONS, MIN_SPEED_OPTIONS, SPEED_OPTIONS,
@@ -133,7 +136,7 @@ class ApplicationWindow:
         self.config = make_defaults_struct(extras_defaults=LAUNCHER_DEFAULTS, extras_types=LAUNCHER_OPTION_TYPES, extras_validation=self.get_launcher_validation())
         ssh_cmd = parse_ssh_string(self.config.ssh)[0].strip().lower()
         self.is_putty = ssh_cmd.endswith("plink") or ssh_cmd.endswith("plink.exe")
-        self.is_paramiko = ssh_cmd=="paramiko"
+        self.is_paramiko = ssh_cmd.startswith("paramiko")
         #TODO: the fixup does not belong here?
         from xpra.scripts.main import fixup_options
         fixup_options(self.config)
@@ -151,7 +154,7 @@ class ApplicationWindow:
             import ssl
             assert ssl
             modes.append("ssl")
-        except:
+        except ImportError:
             pass
         #assume crypto is available
         modes.append("tcp + aes")
@@ -396,42 +399,35 @@ class ApplicationWindow:
         self.info.modify_fg(STATE_NORMAL, red)
         vbox.pack_start(self.info)
 
-        #hide encoding options by default
-        self.encoding_combo = None
-        self.encoding_options_check = None
-        self.encoding_box = None
+        hbox = gtk.HBox(False, 0)
+        hbox.set_spacing(20)
+        self.advanced_options_check = gtk.CheckButton("Advanced Options")
+        self.advanced_options_check.connect("toggled", self.advanced_options_toggled)
+        self.advanced_options_check.set_active(False)
+        al = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=0)
+        al.add(self.advanced_options_check)
+        hbox.pack_start(al)
+        vbox.pack_start(hbox)
+        self.advanced_box = gtk.VBox()
+        vbox.pack_start(self.advanced_box)
+
+        # Encoding:
+        hbox = gtk.HBox(False, 20)
+        hbox.set_spacing(20)
+        hbox.pack_start(gtk.Label("Encoding: "))
+        self.encoding_combo = OptionMenu()
+        encodings = ["auto"]+[x for x in PREFERED_ENCODING_ORDER]
+        server_encodings = encodings
+        es = make_encodingsmenu(self.get_current_encoding, self.set_new_encoding, encodings, server_encodings)
+        self.encoding_combo.set_menu(es)
+        hbox.pack_start(self.encoding_combo)
+        self.advanced_box.pack_start(hbox)
+        self.set_new_encoding(self.config.encoding)
         if not PYTHON3:
-            #not implemented for gtk3, where we can't use set_menu()...
-            hbox = gtk.HBox(False, 0)
-            hbox.set_spacing(20)
-            self.encoding_options_check = gtk.CheckButton("Advanced Encoding Options")
-            self.encoding_options_check.connect("toggled", self.encoding_options_toggled)
-            self.encoding_options_check.set_active(False)
-            al = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=0)
-            al.add(self.encoding_options_check)
-            hbox.pack_start(al)
-            vbox.pack_start(hbox)
-            self.encoding_box = gtk.VBox()
-            vbox.pack_start(self.encoding_box)
-
-            # Encoding:
-            hbox = gtk.HBox(False, 20)
-            hbox.set_spacing(20)
-            hbox.pack_start(gtk.Label("Encoding: "))
-            self.encoding_combo = OptionMenu()
-            def get_current_encoding():
-                return self.config.encoding
-            def set_new_encoding(e):
-                self.config.encoding = e
-            encodings = ["auto"]+[x for x in PREFERED_ENCODING_ORDER]
-            server_encodings = encodings
-            es = make_encodingsmenu(get_current_encoding, set_new_encoding, encodings, server_encodings)
-            self.encoding_combo.set_menu(es)
-            set_history_from_active(self.encoding_combo)
-            hbox.pack_start(self.encoding_combo)
-            self.encoding_box.pack_start(hbox)
             self.encoding_combo.connect("changed", self.encoding_changed)
-
+        #quality and speed options are not implemented for gtk3,
+        #where we can't use set_menu()...
+        if not PYTHON3:
             # Quality
             hbox = gtk.HBox(False, 20)
             hbox.set_spacing(20)
@@ -451,7 +447,7 @@ class ApplicationWindow:
             self.quality_combo.set_menu(sq)
             set_history_from_active(self.quality_combo)
             hbox.pack_start(self.quality_combo)
-            self.encoding_box.pack_start(hbox)
+            self.advanced_box.pack_start(hbox)
 
             # Speed
             hbox = gtk.HBox(False, 20)
@@ -472,8 +468,15 @@ class ApplicationWindow:
             self.speed_combo.set_menu(ss)
             set_history_from_active(self.speed_combo)
             hbox.pack_start(self.speed_combo)
-            self.encoding_box.pack_start(hbox)
-            self.encoding_box.hide()
+            self.advanced_box.pack_start(hbox)
+            self.advanced_box.hide()
+        # Sharing:
+        self.sharing = gtk.CheckButton("Sharing")
+        self.sharing.set_active(self.config.sharing)
+        self.sharing.set_tooltip_text("allow multiple concurrent users to connect")
+        al = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=0)
+        al.add(self.sharing)
+        self.advanced_box.pack_start(al)
 
         # Buttons:
         hbox = gtk.HBox(False, 20)
@@ -498,7 +501,7 @@ class ApplicationWindow:
 
         add_close_accel(self.window, self.accel_close)
         vbox.show_all()
-        self.encoding_options_toggled()
+        self.advanced_options_toggled()
         self.window.vbox = vbox
         self.window.add(vbox)
 
@@ -520,7 +523,9 @@ class ApplicationWindow:
                 ssh_port = int(ssh_port)
             except ValueError:
                 ssh_port = -1
-            errs.append((self.ssh_port_entry, ssh_port<0 or ssh_port>=2**16, "invalid SSH port number"))
+            errs.append((self.ssh_port_entry,
+                         ssh_port<0 or ssh_port>=2**16,
+                         "invalid SSH port number"))
         if sshtossh:
             #validate ssh port:
             proxy_port = self.proxy_port_entry.get_text()
@@ -528,7 +533,9 @@ class ApplicationWindow:
                 proxy_port = int(proxy_port)
             except ValueError:
                 proxy_port = -1
-            errs.append((self.proxy_port_entry, proxy_port<0 or proxy_port>=2**16, "invalid SSH port number"))
+            errs.append((self.proxy_port_entry,
+                         proxy_port<0 or proxy_port>=2**16,
+                         "invalid SSH port number"))
         port = self.port_entry.get_text()
         if sshtossh:
             if self.password_scb.get_active():
@@ -541,18 +548,24 @@ class ApplicationWindow:
                 self.username_entry.set_text(self.proxy_username_entry.get_text())
             else:
                 self.username_entry.set_sensitive(True)
-            errs.append((self.proxy_host_entry, not bool(self.proxy_host_entry.get_text()), "specify the proxy host"))
+            errs.append((self.proxy_host_entry,
+                         not bool(self.proxy_host_entry.get_text()),
+                         "specify the proxy host"))
         # check username *after* the checkbox action
         if ssh or sshtossh:
-            errs.append((self.username_entry, not bool(self.username_entry.get_text()), "specify username"))
+            errs.append((self.username_entry,
+                         not bool(self.username_entry.get_text()),
+                         "specify username"))
         if sshtossh:
-            errs.append((self.proxy_username_entry, not bool(self.proxy_username_entry.get_text()), "specify proxy username"))
+            errs.append((self.proxy_username_entry,
+                         not bool(self.proxy_username_entry.get_text()),
+                         "specify proxy username"))
         if ssh or sshtossh and not port:
             port = 0        #port optional with ssh
         else:
             try:
                 port = int(port)
-            except:
+            except (ValueError, TypeError):
                 port = -1
         errs.append((self.port_entry, port<0 or port>=2**16, "invalid port number"))
         err_text = []
@@ -650,6 +663,16 @@ class ApplicationWindow:
         else:
             self.nostrict_host_check.hide()
 
+    def get_current_encoding(self):
+        return self.config.encoding
+
+    def set_new_encoding(self, e):
+        if PYTHON3:
+            self.encoding_combo.set_label(e)
+        else:
+            set_history_from_active(self.encoding_combo)
+        self.config.encoding = e
+
     def get_selected_encoding(self, *_args):
         if not self.encoding_combo:
             return ""
@@ -667,14 +690,14 @@ class ApplicationWindow:
             self.quality_combo.hide()
             self.quality_label.hide()
 
-    def encoding_options_toggled(self, *_args):
-        if not self.encoding_box:
+    def advanced_options_toggled(self, *_args):
+        if not self.advanced_box:
             return
-        show_opts = self.encoding_options_check.get_active()
+        show_opts = self.advanced_options_check.get_active()
         if show_opts:
-            self.encoding_box.show()
+            self.advanced_box.show()
         else:
-            self.encoding_box.hide()
+            self.advanced_box.hide()
 
     def reset_errors(self):
         self.set_sensitive(True)
@@ -757,6 +780,7 @@ class ApplicationWindow:
     def connect_builtin(self):
         #cooked vars used by connect_to
         params = {"type"    : self.config.mode}
+        self.config.sharing = self.sharing.get_active()
         username = self.config.username
         if self.config.mode=="ssh" or self.config.mode=="ssh -> ssh":
             if self.config.socket_dir:
@@ -834,7 +858,7 @@ class ApplicationWindow:
         self.client.init(self.config)
         self.client.username = username
         self.set_info_text("Connecting...")
-        thread.start_new_thread(self.do_connect_builtin, (params,))
+        start_thread(self.do_connect_builtin, "connect", daemon=True, args=(params,))
 
     def ssh_failed(self, message):
         log("ssh_failed(%s)", message)
@@ -857,7 +881,6 @@ class ApplicationWindow:
             self.handle_exception(e)
             return
         log("connect_to(..)=%s, hiding launcher window, starting client", conn)
-        glib.idle_add(self.window.hide)
         glib.idle_add(self.start_XpraClient, conn, display_desc)
 
 
@@ -874,6 +897,7 @@ class ApplicationWindow:
         self.client.display_desc = display_desc
         self.client.init_ui(self.config)
         self.client.setup_connection(conn)
+        self.set_info_text("Connection established")
         log("start_XpraClient() client initialized")
 
         if self.config.password:
@@ -919,6 +943,13 @@ class ApplicationWindow:
 
         self.client.warn_and_quit = warn_and_quit_override
         self.client.quit = quit_override
+        def after_handshake():
+            self.set_info_text("Handshake complete")
+        self.client.after_handshake(after_handshake)
+        def first_ui_received(*_args):
+            self.set_info_text("Running")
+            self.window.hide()
+        self.client.connect("first-ui-received", first_ui_received)
         try:
             self.client.run()
             log("client.run() returned")
@@ -998,14 +1029,17 @@ class ApplicationWindow:
                 break
         self.mode_combo.set_active(active)
         if self.config.encoding and self.encoding_combo:
-            index = self.encoding_combo.get_menu().encoding_to_index.get(self.config.encoding, -1)
-            log("setting encoding combo to %s / %s", self.config.encoding, index)
-            #make sure the right one is the only one selected:
-            for i,item in enumerate(self.encoding_combo.get_menu().get_children()):
-                item.set_active(i==index)
-            #then select it in the combo:
-            if index>=0:
-                self.encoding_combo.set_history(index)
+            if PYTHON3:
+                self.set_new_encoding(self.config.encoding)
+            else:
+                index = self.encoding_combo.get_menu().encoding_to_index.get(self.config.encoding, -1)
+                log("setting encoding combo to %s / %s", self.config.encoding, index)
+                #make sure the right one is the only one selected:
+                for i,item in enumerate(self.encoding_combo.get_menu().get_children()):
+                    item.set_active(i==index)
+                #then select it in the combo:
+                if index>=0:
+                    self.encoding_combo.set_history(index)
         self.username_entry.set_text(self.config.username)
         self.password_entry.set_text(self.config.password)
         self.host_entry.set_text(self.config.host)
@@ -1030,6 +1064,7 @@ class ApplicationWindow:
         self.proxy_password_entry.set_text(self.config.proxy_password)
         if self.is_putty:
             self.proxy_key_entry.set_text(self.config.proxy_key)
+        self.sharing.set_active(bool(self.config.sharing))
 
     def close_window(self, *_args):
         w = self.window
@@ -1153,7 +1188,9 @@ def do_main(argv):
         app = ApplicationWindow()
         def handle_signal(signum):
             app.show()
-            app.client.cleanup()
+            client = app.client
+            if client:
+                client.cleanup()
             glib.timeout_add(1000, app.set_info_text, "got signal %s" % SIGNAMES.get(signum, signum))
             glib.timeout_add(1000, app.set_info_color, True)
         register_os_signals(handle_signal)
