@@ -15,7 +15,10 @@ from xpra.util import (
     updict, pver, iround, flatten_dict, envbool, repr_ellipsized, csv, first_time,
     DEFAULT_METADATA_SUPPORTED, XPRA_OPENGL_NOTIFICATION_ID,
     )
-from xpra.os_util import bytestostr, strtobytes, hexstr, monotonic_time, WIN32, OSX, POSIX
+from xpra.os_util import (
+    bytestostr, strtobytes, hexstr, monotonic_time,
+    WIN32, OSX, POSIX, PYTHON3, is_Wayland,
+    )
 from xpra.simple_stats import std_unit
 from xpra.exit_codes import EXIT_PASSWORD_REQUIRED
 from xpra.scripts.config import TRUE_OPTIONS, FALSE_OPTIONS
@@ -28,7 +31,7 @@ from xpra.gtk_common.gtk_util import (
     get_screen_sizes, load_contents_async, load_contents_finish, GDKWindow,
     FILE_CHOOSER_ACTION_OPEN,
     CLASS_INPUT_ONLY,
-    RESPONSE_CANCEL, RESPONSE_OK, RESPONSE_ACCEPT,
+    RESPONSE_CANCEL, RESPONSE_OK, RESPONSE_ACCEPT, RESPONSE_REJECT,
     INTERP_BILINEAR, WINDOW_TOPLEVEL, DIALOG_MODAL, DESTROY_WITH_PARENT, MESSAGE_INFO,
     BUTTONS_CLOSE, ICON_SIZE_BUTTON, GRAB_STATUS_STRING,
     BUTTON_PRESS_MASK, BUTTON_RELEASE_MASK, POINTER_MOTION_MASK, POINTER_MOTION_HINT_MASK,
@@ -42,7 +45,7 @@ from xpra.client.mixins.window_manager import WindowClient
 from xpra.platform.paths import get_icon_filename
 from xpra.platform.gui import (
     get_window_frame_sizes, get_window_frame_size,
-    system_bell, get_wm_name, get_fixed_cursor_size, get_menu_support_function,
+    system_bell, get_wm_name, get_fixed_cursor_size,
     )
 from xpra.log import Logger
 
@@ -91,13 +94,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         self.border = None
         self.data_send_requests = {}
         #clipboard bits:
-        self.local_clipboard_requests = 0
-        self.remote_clipboard_requests = 0
         self.clipboard_notification_timer = None
         self.last_clipboard_notification = 0
-        #only used with the translated clipboard class:
-        self.local_clipboard = ""
-        self.remote_clipboard = ""
         #opengl bits:
         self.client_supports_opengl = False
         self.opengl_enabled = False
@@ -110,7 +108,6 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         #group leader bits:
         self._ref_to_group_leader = {}
         self._group_leader_wids = {}
-        self._set_window_menu = get_menu_support_function()
         try:
             self.connect("scaling-changed", self.reset_windows_cursors)
         except TypeError:
@@ -123,8 +120,6 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     def init(self, opts):
         GObjectXpraClient.init(self, opts)
         UIXpraClient.init(self, opts)
-        self.remote_clipboard = opts.remote_clipboard
-        self.local_clipboard = opts.local_clipboard
 
 
     def setup_frame_request_windows(self):
@@ -208,7 +203,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
     def start_UI_watcher(self, _client):
         from xpra.platform.ui_thread_watcher import get_UI_watcher
-        self.UI_watcher = get_UI_watcher(self.timeout_add)
+        self.UI_watcher = get_UI_watcher(self.timeout_add, self.source_remove)
         self.UI_watcher.start()
         #if server supports it, enable UI thread monitoring workaround when needed:
         def UI_resumed():
@@ -238,8 +233,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         return ncs
 
 
-    def _process_startup_complete(self, *args):
-        UIXpraClient._process_startup_complete(self, *args)
+    def _process_startup_complete(self, packet):
+        UIXpraClient._process_startup_complete(self, packet)
         gdk.notify_startup_complete()
 
 
@@ -247,8 +242,6 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         dialog = gtk.Dialog("Server Authentication",
                None,
                DIALOG_MODAL | DESTROY_WITH_PARENT)
-        RESPONSE_REJECT = 1
-        RESPONSE_ACCEPT = 2
         dialog.add_button(gtk.STOCK_CANCEL, RESPONSE_REJECT)
         dialog.add_button(gtk.STOCK_OK,     RESPONSE_ACCEPT)
         def add(widget, padding=0):
@@ -364,7 +357,9 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             from xpra.client.gtk_base.start_new_command import getStartNewCommand
             def run_command_cb(command, sharing=True):
                 self.send_start_command(command, command, False, sharing)
-            self.start_new_command = getStartNewCommand(run_command_cb, self.server_sharing and self.server_window_filters, self.xdg_menu)
+            self.start_new_command = getStartNewCommand(run_command_cb,
+                                                        self.server_sharing and self.server_window_filters,
+                                                        self.server_xdg_menu)
         self.start_new_command.show()
 
 
@@ -551,7 +546,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             log("get_pixbuf(%s) icon_filename=%s", icon_name, icon_filename)
             if icon_filename:
                 return pixbuf_new_from_file(icon_filename)
-        except:
+        except Exception:
             log.error("get_pixbuf(%s)", icon_name, exc_info=True)
         return None
 
@@ -563,7 +558,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             if not pixbuf:
                 return  None
             return scaled_image(pixbuf, size)
-        except:
+        except Exception:
             log.error("get_image(%s, %s)", icon_name, size, exc_info=True)
             return None
 
@@ -581,7 +576,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         #try native platform code first:
         x, y = window.get_position()
         w, h = window.get_size()
-        v = get_window_frame_size(x, y, w, h)
+        v = get_window_frame_size(x, y, w, h)   #pylint: disable=assignment-from-none
         framelog("get_window_frame_size%s=%s", (x, y, w, h), v)
         if v:
             #(OSX does give us these values via Quartz API)
@@ -602,8 +597,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             v = self.get_frame_extents(self.frame_request_window)
             if v:
                 try:
-                    wm_name = get_wm_name()
-                except:
+                    wm_name = get_wm_name() #pylint: disable=assignment-from-none
+                except Exception:
                     wm_name = None
                 try:
                     if len(v)==8:
@@ -646,25 +641,6 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         return True
 
 
-    def cook_metadata(self, new_window, metadata):
-        metadata = UIXpraClient.cook_metadata(self, new_window, metadata)
-        #ensures we will call set_window_menu for this window when we create it:
-        if new_window and self._set_window_menu:
-            menu = metadata.dictget("menu")
-            if menu is None:
-                metadata["menu"] = {}
-        return metadata
-
-
-    def set_window_menu(self, add, wid, menus, application_action_callback=None, window_action_callback=None):
-        assert self._set_window_menu
-        model = self._id_to_window.get(wid)
-        window = None
-        if model:
-            window = model.get_window()
-        self._set_window_menu(add, wid, window, menus, application_action_callback, window_action_callback)
-
-
     def get_root_window(self):
         return get_default_root_window()
 
@@ -677,34 +653,39 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         return self.cp(p[0], p[1])
 
     def get_current_modifiers(self):
-        modifiers_mask = self.get_root_window().get_pointer()[-1]
+        root = self.get_root_window()
+        if root is None:
+            return ()
+        modifiers_mask = root.get_pointer()[-1]
         return self.mask_to_names(modifiers_mask)
 
 
     def make_hello(self):
         capabilities = UIXpraClient.make_hello(self)
         capabilities["named_cursors"] = len(cursor_types)>0
+        capabilities["transparency"] = self.has_transparency()
         capabilities.update(flatten_dict(get_gtk_version_info()))
         if EXPORT_ICON_DATA:
             #tell the server which icons GTK can use
             #so it knows when it should supply one as fallback
             it = icon_theme_get_default()
-            #this would add our bundled icon directory
-            #to the search path, but I don't think we have
-            #any extra icons that matter in there:
-            #from xpra.platform.paths import get_icon_dir
-            #d = get_icon_dir()
-            #if d not in it.get_search_path():
-            #    it.append_search_path(d)
-            #    it.rescan_if_needed()
-            log("default icon theme: %s", it)
-            log("icon search path: %s", it.get_search_path())
-            log("contexts: %s", it.list_contexts())
-            icons = []
-            for context in it.list_contexts():
-                icons += it.list_icons(context)
-            log("icons: %s", icons)
-            capabilities["theme.default.icons"] = tuple(set(icons))
+            if it:
+                #this would add our bundled icon directory
+                #to the search path, but I don't think we have
+                #any extra icons that matter in there:
+                #from xpra.platform.paths import get_icon_dir
+                #d = get_icon_dir()
+                #if d not in it.get_search_path():
+                #    it.append_search_path(d)
+                #    it.rescan_if_needed()
+                log("default icon theme: %s", it)
+                log("icon search path: %s", it.get_search_path())
+                log("contexts: %s", it.list_contexts())
+                icons = []
+                for context in it.list_contexts():
+                    icons += it.list_icons(context)
+                log("icons: %s", icons)
+                capabilities["theme.default.icons"] = tuple(set(icons))
         if METADATA_SUPPORTED:
             ms = [x.strip() for x in METADATA_SUPPORTED.split(",")]
         else:
@@ -723,10 +704,10 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             #this is only really supported on X11, but posix is easier to check for..
             #"strut" and maybe even "fullscreen-monitors" could also be supported on other platforms I guess
             ms += ["shaded", "bypass-compositor", "strut", "fullscreen-monitors"]
-        if HAS_X11_BINDINGS and XSHAPE:
-            ms += ["shape"]
-        if self._set_window_menu:
-            ms += ["menu"]
+        if HAS_X11_BINDINGS:
+            ms += ["x11-property"]
+            if XSHAPE:
+                ms += ["shape"]
         #figure out if we can handle the "global menu" stuff:
         if POSIX and not OSX:
             try:
@@ -762,7 +743,10 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
 
     def has_transparency(self):
-        return screen_get_default().get_rgba_visual() is not None
+        screen = screen_get_default()
+        if screen is None:
+            return is_Wayland()
+        return screen.get_rgba_visual() is not None
 
 
     def get_screen_sizes(self, xscale=1, yscale=1):
@@ -808,7 +792,12 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 gdk_cursor = cursor_types.get(cursor_name.upper())
                 if gdk_cursor is not None:
                     cursorlog("setting new cursor by name: %s=%s", cursor_name, gdk_cursor)
-                    return new_Cursor_for_display(display, gdk_cursor)
+                    try:
+                        return new_Cursor_for_display(display, gdk_cursor)
+                    except TypeError as e:
+                        log("new_Cursor_for_display(%s, %s)", display, gdk_cursor, exc_info=True)
+                        if first_time("cursor:%s" % cursor_name.upper()):
+                            log.error("Error creating cursor %s: %s", cursor_name.upper(), e)
                 global missing_cursor_names
                 if cursor_name not in missing_cursor_names:
                     cursorlog("cursor name '%s' not found", cursor_name)
@@ -982,16 +971,17 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         if enable_option in FALSE_OPTIONS:
             self.opengl_props["info"] = "disabled by configuration"
             return
-        from xpra.scripts.config import OpenGL_safety_check
-        from xpra.platform.gui import gl_check as platform_gl_check
         warnings = []
-        for check in (OpenGL_safety_check, platform_gl_check):
-            opengllog("checking with %s", check)
-            warning = check()
-            opengllog("%s()=%s", check, warning)
-            if warning:
-                warnings.append(warning)
         self.opengl_props["info"] = ""
+        if enable_option!="probe-success":
+            from xpra.scripts.config import OpenGL_safety_check
+            from xpra.platform.gui import gl_check as platform_gl_check
+            for check in (OpenGL_safety_check, platform_gl_check):
+                opengllog("checking with %s", check)
+                warning = check()
+                opengllog("%s()=%s", check, warning)
+                if warning:
+                    warnings.append(warning)
 
         def err(msg, e):
             opengllog("OpenGL initialization error", exc_info=True)
@@ -1042,6 +1032,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             self.gl_texture_size_limit = self.opengl_props.get("texture-size-limit", 16*1024)
             self.gl_max_viewport_dims = self.opengl_props.get("max-viewport-dims",
                                                               (self.gl_texture_size_limit, self.gl_texture_size_limit))
+            driver_info = self.opengl_props.get("renderer") or self.opengl_props.get("vendor") or "unknown card"
             if min(self.gl_max_viewport_dims)<4*1024:
                 opengllog.warn("Warning: OpenGL is disabled:")
                 opengllog.warn(" the maximum viewport size is too low: %s", self.gl_max_viewport_dims)
@@ -1050,6 +1041,10 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 opengllog.warn("Warning: OpenGL is disabled:")
                 opengllog.warn(" the texture size limit is too low: %s", self.gl_texture_size_limit)
                 self.opengl_enabled = False
+            elif driver_info.startswith("SVGA3D") and os.environ.get("WAYLAND_DISPLAY"):
+                opengllog.warn("Warning: OpenGL is disabled:")
+                opengllog.warn(" SVGA3D driver is buggy under Wayland")
+                self.opengl_enabled = False
             self.GLClientWindowClass.MAX_VIEWPORT_DIMS = self.gl_max_viewport_dims
             self.GLClientWindowClass.MAX_BACKING_DIMS = self.gl_texture_size_limit, self.gl_texture_size_limit
             mww, mwh = self.max_window_size
@@ -1067,13 +1062,12 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 l("Warning: OpenGL windows will be clamped to the maximum texture size %ix%i",
                   self.gl_texture_size_limit, self.gl_texture_size_limit)
                 l(" for OpenGL %s renderer '%s'", pver(self.opengl_props.get("opengl", "")), self.opengl_props.get("renderer", "unknown"))
-            if self.opengl_enabled:
+            if self.opengl_enabled and enable_opengl!="probe-success":
                 draw_result = test_gl_client_window(self.GLClientWindowClass, max_window_size=self.max_window_size, pixel_depth=self.pixel_depth)
                 if not draw_result.get("success", False):
                     err("OpenGL test rendering failed:", draw_result.get("message", "unknown error"))
                     return
                 log("OpenGL test rendering succeeded")
-            driver_info = self.opengl_props.get("renderer") or self.opengl_props.get("vendor") or "unknown card"
             if self.opengl_enabled:
                 opengllog.info("OpenGL enabled with %s", driver_info)
                 #don't try to handle video dimensions bigger than this:
@@ -1090,9 +1084,10 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             opengllog("init_opengl(%s)", enable_opengl, exc_info=True)
 
     def get_client_window_classes(self, w, h, metadata, override_redirect):
-        log("get_client_window_class%s GLClientWindowClass=%s, opengl_enabled=%s, mmap_enabled=%s, encoding=%s",
+        log("get_client_window_class%s ClientWindowClass=%s, GLClientWindowClass=%s, opengl_enabled=%s, mmap_enabled=%s, encoding=%s",
             (w, h, metadata, override_redirect),
-            self.GLClientWindowClass,self.opengl_enabled, self.mmap_enabled, self.encoding)
+            self.ClientWindowClass, self.GLClientWindowClass,
+            self.opengl_enabled, self.mmap_enabled, self.encoding)
         if self.GLClientWindowClass is None or not self.opengl_enabled:
             return (self.ClientWindowClass,)
         #verify texture limits:
@@ -1105,8 +1100,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 return (self.ClientWindowClass,)
             if metadata.boolget("has-alpha", False):
                 return (self.ClientWindowClass,)
-        if OSX:
-            #OSX doesn't do alpha:
+        if OSX and not PYTHON3:
+            #GTK2 on OSX doesn't do alpha:
             if metadata.boolget("has-alpha", False):
                 return (self.ClientWindowClass,)
         return (self.GLClientWindowClass, self.ClientWindowClass)
@@ -1208,106 +1203,10 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         group_leader.destroy()
 
 
-    def get_clipboard_helper_classes(self):
-        ct = self.client_clipboard_type
-        if ct and ct.lower() in FALSE_OPTIONS:
-            return []
-        from xpra.platform.features import CLIPBOARD_NATIVE_CLASS
-        from xpra.scripts.main import CLIPBOARD_CLASS
-        #first add the platform specific one, (may be None):
-        clipboard_options = []
-        if CLIPBOARD_CLASS:
-            clipboard_options.append(CLIPBOARD_CLASS)
-        if CLIPBOARD_NATIVE_CLASS:
-            clipboard_options.append(CLIPBOARD_NATIVE_CLASS)
-        clipboard_options.append("xpra.clipboard.gdk_clipboard.GDKClipboardProtocolHelper")
-        clipboard_options.append("xpra.clipboard.clipboard_base.DefaultClipboardProtocolHelper")
-        clipboard_options.append("xpra.clipboard.translated_clipboard.TranslatedClipboardProtocolHelper")
-        clipboardlog("get_clipboard_helper_classes() unfiltered list=%s", clipboard_options)
-        if ct and ct.lower()!="auto" and ct.lower() not in TRUE_OPTIONS:
-            #try to match the string specified:
-            filtered = [x for x in clipboard_options if x.lower().find(self.client_clipboard_type)>=0]
-            if not filtered:
-                clipboardlog.warn("Warning: no clipboard types matching '%s'", self.client_clipboard_type)
-                clipboardlog.warn(" clipboard synchronization is disabled")
-                return []
-            clipboardlog(" found %i clipboard types matching '%s'", len(filtered), self.client_clipboard_type)
-            clipboard_options = filtered
-        #now try to load them:
-        clipboardlog("get_clipboard_helper_classes() options=%s", clipboard_options)
-        loadable = []
-        for co in clipboard_options:
-            try:
-                parts = co.split(".")
-                mod = ".".join(parts[:-1])
-                module = __import__(mod, {}, {}, [parts[-1]])
-                helperclass = getattr(module, parts[-1])
-                loadable.append(helperclass)
-            except ImportError:
-                clipboardlog("cannot load %s", co, exc_info=True)
-                continue
-        clipboardlog("get_clipboard_helper_classes()=%s", loadable)
-        return loadable
-
-    def process_clipboard_packet(self, packet):
-        clipboardlog("process_clipboard_packet(%s) level=%s", packet, gtk.main_level())
-        #check for clipboard loops:
-        from xpra.clipboard.clipboard_base import nesting_check
-        if not nesting_check():
-            self.clipboard_enabled = False
-            self.emit("clipboard-toggled")
-            return
-        self.idle_add(self.clipboard_helper.process_clipboard_packet, packet)
-
-
     def setup_clipboard_helper(self, helperClass):
-        clipboardlog("setup_clipboard_helper(%s)", helperClass)
-        #first add the platform specific one, (may be None):
-        from xpra.platform.features import CLIPBOARDS
-        kwargs= {
-                #all the local clipboards supported:
-                 "clipboards.local"     : CLIPBOARDS,
-                 #all the remote clipboards supported:
-                 "clipboards.remote"    : self.server_clipboards,
-                 "can-send"             : self.client_clipboard_direction in ("to-server", "both"),
-                 "can-receive"          : self.client_clipboard_direction in ("to-client", "both"),
-                 "remote-loop-uuids"    : self.server_clipboard_loop_uuids,
-                 }
-        #only allow translation overrides if we have a way of telling the server about them:
-        if self.server_clipboard_enable_selections:
-            kwargs.update({
-                #the local clipboard we want to sync to (with the translated clipboard only):
-                 "clipboard.local"      : self.local_clipboard,
-                 #the remote clipboard we want to we sync to (with the translated clipboard only):
-                 "clipboard.remote"     : self.remote_clipboard
-                 })
-        clipboardlog("setup_clipboard_helper() kwargs=%s", kwargs)
-        def clipboard_send(*parts):
-            clipboardlog("clipboard_send: %s", parts[0])
-            if not self.clipboard_enabled:
-                clipboardlog("clipboard is disabled, not sending clipboard packet")
-                return
-            #handle clipboard compression if needed:
-            from xpra.net.compression import Compressible
-            packet = list(parts)
-            for v in packet:
-                if isinstance(v, Compressible):
-                    register_clipboard_compress_cb(v)
-            self.send_now(*packet)
-        def register_clipboard_compress_cb(compressible):
-            #register the compressor which will fire in protocol.encode:
-            def compress_clipboard():
-                clipboardlog("compress_clipboard() compressing %s", compressible)
-                return self.compressed_wrapper(compressible.datatype, compressible.data)
-            compressible.compress = compress_clipboard
-        def clipboard_progress(local_requests, remote_requests):
-            clipboardlog("clipboard_progress(%s, %s)", local_requests, remote_requests)
-            if local_requests is not None:
-                self.local_clipboard_requests = local_requests
-            if remote_requests is not None:
-                self.remote_clipboard_requests = remote_requests
-            n = self.local_clipboard_requests+self.remote_clipboard_requests
-            self.clipboard_notify(n)
+        from xpra.client.mixins.clipboard import ClipboardClient
+        ch = ClipboardClient.setup_clipboard_helper(self, helperClass)
+        #check for loops after handshake:
         def register_clipboard_toggled(*_args):
             def clipboard_toggled(*_args):
                 #reset tray icon:
@@ -1331,7 +1230,10 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 return False
             self.timeout_add(5*1000, loop_disabled_notify)
         self.after_handshake(register_clipboard_toggled)
-        return helperClass(clipboard_send, clipboard_progress, **kwargs)
+        if self.server_clipboard:
+            #from now on, we will send a message to the server whenever the clipboard flag changes:
+            self.connect("clipboard-toggled", self.clipboard_toggled)
+        return ch
 
     def cancel_clipboard_notification_timer(self):
         cnt = self.clipboard_notification_timer
