@@ -1,21 +1,21 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.log import Logger
-avsynclog = Logger("av-sync")
-log = Logger("client", "sound")
-
 from xpra.platform.paths import get_icon_filename
 from xpra.scripts.parsing import sound_option
 from xpra.net.compression import Compressed
 from xpra.os_util import get_machine_id, get_user_uuid, bytestostr, OSX, POSIX
 from xpra.util import envint, typedict, csv, updict
 from xpra.client.mixins.stub_client_mixin import StubClientMixin
+from xpra.log import Logger
 
+avsynclog = Logger("av-sync")
+log = Logger("client", "sound")
 
 AV_SYNC_DELTA = envint("XPRA_AV_SYNC_DELTA")
+DEFAULT_AV_SYNC_DELAY = envint("XPRA_DEFAULT_AV_SYNC_DELAY", 150)
 
 
 """
@@ -39,9 +39,10 @@ class AudioClient(StubClientMixin):
         #sound state:
         self.on_sink_ready = None
         self.sound_sink = None
-        self.min_sound_sequence = 0
+        self.sound_sink_sequence = 0
         self.server_sound_eos_sequence = False
         self.sound_source = None
+        self.sound_source_sequence = 0
         self.sound_in_bytecount = 0
         self.sound_out_bytecount = 0
         self.server_av_sync = False
@@ -54,8 +55,10 @@ class AudioClient(StubClientMixin):
         self.server_sound_bundle_metadata = False
         self.server_ogg_latency_fix = False
         self.queue_used_sent = None
+        #duplicated from ServerInfo mixin:
+        self._remote_machine_id = None
 
-    def init(self, opts, _extra_args=[]):
+    def init(self, opts, _extra_args=()):
         self.av_sync = opts.av_sync
         self.sound_properties = typedict()
         self.speaker_allowed = sound_option(opts.speaker) in ("on", "off")
@@ -89,7 +92,8 @@ class AudioClient(StubClientMixin):
                         assert val, "%s not found in sound properties" % k
                         return ".".join(val[:3])
                     bits = self.sound_properties.intget("python.bits", 32)
-                    log.info("GStreamer version %s for Python %s %s-bit", vinfo("gst.version"), vinfo("python.version"), bits)
+                    log.info("GStreamer version %s for Python %s %s-bit",
+                             vinfo("gst.version"), vinfo("python.version"), bits)
                 except Exception as e:
                     log("failed to query sound", exc_info=True)
                     log.error("Error: failed to query sound subsystem:")
@@ -106,9 +110,9 @@ class AudioClient(StubClientMixin):
             self.microphone_allowed = False
         self.speaker_enabled = self.speaker_allowed and sound_option(opts.speaker)=="on"
         self.microphone_enabled = self.microphone_allowed and opts.microphone.lower()=="on"
-        self.av_sync = opts.av_sync
         log("speaker: codecs=%s, allowed=%s, enabled=%s", encoders, self.speaker_allowed, csv(self.speaker_codecs))
-        log("microphone: codecs=%s, allowed=%s, enabled=%s, default device=%s", decoders, self.microphone_allowed, csv(self.microphone_codecs), self.microphone_device)
+        log("microphone: codecs=%s, allowed=%s, enabled=%s, default device=%s",
+            decoders, self.microphone_allowed, csv(self.microphone_codecs), self.microphone_device)
         log("av-sync=%s", self.av_sync)
         if POSIX and not OSX:
             try:
@@ -135,6 +139,13 @@ class AudioClient(StubClientMixin):
         if self.sound_sink:
             self.stop_receiving_sound()
 
+
+    def get_caps(self):
+        d = {}
+        updict(d, "av-sync", self.get_avsync_capabilities())
+        updict(d, "sound", self.get_audio_capabilities())
+        return d
+
     def get_audio_capabilities(self):
         if not self.sound_properties:
             return {}
@@ -151,8 +162,11 @@ class AudioClient(StubClientMixin):
         log("audio capabilities: %s", caps)
         return caps
 
-    def get_caps(self):
-        return updict({}, "sound", self.get_audio_capabilities())
+    def get_avsync_capabilities(self):
+        return {
+            ""              : self.av_sync,
+            "delay.default" : max(0, DEFAULT_AV_SYNC_DELAY + AV_SYNC_DELTA),
+            }
 
 
     def setup_connection(self, _conn):
@@ -164,11 +178,8 @@ class AudioClient(StubClientMixin):
         avsynclog("av-sync: server=%s, client=%s", self.server_av_sync, self.av_sync)
         self.server_pulseaudio_id = c.strget("sound.pulseaudio.id")
         self.server_pulseaudio_server = c.strget("sound.pulseaudio.server")
-        try:
-            self.server_sound_decoders = c.strlistget("sound.decoders", [])
-            self.server_sound_encoders = c.strlistget("sound.encoders", [])
-        except:
-            log("Error: cannot parse server sound codec data", exc_info=True)
+        self.server_sound_decoders = c.strlistget("sound.decoders", ())
+        self.server_sound_encoders = c.strlistget("sound.encoders", ())
         self.server_sound_receive = c.boolget("sound.receive")
         self.server_sound_send = c.boolget("sound.send")
         self.server_sound_bundle_metadata = c.boolget("sound.bundle-metadata")
@@ -206,7 +217,7 @@ class AudioClient(StubClientMixin):
 
 
     def get_matching_codecs(self, local_codecs, server_codecs):
-        matching_codecs = [x for x in local_codecs if x in server_codecs]
+        matching_codecs = tuple(x for x in local_codecs if x in server_codecs)
         log("get_matching_codecs(%s, %s)=%s", local_codecs, server_codecs, matching_codecs)
         return matching_codecs
 
@@ -242,7 +253,7 @@ class AudioClient(StubClientMixin):
             log.warn(" %s", x)
         return False
 
-    def no_matching_codec_error(self, forwarding="speaker", server_codecs=[], client_codecs=[]):
+    def no_matching_codec_error(self, forwarding="speaker", server_codecs=(), client_codecs=()):
         summary = "Failed to start %s forwarding" % forwarding
         body = "No matching codecs between client and server"
         self.may_notify_audio(summary, body)
@@ -282,16 +293,19 @@ class AudioClient(StubClientMixin):
         #find the matching codecs:
         matching_codecs = self.get_matching_codecs(self.microphone_codecs, self.server_sound_decoders)
         log("start_sound_source(%s) matching codecs: %s", device, csv(matching_codecs))
-        if len(matching_codecs)==0:
+        if not matching_codecs:
             self.no_matching_codec_error("microphone", self.server_sound_decoders, self.microphone_codecs)
             return False
         try:
             from xpra.sound.wrapper import start_sending_sound
             plugins = self.sound_properties.get("plugins")
-            ss = start_sending_sound(plugins, self.sound_source_plugin, device or self.microphone_device, None, 1.0, False, matching_codecs, self.server_pulseaudio_server, self.server_pulseaudio_id)
+            ss = start_sending_sound(plugins, self.sound_source_plugin, device or self.microphone_device,
+                                     None, 1.0, False, matching_codecs,
+                                     self.server_pulseaudio_server, self.server_pulseaudio_id)
             if not ss:
                 return False
             self.sound_source = ss
+            ss.sequence = self.sound_source_sequence
             ss.connect("new-buffer", self.new_sound_buffer)
             ss.connect("state-changed", sound_source_state_changed)
             ss.connect("new-stream", self.new_stream)
@@ -330,7 +344,11 @@ class AudioClient(StubClientMixin):
             log.warn("Warning: cannot stop sound source which has not been started")
             return
         #tell the server to stop:
-        self.send("sound-data", ss.codec or "", "", {"end-of-stream" : True})
+        self.send("sound-data", ss.codec or "", "", {
+            "end-of-stream" : True,
+            "sequence"      : ss.sequence,
+            })
+        self.sound_source_sequence += 1
         ss.cleanup()
 
     def start_receiving_sound(self):
@@ -342,7 +360,7 @@ class AudioClient(StubClientMixin):
                 log("start_receiving_sound: we already have a sound sink")
                 enabled = True
                 return
-            elif not self.server_sound_send:
+            if not self.server_sound_send:
                 log.error("Error receiving sound: support not enabled on the server")
                 return
             if not self.audio_loop_check("speaker"):
@@ -350,7 +368,7 @@ class AudioClient(StubClientMixin):
             #choose a codec:
             matching_codecs = self.get_matching_codecs(self.speaker_codecs, self.server_sound_encoders)
             log("start_receiving_sound() matching codecs: %s", csv(matching_codecs))
-            if len(matching_codecs)==0:
+            if not matching_codecs:
                 self.no_matching_codec_error("speaker", self.server_sound_encoders, self.speaker_codecs)
                 return
             codec = matching_codecs[0]
@@ -377,12 +395,12 @@ class AudioClient(StubClientMixin):
         if self.speaker_enabled:
             self.speaker_enabled = False
             self.emit("speaker-changed")
-        if tell_server:
-            self.send("sound-control", "stop", self.min_sound_sequence)
-        self.min_sound_sequence += 1
-        self.send("sound-control", "new-sequence", self.min_sound_sequence)
-        if ss is None:
+        if not ss:
             return
+        if tell_server and ss.sequence==self.sound_sink_sequence:
+            self.send("sound-control", "stop", self.sound_sink_sequence)
+        self.sound_sink_sequence += 1
+        self.send("sound-control", "new-sequence", self.sound_sink_sequence)
         self.sound_sink = None
         log("stop_receiving_sound(%s) calling %s", tell_server, ss.cleanup)
         ss.cleanup()
@@ -451,6 +469,7 @@ class AudioClient(StubClientMixin):
             if not ss:
                 return False
             self.sound_sink = ss
+            ss.sequence = self.sound_sink_sequence
             ss.connect("state-changed", self.sound_sink_state_changed)
             ss.connect("error", self.sound_sink_error)
             ss.connect("exit", self.sound_sink_exit)
@@ -464,25 +483,28 @@ class AudioClient(StubClientMixin):
             self.sound_sink_error(self.sound_sink, e)
             return False
 
-    def new_sound_buffer(self, sound_source, data, metadata, packet_metadata=[]):
-        log("new_sound_buffer(%s, %s, %s, %s)", sound_source, len(data or []), metadata, packet_metadata)
-        if not self.sound_source:
+    def new_sound_buffer(self, sound_source, data, metadata, packet_metadata=()):
+        log("new_sound_buffer(%s, %s, %s, %s)", sound_source, len(data or ()), metadata, packet_metadata)
+        if sound_source.sequence<self.sound_source_sequence:
+            log("sound buffer dropped: old sequence number: %s (current is %s)",
+                sound_source.sequence, self.sound_source_sequence)
             return
         self.sound_out_bytecount += len(data)
         for x in packet_metadata:
             self.sound_out_bytecount += len(x)
+        metadata["sequence"] = sound_source.sequence
         if packet_metadata:
             if not self.server_sound_bundle_metadata:
                 #server does not support bundling, send packet metadata as individual packets before the main packet:
                 for x in packet_metadata:
-                    self.send_sound_data(sound_source, x)
+                    self.send_sound_data(sound_source, x, metadata)
                 packet_metadata = ()
             else:
                 #the packet metadata is compressed already:
                 packet_metadata = Compressed("packet metadata", packet_metadata, can_inline=True)
         self.send_sound_data(sound_source, data, metadata, packet_metadata)
 
-    def send_sound_data(self, sound_source, data, metadata={}, packet_metadata=()):
+    def send_sound_data(self, sound_source, data, metadata, packet_metadata=None):
         codec = sound_source.codec
         packet_data = [codec, Compressed(codec, data), metadata]
         if packet_metadata:
@@ -504,8 +526,8 @@ class AudioClient(StubClientMixin):
             self.sound_in_bytecount += len(data)
         #verify sequence number if present:
         seq = metadata.intget("sequence", -1)
-        if self.min_sound_sequence>0 and seq>=0 and seq<self.min_sound_sequence:
-            log("ignoring sound data with old sequence number %s (now on %s)", seq, self.min_sound_sequence)
+        if self.sound_sink_sequence>0 and 0<=seq<self.sound_sink_sequence:
+            log("ignoring sound data with old sequence number %s (now on %s)", seq, self.sound_sink_sequence)
             return
 
         if not self.speaker_enabled:
@@ -539,7 +561,7 @@ class AudioClient(StubClientMixin):
             log.error(" stream tried to switch from %s to %s", ss.codec, codec)
             self.stop_receiving_sound()
             return
-        elif ss.get_state()=="stopped":
+        if ss.get_state()=="stopped":
             log("sound data received, sound sink is stopped - telling server to stop")
             self.stop_receiving_sound()
             return
@@ -553,7 +575,7 @@ class AudioClient(StubClientMixin):
                     ss.add_data(x)
                 packet_metadata = ()
         #(some packets (ie: sos, eos) only contain metadata)
-        if len(data)>0 or packet_metadata:
+        if data or packet_metadata:
             ss.add_data(data, metadata, packet_metadata)
         if self.av_sync and self.server_av_sync:
             info = ss.get_info()
@@ -561,9 +583,11 @@ class AudioClient(StubClientMixin):
             if queue_used is None:
                 return
             delta = (self.queue_used_sent or 0)-queue_used
-            #avsynclog("server sound sync: queue info=%s, last sent=%s, delta=%s", dict((k,v) for (k,v) in info.items() if k.startswith("queue")), self.queue_used_sent, delta)
+            #avsynclog("server sound sync: queue info=%s, last sent=%s, delta=%s",
+            #    dict((k,v) for (k,v) in info.items() if k.startswith("queue")), self.queue_used_sent, delta)
             if self.queue_used_sent is None or abs(delta)>=80:
-                avsynclog("server sound sync: sending updated queue.used=%i (was %s)", queue_used, (self.queue_used_sent or "unset"))
+                avsynclog("server sound sync: sending updated queue.used=%i (was %s)",
+                          queue_used, (self.queue_used_sent or "unset"))
                 self.queue_used_sent = queue_used
                 v = queue_used + self.av_sync_delta
                 if self.av_sync_delta:
@@ -574,6 +598,4 @@ class AudioClient(StubClientMixin):
     def init_authenticated_packet_handlers(self):
         log("init_authenticated_packet_handlers()")
         #these handlers can run directly from the network thread:
-        self.set_packet_handlers(self._packet_handlers, {
-            "sound-data":           self._process_sound_data,
-            })
+        self.add_packet_handler("sound-data", self._process_sound_data, False)
