@@ -1,17 +1,12 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
 
-from xpra.log import Logger
-log = Logger("client")
-keylog = Logger("client", "keyboard")
-
-
 from xpra.client.client_base import XpraClientBase
 from xpra.client.keyboard_helper import KeyboardHelper
 from xpra.platform import set_name
@@ -21,12 +16,17 @@ from xpra.version_util import full_version_str
 from xpra.net import compression, packet_encoding
 from xpra.child_reaper import reaper_cleanup
 from xpra.os_util import platform_name, bytestostr, strtobytes, BITS
-from xpra.util import std, envbool, typedict, updict, repr_ellipsized, XPRA_AUDIO_NOTIFICATION_ID
+from xpra.util import (
+    std, envbool, envint, typedict, updict, repr_ellipsized,
+    XPRA_AUDIO_NOTIFICATION_ID, XPRA_DISCONNECT_NOTIFICATION_ID,
+    )
+from xpra.exit_codes import EXIT_CONNECTION_FAILED, EXIT_OK, EXIT_CONNECTION_LOST
 from xpra.version_util import get_version_info_full, get_platform_info
+from xpra.client import mixin_features
+from xpra.log import Logger
 
 
 CLIENT_BASES = [XpraClientBase]
-from xpra.client import mixin_features
 if mixin_features.display:
     from xpra.client.mixins.display import DisplayClient
     CLIENT_BASES.append(DisplayClient)
@@ -66,9 +66,12 @@ if mixin_features.tray:
 
 CLIENT_BASES = tuple(CLIENT_BASES)
 ClientBaseClass = type('ClientBaseClass', CLIENT_BASES, {})
+
+log = Logger("client")
+keylog = Logger("client", "keyboard")
 log("UIXpraClient%s: %s", ClientBaseClass, CLIENT_BASES)
 
-
+NOTIFICATION_EXIT_DELAY = envint("XPRA_NOTIFICATION_EXIT_DELAY", 2)
 MOUSE_DELAY_AUTO = envbool("XPRA_MOUSE_DELAY_AUTO", True)
 
 
@@ -94,14 +97,15 @@ class UIXpraClient(ClientBaseClass):
         #same for tray:
         self.tray = None
         for c in CLIENT_BASES:
+            log("calling %s.__init__()", c)
             c.__init__(self)
         try:
             pinfo = get_platform_info()
             osinfo = "%s" % platform_name(sys.platform, pinfo.get("linux_distribution") or pinfo.get("sysrelease", ""))
             log.info(" running on %s", osinfo)
-        except:
+        except Exception:
             log("platform name error:", exc_info=True)
-        wm = get_wm_name()
+        wm = get_wm_name()      #pylint: disable=assignment-from-none
         if wm:
             log.info(" window manager is '%s'", wm)
 
@@ -118,6 +122,7 @@ class UIXpraClient(ClientBaseClass):
         self.readonly = False
         self.xsettings_enabled = False
         self.server_start_new_commands = False
+        self.server_xdg_menu = None
         self.start_new_commands  = []
         self.start_child_new_commands  = []
 
@@ -151,7 +156,7 @@ class UIXpraClient(ClientBaseClass):
         self._on_server_setting_changed = {}
 
 
-    def init(self, opts, extra_args=[]):
+    def init(self, opts, extra_args=()):
         """ initialize variables from configuration """
         for c in CLIENT_BASES:
             log("init: %s", c)
@@ -174,9 +179,14 @@ class UIXpraClient(ClientBaseClass):
                 if str(v).lower()=="auto":
                     return None
                 return v
-            overrides = [noauto(getattr(opts, "keyboard_%s" % x)) for x in ("layout", "layouts", "variant", "variants", "options")]
+            overrides = [noauto(getattr(opts, "keyboard_%s" % x)) for x in (
+                "layout", "layouts", "variant", "variants", "options",
+                )]
             try:
-                self.keyboard_helper = self.keyboard_helper_class(self.send, opts.keyboard_sync, opts.shortcut_modifiers, opts.key_shortcut, opts.keyboard_raw, *overrides)
+                self.keyboard_helper = self.keyboard_helper_class(self.send, opts.keyboard_sync,
+                                                                  opts.shortcut_modifiers,
+                                                                  opts.key_shortcut,
+                                                                  opts.keyboard_raw, *overrides)
             except ImportError as e:
                 keylog("error instantiating %s", self.keyboard_helper_class, exc_info=True)
                 keylog.warn("Warning: no keyboard support, %s", e)
@@ -185,14 +195,14 @@ class UIXpraClient(ClientBaseClass):
             self.init_opengl(opts.opengl)
 
         if ClientExtras is not None:
-            self.client_extras = ClientExtras(self, opts)
+            self.client_extras = ClientExtras(self, opts)   #pylint: disable=not-callable
 
         if opts.start or opts.start_child:
             from xpra.scripts.main import strip_defaults_start_child
             from xpra.scripts.config import make_defaults_struct
             defaults = make_defaults_struct()
-            self.start_new_commands  = strip_defaults_start_child(opts.start, defaults.start)
-            self.start_child_new_commands  = strip_defaults_start_child(opts.start_child, defaults.start_child)
+            self.start_new_commands  = strip_defaults_start_child(opts.start, defaults.start)   #pylint: disable=no-member
+            self.start_child_new_commands  = strip_defaults_start_child(opts.start_child, defaults.start_child) #pylint: disable=no-member
 
         if MOUSE_DELAY_AUTO:
             try:
@@ -216,8 +226,8 @@ class UIXpraClient(ClientBaseClass):
         self.send_hello()
 
 
-    def quit(self, exit_code=0):
-        raise Exception("override me!")
+    def quit(self, _exit_code=0):
+        raise NotImplementedError()
 
     def cleanup(self):
         log("UIXpraClient.cleanup()")
@@ -229,7 +239,7 @@ class UIXpraClient(ClientBaseClass):
             log("UIXpraClient.cleanup() calling %s.cleanup()", type(x))
             try:
                 x.cleanup()
-            except:
+            except Exception:
                 log.error("error on %s cleanup", type(x), exc_info=True)
         #the protocol has been closed, it is now safe to close all the windows:
         #(cleaner and needed when we run embedded in the client launcher)
@@ -274,7 +284,8 @@ class UIXpraClient(ClientBaseClass):
 
 
     def send_start_new_commands(self):
-        log("send_start_new_commands() start_new_commands=%s, start_child_new_commands=%s", self.start_new_commands, self.start_child_new_commands)
+        log("send_start_new_commands() start_new_commands=%s, start_child_new_commands=%s",
+            self.start_new_commands, self.start_child_new_commands)
         import shlex
         for cmd in self.start_new_commands:
             cmd_parts = shlex.split(cmd)
@@ -285,12 +296,42 @@ class UIXpraClient(ClientBaseClass):
 
     def send_start_command(self, name, command, ignore, sharing=True):
         log("send_start_command(%s, %s, %s, %s)", name, command, ignore, sharing)
+        assert name is not None and command is not None and ignore is not None
         self.send("start-command", name, command, ignore, sharing)
 
     def get_version_info(self):
         return get_version_info_full()
 
 
+    ######################################################################
+    # trigger notifications on disconnection,
+    # and wait before actually exiting so the notification has a chance of being seen
+    def server_disconnect_warning(self, reason, *info):
+        if self.exit_code is None:
+            body = "\n".join(info)
+            if self.server_capabilities:
+                title = "Xpra Session Disconnected: %s" % reason
+                self.exit_code = EXIT_CONNECTION_LOST
+            else:
+                title = "Connection Failed: %s" % reason
+                self.exit_code = EXIT_CONNECTION_FAILED
+            self.may_notify(XPRA_DISCONNECT_NOTIFICATION_ID,
+                            title, body, icon_name="disconnected")
+            #show text notification then quit:
+            delay = NOTIFICATION_EXIT_DELAY*mixin_features.notifications
+            self.timeout_add(delay*1000, XpraClientBase.server_disconnect_warning, self, reason, *info)
+        self.cleanup()
+
+    def server_disconnect(self, reason, *info):
+        body = "\n".join(info)
+        self.may_notify(XPRA_DISCONNECT_NOTIFICATION_ID,
+                        "Xpra Session Disconnected: %s" % reason, body, icon_name="disconnected")
+        self.exit_code = EXIT_OK
+        delay = NOTIFICATION_EXIT_DELAY*mixin_features.notifications
+        self.timeout_add(delay*1000, XpraClientBase.server_disconnect, self, reason, *info)
+        self.cleanup()
+
+
     ######################################################################
     # hello:
     def make_hello(self):
@@ -307,6 +348,7 @@ class UIXpraClient(ClientBaseClass):
                 pass
         for x in (#generic feature flags:
             "notify-startup-complete", "wants_events", "setting-change",
+            "xdg-menu-update",
             ):
             caps[x] = True
         #FIXME: the messy bits without proper namespace:
@@ -357,6 +399,8 @@ class UIXpraClient(ClientBaseClass):
         self.server_toggle_keyboard_sync = self.server_keyboard and c.boolget("toggle_keyboard_sync", True)
         self.server_pointer = c.boolget("pointer", True)
         self.server_start_new_commands = c.boolget("start-new-commands")
+        if self.server_start_new_commands:
+            self.server_xdg_menu = c.dictget("xdg-menu", None)
         if self.start_new_commands or self.start_child_new_commands:
             if self.server_start_new_commands:
                 self.after_handshake(self.send_start_new_commands)
@@ -371,11 +415,12 @@ class UIXpraClient(ClientBaseClass):
             self.readonly = True
         if not self.server_keyboard and self.keyboard_helper:
             #swallow packets:
-            def nosend(*args):
+            def nosend(*_args):
                 pass
             self.keyboard_helper.send = nosend
 
-        i = platform_name(self._remote_platform, c.strlistget("platform.linux_distribution") or c.strget("platform.release", ""))
+        i = platform_name(self._remote_platform,
+                          c.strlistget("platform.linux_distribution") or c.strget("platform.release", ""))
         r = self._remote_version
         if self._remote_revision:
             r += "-r%s" % self._remote_revision
@@ -390,8 +435,11 @@ class UIXpraClient(ClientBaseClass):
             proxy_release = c.strget("proxy.platform.release")
             proxy_version = c.strget("proxy.version")
             proxy_version = c.strget("proxy.build.version", proxy_version)
-            proxy_distro = c.strget("linux_distribution")
-            msg = "via: %s proxy version %s" % (platform_name(proxy_platform, proxy_distro or proxy_release), std(proxy_version or "unknown"))
+            proxy_distro = c.strget("proxy.linux_distribution")
+            msg = "via: %s proxy version %s" % (
+                platform_name(proxy_platform, proxy_distro or proxy_release),
+                std(proxy_version or "unknown")
+                )
             if proxy_hostname:
                 msg += " on '%s'" % std(proxy_hostname)
             log.info(msg)
@@ -404,7 +452,7 @@ class UIXpraClient(ClientBaseClass):
         #keyboard:
         c = self.server_capabilities
         if self.keyboard_helper:
-            modifier_keycodes = c.dictget("modifier_keycodes")
+            modifier_keycodes = c.dictget("modifier_keycodes", {})
             if modifier_keycodes:
                 self.keyboard_helper.set_modifier_mappings(modifier_keycodes)
         self.key_repeat_delay, self.key_repeat_interval = c.intpair("key_repeat", (-1,-1))
@@ -426,7 +474,7 @@ class UIXpraClient(ClientBaseClass):
         for cb, args in oh:
             try:
                 cb(*args)
-            except:
+            except Exception:
                 log.error("Error processing handshake callback %s", cb, exc_info=True)
 
     def after_handshake(self, cb, *args):
@@ -451,17 +499,25 @@ class UIXpraClient(ClientBaseClass):
         setting = bytestostr(setting)
         #convert "hello" / "setting" variable names to client variables:
         if setting in (
+            "clipboard-limits",
+            ):
+            pass
+        elif setting in (
             "bell", "randr", "cursors", "notifications", "dbus-proxy", "clipboard",
             "clipboard-direction", "session_name",
             "sharing", "sharing-toggle", "lock", "lock-toggle",
             "start-new-commands", "client-shutdown", "webcam",
-            "bandwidth-limit",
+            "bandwidth-limit", "clipboard-limits",
+            "xdg-menu",
             ):
             setattr(self, "server_%s" % setting.replace("-", "_"), value)
         else:
-            log.info("unknown server setting changed: %s=%s", setting, value)
+            log.info("unknown server setting changed: %s=%s", setting, repr_ellipsized(bytestostr(value)))
             return
-        log.info("server setting changed: %s=%s", setting, value)
+        log("_process_setting_change: %s=%s", setting, value)
+        #xdg-menu is too big to log
+        if setting not in ("xdg-menu", ):
+            log.info("server setting changed: %s=%s", setting, repr_ellipsized(str(value)))
         self.server_setting_changed(setting, value)
 
     def server_setting_changed(self, setting, value):
@@ -622,7 +678,7 @@ class UIXpraClient(ClientBaseClass):
             #force it fullscreen:
             try:
                 del metadata["size-constraints"]
-            except:
+            except KeyError:
                 pass
             metadata["fullscreen"] = True
             #FIXME: try to figure out the monitors we go fullscreen on for X11:
@@ -664,15 +720,13 @@ class UIXpraClient(ClientBaseClass):
         for c in CLIENT_BASES:
             c.init_authenticated_packet_handlers(self)
         #run from the UI thread:
-        self.set_packet_handlers(self._ui_packet_handlers, {
+        self.add_packet_handlers({
             "startup-complete":     self._process_startup_complete,
             "setting-change":       self._process_setting_change,
             "control" :             self._process_control,
             })
         #run directly from the network thread:
-        self.set_packet_handlers(self._packet_handlers, {
-            "server-event":         self._process_server_event,
-            })
+        self.add_packet_handler("server-event", self._process_server_event, False)
 
 
     def process_packet(self, proto, packet):
