@@ -1,30 +1,57 @@
 # This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import hashlib
-from xpra.log import Logger
-log = Logger("paint")
-deltalog = Logger("delta")
-
 from threading import Lock
+
 from xpra.net.mmap_pipe import mmap_read
 from xpra.net import compression
-from xpra.util import typedict, csv, envint, envbool, repr_ellipsized
+from xpra.util import typedict, csv, envint, envbool, repr_ellipsized, first_time
 from xpra.codecs.loader import get_codec
 from xpra.codecs.video_helper import getVideoHelper
-from xpra.os_util import BytesIOClass, bytestostr, _buffer
+from xpra.os_util import bytestostr, _buffer
 try:
     from xpra.codecs.xor.cyxor import xor_str   #@UnresolvedImport
 except ImportError:
     from xpra.util import xor as xor_str
+from xpra.log import Logger
+
+#X11 constants we use for gravity:
+NorthWestGravity = 1
+NorthGravity     = 2
+NorthEastGravity = 3
+WestGravity      = 4
+CenterGravity    = 5
+EastGravity      = 6
+SouthWestGravity = 7
+SouthGravity     = 8
+SouthEastGravity = 9
+StaticGravity    = 10
+
+GRAVITY_STR = {
+    NorthWestGravity : "NorthWest",
+    NorthGravity     : "North",
+    NorthEastGravity : "NorthEast",
+    WestGravity      : "West",
+    CenterGravity    : "Center",
+    EastGravity      : "East",
+    SouthWestGravity : "SouthWest",
+    SouthGravity     : "South",
+    SouthEastGravity : "SouthEast",
+    StaticGravity    : "South",
+    }
+
+log = Logger("paint")
+deltalog = Logger("delta")
 
 DELTA_BUCKETS = envint("XPRA_DELTA_BUCKETS", 5)
 INTEGRITY_HASH = envbool("XPRA_INTEGRITY_HASH", False)
 PAINT_BOX = envint("XPRA_PAINT_BOX", 0) or envint("XPRA_OPENGL_PAINT_BOX", 0)
 WEBP_PILLOW = envbool("XPRA_WEBP_PILLOW", False)
+SCROLL_ENCODING = envbool("XPRA_SCROLL_ENCODING", True)
 
 
 #ie:
@@ -50,7 +77,7 @@ def load_video_decoders():
             specs = vh.get_decoder_specs(encoding)
             for colorspace, decoders in specs.items():
                 log("%-5s decoders for %7s: %s", encoding, colorspace, csv([d.get_type() for _,d in decoders]))
-                assert len(decoders)>0
+                assert decoders
                 #use the first one:
                 _, decoder_module = decoders[0]
                 VIDEO_DECODERS[encoding] = decoder_module
@@ -62,9 +89,7 @@ def fire_paint_callbacks(callbacks, success=True, message=""):
     for x in callbacks:
         try:
             x(success, message)
-        except KeyboardInterrupt:
-            raise
-        except:
+        except Exception:
             log.error("error calling %s(%s)", x, success, exc_info=True)
 
 
@@ -73,6 +98,8 @@ Generic superclass for all Backing code,
 see CairoBackingBase and GTK2WindowBacking subclasses for actual implementations
 """
 class WindowBackingBase(object):
+    RGB_MODES = ()
+
     def __init__(self, wid, window_alpha):
         load_csc_options()
         load_video_decoders()
@@ -80,6 +107,7 @@ class WindowBackingBase(object):
         self.size = 0, 0
         self.render_size = 0, 0
         self.offsets = 0, 0, 0, 0       #top,left,bottom,right
+        self.gravity = 0
         self._alpha_enabled = window_alpha
         self._backing = None
         self._delta_pixel_data = [None for _ in range(DELTA_BUCKETS)]
@@ -92,14 +120,16 @@ class WindowBackingBase(object):
         self.pointer_overlay = None
         self.cursor_data = None
         self.default_cursor_data = None
-        PIL = get_codec("dec_pillow")
-        if PIL:
-            self._PIL_encodings = PIL.get_encodings()
+        self.jpeg_decoder = None
+        self.webp_decoder = None
+        self.pil_decoder = get_codec("dec_pillow")
+        if self.pil_decoder:
+            self._PIL_encodings = self.pil_decoder.get_encodings()
+        self.jpeg_decoder = get_codec("dec_jpeg")
+        self.webp_decoder = get_codec("dec_webp")
         self.draw_needs_refresh = True
         self.mmap = None
         self.mmap_enabled = False
-        self.jpeg_decoder = get_codec("dec_jpeg")
-        self.webp_decoder = get_codec("dec_webp")
 
     def idle_add(self, *_args, **_kwargs):
         raise NotImplementedError()
@@ -108,6 +138,123 @@ class WindowBackingBase(object):
         self.mmap = mmap_area
         self.mmap_enabled = True
 
+    def gravity_copy_coords(self, oldw, oldh, bw, bh):
+        sx = sy = dx = dy = 0
+        def center_y():
+            if bh>=oldh:
+                #take the whole source, paste it in the middle
+                return 0, (bh-oldh)//2
+            #skip the edges of the source, paste all of it
+            return (oldh-bh)//2, 0
+        def center_x():
+            if bw>=oldw:
+                return 0, (bw-oldw)//2
+            return (oldw-bw)//2, 0
+        def east_x():
+            if bw>=oldw:
+                return 0, bw-oldw
+            return oldw-bw, 0
+        def west_x():
+            return 0, 0
+        def north_y():
+            return 0, 0
+        def south_y():
+            if bh>=oldh:
+                return 0, bh-oldh
+            return oldh-bh, 0
+        g = self.gravity
+        if not g or g==NorthWestGravity:
+            #undefined (or 0), use NW
+            sx, dx = west_x()
+            sy, dy = north_y()
+        elif g==NorthGravity:
+            sx, dx = center_x()
+            sy, dy = north_y()
+        elif g==NorthEastGravity:
+            sx, dx = east_x()
+            sy, dy = north_y()
+        elif g==WestGravity:
+            sx, dx = west_x()
+            sy, dy = center_y()
+        elif g==CenterGravity:
+            sx, dx = center_x()
+            sy, dy = center_y()
+        elif g==EastGravity:
+            sx, dx = east_x()
+            sy, dy = center_y()
+        elif g==SouthWestGravity:
+            sx, dx = west_x()
+            sy, dy = south_y()
+        elif g==SouthGravity:
+            sx, dx = center_x()
+            sy, dy = south_y()
+        elif g==SouthEastGravity:
+            sx, dx = east_x()
+            sy, dy = south_y()
+        elif g==StaticGravity:
+            if first_time("StaticGravity-%i" % self.wid):
+                log.warn("Warning: static gravity is not handled")
+        w = min(bw, oldw)
+        h = min(bh, oldh)
+        return sx, sy, dx, dy, w, h
+
+    def gravity_adjust(self, x, y, options):
+        #if the window size has changed,
+        #adjust the coordinates honouring the window gravity:
+        window_size = options.intlistget("window-size", None)
+        g = self.gravity
+        log("gravity_adjust%s window_size=%s, size=%s, gravity=%s",
+            (x, y, options), window_size, self.size, GRAVITY_STR.get(g, "unknown"))
+        if not window_size:
+            return x, y
+        window_size = tuple(window_size)
+        if window_size==self.size:
+            return x, y
+        if g==0 or self.gravity==NorthWestGravity:
+            return x, y
+        oldw, oldh = window_size
+        bw, bh = self.size
+        def center_y():
+            if bh>=oldh:
+                return y + (bh-oldh)//2
+            return y - (oldh-bh)//2
+        def center_x():
+            if bw>=oldw:
+                return x + (bw-oldw)//2
+            return x - (oldw-bw)//2
+        def east_x():
+            if bw>=oldw:
+                return x + (bw-oldw)
+            return x - (oldw-bw)
+        def west_x():
+            return x
+        def north_y():
+            return y
+        def south_y():
+            if bh>=oldh:
+                return y + (bh-oldh)
+            return y - (oldh-bh)
+        if g==NorthGravity:
+            return center_x(), north_y()
+        if g==NorthEastGravity:
+            return east_x(), north_y()
+        if g==WestGravity:
+            return west_x(), center_y()
+        if g==CenterGravity:
+            return center_x(), center_y()
+        if g==EastGravity:
+            return east_x(), center_y()
+        if g==SouthWestGravity:
+            return west_x(), south_y()
+        if g==SouthGravity:
+            return center_x(), south_y()
+        if g==SouthEastGravity:
+            return east_x(), south_y()
+        #if self.gravity==StaticGravity:
+        #    pass
+        return x, y
+
+
     def close(self):
         self._backing = None
         log("%s.close() video_decoder=%s", self, self._video_decoder)
@@ -146,6 +293,7 @@ class WindowBackingBase(object):
                  "encodings.rgb_formats"    : self.RGB_MODES,
                  "encoding.transparency"    : self._alpha_enabled,
                  "encoding.full_csc_modes"  : self._get_full_csc_modes(self.RGB_MODES),
+                 "encoding.send-window-size" : True,
                  }
 
     def _get_full_csc_modes(self, rgb_modes):
@@ -167,7 +315,7 @@ class WindowBackingBase(object):
             try:
                 unpremultiply_argb_in_place(img_data)
                 return img_data
-            except:
+            except Exception:
                 log.warn("failed to unpremultiply %s (len=%s)" % (type(img_data), len(img_data)))
         return unpremultiply_argb(img_data)
 
@@ -189,7 +337,8 @@ class WindowBackingBase(object):
                 assert len(comp)==1, "more than one compressor specified: %s" % str(comp)
                 img_data = compression.decompress_by_name(raw_data, algo=comp[0])
         if len(img_data)!=rowstride * height:
-            deltalog.error("invalid img data length: expected %s but got %s (%s: %s)", rowstride * height, len(img_data), type(img_data), repr_ellipsized(img_data))
+            deltalog.error("Error: invalid img data length: expected %s but got %s (%s: %s)",
+                           rowstride * height, len(img_data), type(img_data), repr_ellipsized(img_data))
             raise Exception("expected %s bytes for %sx%s with rowstride=%s but received %s (%s compressed)" %
                                 (rowstride * height, width, height, rowstride, len(img_data), len(raw_data)))
         delta = options.intget(b"delta", -1)
@@ -197,12 +346,13 @@ class WindowBackingBase(object):
         rgb_format = options.strget(b"rgb_format")
         rgb_data = img_data
         if delta>=0:
-            assert bucket>=0 and bucket<DELTA_BUCKETS, "invalid delta bucket number: %s" % bucket
+            assert 0<=bucket<DELTA_BUCKETS, "invalid delta bucket number: %s" % bucket
             if self._delta_pixel_data[bucket] is None:
                 raise Exception("delta region bucket %s references pixmap data we do not have!" % bucket)
             lwidth, lheight, lrgb_format, seq, ldata = self._delta_pixel_data[bucket]
             assert width==lwidth and height==lheight and delta==seq, \
-                "delta bucket %s data does not match: expected %s but got %s" % (bucket, (width, height, delta), (lwidth, lheight, seq))
+                "delta bucket %s data does not match: expected %s but got %s" % (
+                    bucket, (width, height, delta), (lwidth, lheight, seq))
             assert lrgb_format==rgb_format, "delta region uses %s format, was expecting %s" % (rgb_format, lrgb_format)
             deltalog("delta: xoring with bucket %i", bucket)
             rgb_data = xor_str(img_data, ldata)
@@ -225,82 +375,23 @@ class WindowBackingBase(object):
 
 
     def paint_image(self, coding, img_data, x, y, width, height, options, callbacks):
-        """ can be called from any thread """
-        #log("paint_image(%s, %s bytes, %s, %s, %s, %s, %s, %s)", coding, len(img_data), x, y, width, height, options, callbacks)
-        PIL = get_codec("PIL")
-        assert PIL.Image, "PIL.Image not found"
-        buf = BytesIOClass(img_data)
-        img = PIL.Image.open(buf)
-        assert img.mode in ("L", "P", "RGB", "RGBA", "RGBX"), "invalid image mode: %s" % img.mode
-        transparency = options.intget("transparency", -1)
-        if img.mode=="P":
-            if transparency>=0:
-                #this deals with alpha without any extra work
-                img = img.convert("RGBA")
-            else:
-                img = img.convert("RGB")
-        elif img.mode=="L":
-            if transparency>=0:
-                #why do we have to deal with alpha ourselves??
-                def mask_value(a):
-                    if a!=transparency:
-                        return 255
-                    return 0
-                mask = PIL.Image.eval(img, mask_value)
-                mask = mask.convert("L")
-                def nomask_value(a):
-                    if a!=transparency:
-                        return a
-                    return 0
-                img = PIL.Image.eval(img, nomask_value)
-                img = img.convert("RGBA")
-                img.putalpha(mask)
-            else:
-                img = img.convert("RGB")
-
-        raw_data = img.tobytes("raw", img.mode)
-        paint_options = typedict(options)
-        if img.mode=="RGB":
-            #PIL flattens the data to a continuous straightforward RGB format:
-            rowstride = width*3
-            rgb_format = options.strget("rgb_format", "")
-            rgb_format = rgb_format.replace("A", "").replace("X", "")
-            #the webp encoder only takes BGRX input,
-            #so we have to swap things around if it was fed "RGB":
-            if rgb_format=="RGB":
-                rgb_format = "BGR"
-            else:
-                rgb_format = "RGB"
-            img_data = self.process_delta(raw_data, width, height, rowstride, options)
-        elif img.mode in ("RGBA", "RGBX"):
-            rowstride = width*4
-            rgb_format = options.strget("rgb_format", img.mode)
-            if coding=="webp":
-                #the webp encoder only takes BGRX input,
-                #so we have to swap things around if it was fed "RGBA":
-                if rgb_format=="RGBA":
-                    rgb_format = "BGRA"
-                elif rgb_format=="RGBX":
-                    rgb_format = "BGRX"
-                elif rgb_format=="BGRA":
-                    rgb_format = "RGBA"
-                elif rgb_format=="BGRX":
-                    rgb_format = "RGBX"
-                else:
-                    log.warn("Warning: unexpected RGB format '%s'", rgb_format)
-            img_data = self.process_delta(raw_data, width, height, rowstride, options)
-        else:
-            raise Exception("invalid image mode: %s" % img.mode)
-        self.idle_add(self.do_paint_rgb, rgb_format, img_data, x, y, width, height, rowstride, paint_options, callbacks)
-        return False
+        # can be called from any thread
+        rgb_format, raw_data, rowstride = self.pil_decoder.decompress(coding, img_data, options)
+        img_data = self.process_delta(raw_data, width, height, rowstride, options)
+        self.idle_add(self.do_paint_rgb, rgb_format, img_data, x, y, width, height, rowstride, options, callbacks)
 
     def paint_webp(self, img_data, x, y, width, height, options, callbacks):
         if not self.webp_decoder or WEBP_PILLOW:
             #if webp is enabled, then Pillow should be able to take care of it:
-            return self.paint_image("webp", img_data, x, y, width, height, options, callbacks)
+            self.paint_image("webp", img_data, x, y, width, height, options, callbacks)
+            return
         rgb_format = options.strget("rgb_format")
         has_alpha = options.boolget("has_alpha", False)
-        buffer_wrapper, width, height, stride, has_alpha, rgb_format = self.webp_decoder.decompress(img_data, has_alpha, rgb_format, self.RGB_MODES)
+        (
+            buffer_wrapper,
+            width, height, stride, has_alpha,
+            rgb_format,
+            ) = self.webp_decoder.decompress(img_data, has_alpha, rgb_format, self.RGB_MODES)
         def free_buffer(*_args):
             buffer_wrapper.free()
         callbacks.append(free_buffer)
@@ -310,8 +401,8 @@ class WindowBackingBase(object):
         if rgb_format not in self.RGB_MODES:
             from xpra.codecs.rgb_transform import rgb_reformat
             from xpra.codecs.image_wrapper import ImageWrapper
-            img = ImageWrapper(x, y, width, height, data, rgb_format, len(rgb_format)*8, stride, len(rgb_format), ImageWrapper.PACKED, True, None)
-            #log("rgb_reformat(%s, %s, %s) %i bytes", img, self.RGB_MODES, has_alpha and self._alpha_enabled, len(img_data))
+            img = ImageWrapper(x, y, width, height, data, rgb_format,
+                               len(rgb_format)*8, stride, len(rgb_format), ImageWrapper.PACKED, True, None)
             rgb_reformat(img, self.RGB_MODES, has_alpha and self._alpha_enabled)
             rgb_format = img.get_pixel_format()
             data = img.get_pixels()
@@ -326,6 +417,7 @@ class WindowBackingBase(object):
             before calling _do_paint_rgb from the UI thread via idle_add
         """
         rgb_data = self.process_delta(raw_data, width, height, rowstride, options)
+        x, y = self.gravity_adjust(x, y, options)
         self.idle_add(self.do_paint_rgb, rgb_format, rgb_data, x, y, width, height, rowstride, options, callbacks)
 
     def do_paint_rgb(self, rgb_format, img_data, x, y, width, height, rowstride, options, callbacks):
@@ -346,8 +438,6 @@ class WindowBackingBase(object):
             options[b"rgb_format"] = rgb_format
             success = paint_fn(img_data, x, y, width, height, rowstride, options)
             fire_paint_callbacks(callbacks, success)
-        except KeyboardInterrupt:
-            raise
         except Exception as e:
             if not self._backing:
                 fire_paint_callbacks(callbacks, -1, "paint error on closed backing ignored")
@@ -374,10 +464,11 @@ class WindowBackingBase(object):
                        dst_width, dst_height, dst_format_options, speed):
         global CSC_OPTIONS
         in_options = CSC_OPTIONS.get(src_format, {})
-        assert len(in_options)>0, "no csc options for '%s' input in %s" % (src_format, CSC_OPTIONS)
+        assert in_options, "no csc options for '%s' input in %s" % (src_format, CSC_OPTIONS)
         for dst_format in dst_format_options:
             specs = in_options.get(dst_format)
-            log("make_csc%s specs=%s", (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed), specs)
+            log("make_csc%s specs=%s",
+                (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed), specs)
             if not specs:
                 continue
             for spec in specs:
@@ -390,7 +481,9 @@ class WindowBackingBase(object):
                                dst_width, dst_height, dst_format, speed)
                     return csc
                 except Exception as e:
-                    log("make_csc%s", (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed), exc_info=True)
+                    log("make_csc%s",
+                        (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed),
+                        exc_info=True)
                     log.error("Error: failed to create csc instance %s", spec.codec_class)
                     log.error(" for %s to %s: %s", src_format, dst_format, e)
         log.error("Error: no matching CSC module found")
@@ -408,31 +501,32 @@ class WindowBackingBase(object):
                 v = self.validate_csc_size(spec, src_width, src_height, dst_width, dst_height)
                 if v:
                     log.error("       "+v[0], *v[1:])
-        raise Exception("no csc module found for %s(%sx%s) to %s(%sx%s) in %s" % (src_format, src_width, src_height, " or ".join(dst_format_options), dst_width, dst_height, CSC_OPTIONS))
+        raise Exception("no csc module found for %s(%sx%s) to %s(%sx%s) in %s" %
+                        (src_format, src_width, src_height, " or ".join(dst_format_options),
+                         dst_width, dst_height, CSC_OPTIONS))
 
     def validate_csc_size(self, spec, src_width, src_height, dst_width, dst_height):
         if not spec.can_scale and (src_width!=dst_width or src_height!=dst_height):
             return "scaling not suported"
-        elif src_width<spec.min_w:
+        if src_width<spec.min_w:
             return "source width %i is out of range: minimum is %i", src_width, spec.min_w
-        elif src_height<spec.min_h:
+        if src_height<spec.min_h:
             return "source height %i is out of range: minimum is %i", src_height, spec.min_h
-        elif dst_width<spec.min_w:
+        if dst_width<spec.min_w:
             return "target width %i is out of range: minimum is %i", dst_width, spec.min_w
-        elif dst_height<spec.min_h:
+        if dst_height<spec.min_h:
             return "target height %i is out of range: minimum is %i", dst_height, spec.min_h
-        elif src_width>spec.max_w:
+        if src_width>spec.max_w:
             return "source width %i is out of range: maximum is %i", src_width, spec.max_w
-        elif src_height>spec.max_h:
+        if src_height>spec.max_h:
             return "source height %i is out of range: maximum is %i", src_height, spec.max_h
-        elif dst_width>spec.max_w:
+        if dst_width>spec.max_w:
             return "target width %i is out of range: maximum is %i", dst_width, spec.max_w
-        elif dst_height>spec.max_h:
+        if dst_height>spec.max_h:
             return "target height %i is out of range: maximum is %i", dst_height, spec.max_h
         return None
 
     def paint_with_video_decoder(self, decoder_module, coding, img_data, x, y, width, height, options, callbacks):
-        #log("paint_with_video_decoder%s", (decoder_module, coding, "%s bytes" % len(img_data), x, y, width, height, options, callbacks))
         assert decoder_module, "decoder module not found for %s" % coding
         dl = self._decoder_lock
         if dl is None:
@@ -464,14 +558,17 @@ class WindowBackingBase(object):
                     log("paint_with_video_decoder: encoding changed from %s to %s", vd.get_encoding(), coding)
                     self.do_clean_video_decoder()
                 elif vd.get_width()!=enc_width or vd.get_height()!=enc_height:
-                    log("paint_with_video_decoder: video dimensions have changed from %s to %s", (vd.get_width(), vd.get_height()), (enc_width, enc_height))
+                    log("paint_with_video_decoder: video dimensions have changed from %s to %s",
+                        (vd.get_width(), vd.get_height()), (enc_width, enc_height))
                     self.do_clean_video_decoder()
                 elif vd.get_colorspace()!=input_colorspace:
                     #this should only happen on encoder restart, which means this should be the first frame:
-                    log.warn("Warning: colorspace unexpectedly changed from %s to %s", vd.get_colorspace(), input_colorspace)
+                    log.warn("Warning: colorspace unexpectedly changed from %s to %s",
+                             vd.get_colorspace(), input_colorspace)
                     self.do_clean_video_decoder()
             if self._video_decoder is None:
-                log("paint_with_video_decoder: new %s(%s,%s,%s)", decoder_module.Decoder, width, height, input_colorspace)
+                log("paint_with_video_decoder: new %s(%s,%s,%s)",
+                    decoder_module.Decoder, width, height, input_colorspace)
                 vd = decoder_module.Decoder()
                 vd.init_context(coding, enc_width, enc_height, input_colorspace)
                 self._video_decoder = vd
@@ -484,13 +581,17 @@ class WindowBackingBase(object):
                     #and this frame references those, so assume all is well:
                     fire_paint_callbacks(callbacks)
                 else:
-                    fire_paint_callbacks(callbacks, False, "video decoder %s failed to decode %i bytes of %s data" % (vd.get_type(), len(img_data), coding))
+                    fire_paint_callbacks(callbacks, False,
+                                         "video decoder %s failed to decode %i bytes of %s data" % (
+                                             vd.get_type(), len(img_data), coding))
                     log.error("Error: decode failed on %s bytes of %s data", len(img_data), coding)
                     log.error(" %sx%s pixels using %s", width, height, vd.get_type())
                     log.error(" frame options:")
                     for k,v in options.items():
                         log.error("   %s=%s", k, v)
                 return
+
+            x, y = self.gravity_adjust(x, y, options)
             self.do_video_paint(img, x, y, enc_width, enc_height, width, height, options, callbacks)
         if self._backing is None:
             self.close_decoder(True)
@@ -550,9 +651,10 @@ class WindowBackingBase(object):
         data = mmap_read(self.mmap, *img_data)
         rgb_format = options.strget(b"rgb_format", b"RGB")
         #Note: BGR(A) is only handled by gl_window_backing
+        x, y = self.gravity_adjust(x, y, options)
         self.do_paint_rgb(rgb_format, data, x, y, width, height, rowstride, options, callbacks)
 
-    def paint_scroll(self, *args):
+    def paint_scroll(self, _img_data, _options, callbacks):
         raise NotImplementedError("no paint scroll on %s" % type(self))
 
 
@@ -560,37 +662,43 @@ class WindowBackingBase(object):
         """ dispatches the paint to one of the paint_XXXX methods """
         try:
             assert self._backing is not None
-            log("draw_region(%s, %s, %s, %s, %s, %s bytes, %s, %s, %s)", x, y, width, height, coding, len(img_data), rowstride, options, callbacks)
+            log("draw_region(%s, %s, %s, %s, %s, %s bytes, %s, %s, %s)",
+                x, y, width, height, coding, len(img_data), rowstride, options, callbacks)
             coding = bytestostr(coding)
             options["encoding"] = coding            #used for choosing the color of the paint box
             if INTEGRITY_HASH:
                 l = options.intget("z.len")
                 if l:
                     assert l==len(img_data), "compressed pixel data failed length integrity check: expected %i bytes but got %i" % (l, len(img_data))
-                md5 = options.strget("z.md5")
-                if md5:
-                    h = hashlib.md5(img_data)
+                try:
+                    chksum = options.get("z.md5")
+                    if chksum:
+                        h = hashlib.md5(img_data)
+                except ValueError:
+                    chksum = options.get("z.sha1")
+                    if chksum:
+                        h = hashlib.sha1(img_data)
+                if h:
                     hd = h.hexdigest()
-                    assert md5==hd, "pixel data failed compressed md5 integrity check: expected %s but got %s" % (md5, hd)
-                deltalog("passed compressed data integrity checks: len=%s, md5=%s (type=%s)", l, md5, type(img_data))
+                    assert chksum==hd, "pixel data failed compressed chksum integrity check: expected %s but got %s" % (chksum, hd)
+                deltalog("passed compressed data integrity checks: len=%s, chksum=%s (type=%s)", l, chksum, type(img_data))
             if coding == "mmap":
                 self.idle_add(self.paint_mmap, img_data, x, y, width, height, rowstride, options, callbacks)
-            elif coding == "rgb24" or coding == "rgb32":
+            elif coding in ("rgb24", "rgb32"):
                 #avoid confusion over how many bytes-per-pixel we may have:
                 rgb_format = options.strget(b"rgb_format")
-                if rgb_format:
-                    Bpp = len(rgb_format)
-                elif coding=="rgb24":
-                    #legacy:
-                    rgb_format = "RGB"
-                else:
-                    #legacy:
-                    rgb_format = "RGBX"
+                if not rgb_format:
+                    rgb_format = {
+                        "rgb24" : "RGB",
+                        "rgb32" : "RGBX",
+                        }.get(coding)
                 if rowstride==0:
-                    rowstride = width * Bpp
+                    rowstride = width * len(rgb_format)
                 self.paint_rgb(rgb_format, img_data, x, y, width, height, rowstride, options, callbacks)
             elif coding in VIDEO_DECODERS:
-                self.paint_with_video_decoder(VIDEO_DECODERS.get(coding), coding, img_data, x, y, width, height, options, callbacks)
+                self.paint_with_video_decoder(VIDEO_DECODERS.get(coding),
+                                              coding,
+                                              img_data, x, y, width, height, options, callbacks)
             elif self.jpeg_decoder and coding=="jpeg":
                 self.paint_jpeg(img_data, x, y, width, height, options, callbacks)
             elif coding == "webp":
