@@ -1,20 +1,20 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2013-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 #@PydevCodeAnalysisIgnore
 
 import os
-import pycuda
-from pycuda import driver
+import pycuda               #@UnresolvedImport
+from pycuda import driver   #@UnresolvedImport
 
-from xpra.util import engs, print_nested_dict, envbool, envint
+from xpra.util import engs, print_nested_dict, envbool, envint, csv
 from xpra.os_util import monotonic_time, bytestostr, PYTHON2
 from xpra.log import Logger
 
-log = Logger("csc", "cuda")
+log = Logger("cuda")
 
 IGNORE_BLACKLIST = envbool("XPRA_CUDA_IGNORE_BLACKLIST", False)
 MIN_FREE_MEMORY = envint("XPRA_CUDA_MIN_FREE_MEMORY", 10)
@@ -95,6 +95,7 @@ def get_prefs():
                 log("get_prefs() '%s' is not a file!", conf_file)
                 continue
             try:
+                c_prefs = {}
                 with open(conf_file, "rb") as f:
                     for line in f:
                         sline = bytestostr(line.strip().rstrip(b'\r\n').strip())
@@ -103,33 +104,60 @@ def get_prefs():
                             continue
                         name = props[0].strip()
                         value = props[1].strip()
-                        if name in ("blacklist", ):
-                            PREFS.setdefault(name, []).append(value)
+                        if name in ("blacklist", "enabled-devices", "disabled-devices"):
+                            for v in value.split(","):
+                                c_prefs.setdefault(name, []).append(v.strip())
                         elif name in ("device-id", "device-name"):
-                            PREFS[name] = value
+                            c_prefs[name] = value
             except Exception as e:
                 log.error("Error: cannot read cuda configuration file '%s':", conf_file)
                 log.error(" %s", e)
+            log("get_prefs() '%s' : %s", conf_file, c_prefs)
+            PREFS.update(c_prefs)
     return PREFS
 
 def get_pref(name):
-    assert name in ("blacklist", "device-id", "device-name")
+    assert name in ("blacklist", "device-id", "device-name", "enabled-devices", "disabled-devices")
     #ie: env_name("device-id")="XPRA_CUDA_DEVICE_ID"
     env_name = "XPRA_CUDA_%s" % str(name).upper().replace("-", "_")
     env_value = os.environ.get(env_name)
     if env_value is not None:
+        if name in ("blacklist", "enabled-devices", "disabled-devices"):
+            return env_value.split(",")
         return env_value
     return get_prefs().get(name)
 
+def get_gpu_list(list_type):
+    v = get_pref(list_type)
+    log("get_gpu_list(%s) pref=%s", list_type, v)
+    if not v:
+        return None
+    if "all" in v:
+        return True
+    if "none" in v:
+        return []
+    try:
+        return [int(x.strip()) for x in v]
+    except ValueError:
+        log("get_gpu_list(%s)", list_type, exc_info=True)
+        log.error("Error: invalid value for '%s' CUDA preference", list_type)
+        return None
+
 
 DEVICES = None
 def init_all_devices():
     global DEVICES, DEVICE_INFO, DEVICE_NAME
     if DEVICES is not None:
         return DEVICES
-    log.info("CUDA initialization (this may take a few seconds)")
     DEVICES = []
     DEVICE_INFO = {}
+    enabled_gpus = get_gpu_list("enabled-devices")
+    disabled_gpus = get_gpu_list("disabled-devices")
+    if disabled_gpus is True or enabled_gpus==[]:
+        log("all devices are disabled!")
+        return DEVICES
+    log.info("CUDA initialization (this may take a few seconds)")
+    log("enabled: %s, disabled: %s", csv(enabled_gpus), csv(disabled_gpus))
     try:
         driver.init()
     except Exception as e:
@@ -146,6 +174,12 @@ def init_all_devices():
     da = driver.device_attribute
     cf = driver.ctx_flags
     for i in range(ngpus):
+        if enabled_gpus not in (None, True) and i not in enabled_gpus:
+            log("device %i is not in the list of enabled gpus, skipped", i)
+            continue
+        if disabled_gpus is not None and i in disabled_gpus:
+            log("device %i is in the list of disabled gpus, skipped", i)
+            continue
         device = None
         context = None
         devinfo = "gpu %i" % i
@@ -222,17 +256,26 @@ def reset_state():
     DEVICE_STATE = {}
 
 
-def select_device(preferred_device_id=-1, preferred_device_name=None, min_compute=0):
-    if preferred_device_name is None:
-        preferred_device_name = get_pref("device-name")
-    if preferred_device_id<0:
-        device_id = get_pref("device-id")
+def select_device(preferred_device_id=-1, min_compute=0):
+    log("select_device(%s, %s)", preferred_device_id, min_compute)
+    for device_id in (preferred_device_id, get_pref("device-id")):
         if device_id is not None and device_id>=0:
-            preferred_device_id = device_id
+            #try to honour the device specified:
+            try:
+                device, context, compute, tpct = load_device(device_id)
+            finally:
+                context.pop()
+                context.detach()
+            if compute<min_compute:
+                log.warn("Warning: GPU device %i only supports compute %#x", device_id, compute)
+            if tpct<MIN_FREE_MEMORY:
+                log.warn("Warning: GPU device %i is low on memory: %i%%", device_id, tpct)
+            return device_id, device
+    #load preferences:
+    preferred_device_name = get_pref("device-name")
     devices = init_all_devices()
     global DEVICE_STATE
     free_pct = 0
-    cf = driver.ctx_flags
     #split device list according to device state:
     ok_devices = [device_id for device_id in devices if DEVICE_STATE.get(device_id, True) is True]
     nok_devices = [device_id for device_id in devices if DEVICE_STATE.get(device_id, True) is not True]
@@ -243,25 +286,10 @@ def select_device(preferred_device_id=-1, preferred_device_name=None, min_comput
         for device_id in device_list:
             context = None
             try:
-                log("device %i", device_id)
-                device = driver.Device(device_id)
-                log("select_device: testing device %s: %s", device_id, device_info(device))
-                context = device.make_context(flags=cf.SCHED_YIELD | cf.MAP_HOST)
-                log("created context=%s", context)
-                free, total = driver.mem_get_info()
-                log("memory: free=%sMB, total=%sMB",  int(free/1024/1024), int(total/1024/1024))
-                tpct = 100*free/total
-                SMmajor, SMminor = device.compute_capability()
-                compute = (SMmajor<<4) + SMminor
+                device, context, compute, tpct = load_device(device_id)
                 if compute<min_compute:
                     log("ignoring device %s: compute capability %#x (minimum %#x required)",
                         device_info(device), compute, min_compute)
-                elif device_id==preferred_device_id:
-                    l = log
-                    if len(device_list)>1:
-                        l = log.info
-                    l("device matches preferred device id %s: %s", preferred_device_id, device_info(device))
-                    return device_id, device
                 elif preferred_device_name and device_info(device).find(preferred_device_name)>=0:
                     log("device matches preferred device name: %s", preferred_device_name)
                     return device_id, device
@@ -283,6 +311,21 @@ def select_device(preferred_device_id=-1, preferred_device_name=None, min_comput
             return selected_device_id, selected_device
     return -1, None
 
+def load_device(device_id):
+    log("load_device(%i)", device_id)
+    device = driver.Device(device_id)
+    log("select_device: testing device %s: %s", device_id, device_info(device))
+    cf = driver.ctx_flags
+    context = device.make_context(flags=cf.SCHED_YIELD | cf.MAP_HOST)
+    log("created context=%s", context)
+    free, total = driver.mem_get_info()
+    log("memory: free=%sMB, total=%sMB",  int(free/1024/1024), int(total/1024/1024))
+    tpct = 100*free/total
+    SMmajor, SMminor = device.compute_capability()
+    compute = (SMmajor<<4) + SMminor
+    return device, context, compute, tpct
+
+
 
 CUDA_ERRORS_INFO = {
     #this list is taken from the CUDA 7.0 SDK header file,
