@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -27,7 +27,7 @@ log("codec loader settings: SELFTEST=%s, FULL_SELFTEST=%s, CODEC_FAIL_IMPORT=%s,
 
 codec_errors = {}
 codecs = {}
-def codec_import_check(name, description, top_module, class_module, *classnames):
+def codec_import_check(name, description, top_module, class_module, classnames):
     log("%s:", name)
     log(" codec_import_check%s", (name, description, top_module, class_module, classnames))
     try:
@@ -44,41 +44,46 @@ def codec_import_check(name, description, top_module, class_module, *classnames)
         log.warn(" cannot load %s (%s):", name, description, exc_info=True)
         codec_errors[name] = str(e)
         return None
+    classname = None
     try:
         #module is present
-        classname = "?"
         try:
             log(" %s found, will check for %s in %s", top_module, classnames, class_module)
-            for classname in classnames:
-                ic =  __import__(class_module, {}, {}, classname)
-                try:
-                    #run init_module?
-                    init_module = getattr(ic, "init_module", None)
-                    log("%s: init_module=%s", class_module, init_module)
-                    if init_module:
-                        init_module()
-                    selftest = getattr(ic, "selftest", None)
-                    log("%s.selftest=%s", name, selftest)
-                    if SELFTEST and selftest:
-                        if name in CODEC_FAIL_SELFTEST:
-                            raise ImportError("codec found in fail selftest list")
-                        try:
-                            selftest(FULL_SELFTEST)
-                        except Exception as e:
-                            log.warn("Warning: %s failed its self test", name)
-                            for x in str(e).splitlines():
-                                log.warn(" %s", x)
-                            log("%s failed", selftest, exc_info=True)
-                            continue
-                finally:
-                    cleanup_module = getattr(ic, "cleanup_module", None)
-                    log("%s: cleanup_module=%s", class_module, cleanup_module)
-                    if cleanup_module:
-                        cleanup_module()
-                #log.warn("codec_import_check(%s, ..)=%s" % (name, ic))
-                log(" found %s : %s", name, ic)
-                codecs[name] = ic
-                return ic
+            ic =  __import__(class_module, {}, {}, classnames)
+            try:
+                #run init_module?
+                init_module = getattr(ic, "init_module", None)
+                log("%s: init_module=%s", class_module, init_module)
+                if init_module:
+                    init_module()
+
+                if classnames:
+                    for classname in classnames:
+                        clazz = getattr(ic, classname)
+                        log("%s: %s=%s", class_module, classname, clazz)
+
+                selftest = getattr(ic, "selftest", None)
+                log("%s.selftest=%s", name, selftest)
+                if SELFTEST and selftest:
+                    if name in CODEC_FAIL_SELFTEST:
+                        raise ImportError("codec found in fail selftest list")
+                    try:
+                        selftest(FULL_SELFTEST)
+                    except Exception as e:
+                        log.warn("Warning: %s failed its self test", name)
+                        for x in str(e).splitlines():
+                            log.warn(" %s", x)
+                        log("%s failed", selftest, exc_info=True)
+                        return None
+            finally:
+                cleanup_module = getattr(ic, "cleanup_module", None)
+                log("%s: cleanup_module=%s", class_module, cleanup_module)
+                if cleanup_module:
+                    cleanup_module()
+            #log.warn("codec_import_check(%s, ..)=%s" % (name, ic))
+            log(" found %s : %s", name, ic)
+            codecs[name] = ic
+            return ic
         except ImportError as e:
             codec_errors[name] = str(e)
             l = log.error
@@ -125,69 +130,75 @@ def add_codec_version(name, top_module, version="get_version()", alt_version="__
         log.warn("", exc_info=True)
     return None
 
+def xpra_codec_import(name, description, top_module, class_module, classname):
+    xpra_top_module = "xpra.codecs.%s" % top_module
+    xpra_class_module = "%s.%s" % (xpra_top_module, class_module)
+    if codec_import_check(name, description, xpra_top_module, xpra_class_module, classname):
+        version_name = name
+        if name.startswith("enc_") or name.startswith("dec_") or name.startswith("csc_"):
+            version_name = name[4:]
+        add_codec_version(version_name, xpra_class_module)
 
-loaded = None
-def load_codecs(encoders=True, decoders=True, csc=True):
-    global loaded
-    if loaded:
+
+CODEC_OPTIONS = {
+    #encoders:
+    "enc_pillow"    : ("Pillow encoder",    "pillow",       "encoder", "encode"),
+    "enc_webp"      : ("webp encoder",      "webp",         "encoder", "encode"),
+    "enc_jpeg"      : ("JPEG encoder",      "jpeg",         "encoder", "encode"),
+    #video encoders:
+    "enc_vpx"       : ("vpx encoder",       "vpx",          "encoder", "Encoder"),
+    "enc_x264"      : ("x264 encoder",      "enc_x264",     "encoder", "Encoder"),
+    "enc_x265"      : ("x265 encoder",      "enc_x265",     "encoder", "Encoder"),
+    "nvenc"         : ("nvenc encoder",     "nvenc",        "encoder", "Encoder"),
+    "enc_ffmpeg"    : ("ffmpeg encoder",    "enc_ffmpeg",   "encoder", "Encoder"),
+    #csc:
+    "csc_swscale"   : ("swscale colorspace conversion", "csc_swscale", "colorspace_converter", "ColorspaceConverter"),
+    "csc_libyuv"    : ("libyuv colorspace conversion", "csc_libyuv", "colorspace_converter", "ColorspaceConverter"),
+    #decoders:
+    "dec_pillow"    : ("Pillow decoder",    "pillow",       "decoder", "decompress"),
+    "dec_webp"      : ("webp decoder",      "webp",         "decoder", "decompress"),
+    "dec_jpeg"      : ("JPEG decoder",      "jpeg",         "decoder", "decompress_to_rgb", "decompress_to_yuv"),
+    #video decoders:
+    "dec_vpx"       : ("vpx decoder",       "vpx",          "decoder", "Decoder"),
+    "dec_avcodec2"  : ("avcodec2 decoder",  "dec_avcodec2", "decoder", "Decoder"),
+    }
+
+def load_codec(name):
+    if has_codec(name):
         return
-    loaded = True
+    try:
+        option = CODEC_OPTIONS[name]
+        description, top_module, class_module = option[:3]
+        classnames = option[3:]
+    except KeyError:
+        log.error("Error: invalid codec name '%s'", name)
+    else:
+        xpra_codec_import(name, description, top_module, class_module, classnames)
+
+
+def load_codecs(encoders=True, decoders=True, csc=True, video=True):
     show = []
     log("loading codecs")
-    if encoders or decoders:
-        codec_import_check("PIL", "Python Imaging Library", "PIL", "PIL", "Image")
-        add_codec_version("PIL", "PIL.Image", "PILLOW_VERSION", "VERSION")
+
+    def load(*names):
+        for name in names:
+            load_codec(name)
 
     if encoders:
         show += list(ENCODER_CODECS)
-        codec_import_check("enc_pillow", "Pillow encoder", "xpra.codecs.pillow", "xpra.codecs.pillow.encode", "encode")
-        add_codec_version("enc_pillow", "xpra.codecs.pillow.encode")
-
-        codec_import_check("enc_webp", "webp encoder", "xpra.codecs.webp", "xpra.codecs.webp.encode", "compress")
-        add_codec_version("enc_webp", "xpra.codecs.webp.encode")
-
-        codec_import_check("enc_jpeg", "JPEG decoder", "xpra.codecs.jpeg", "xpra.codecs.jpeg.encoder", "encoder")
-        add_codec_version("enc_jpeg", "xpra.codecs.jpeg.encoder")
-
-        codec_import_check("enc_vpx", "vpx encoder", "xpra.codecs.vpx", "xpra.codecs.vpx.encoder", "Encoder")
-        add_codec_version("vpx", "xpra.codecs.vpx.decoder")
-
-        codec_import_check("enc_x264", "x264 encoder", "xpra.codecs.enc_x264", "xpra.codecs.enc_x264.encoder", "Encoder")
-        add_codec_version("x264", "xpra.codecs.enc_x264.encoder")
-
-        codec_import_check("enc_x265", "x265 encoder", "xpra.codecs.enc_x265", "xpra.codecs.enc_x265.encoder", "Encoder")
-        add_codec_version("x265", "xpra.codecs.enc_x265.encoder")
-
-        codec_import_check("nvenc", "nvenc encoder", "xpra.codecs.nvenc", "xpra.codecs.nvenc.encoder", "Encoder")
-        add_codec_version("nvenc", "xpra.codecs.nvenc.encoder")
-
-        codec_import_check("enc_ffmpeg", "ffmpeg encoder", "xpra.codecs.enc_ffmpeg", "xpra.codecs.enc_ffmpeg.encoder", "Encoder")
-        add_codec_version("ffmpeg", "xpra.codecs.enc_ffmpeg.encoder")
-
-    if csc:
+        load(*ENCODER_CODECS)
+        if video:
+            show += list(ENCODER_VIDEO_CODECS)
+            load(*ENCODER_VIDEO_CODECS)
+    if csc and video:
         show += list(CSC_CODECS)
-        codec_import_check("csc_swscale", "swscale colorspace conversion", "xpra.codecs.csc_swscale", "xpra.codecs.csc_swscale.colorspace_converter", "ColorspaceConverter")
-        add_codec_version("swscale", "xpra.codecs.csc_swscale.colorspace_converter")
-
-        codec_import_check("csc_libyuv", "libyuv colorspace conversion", "xpra.codecs.csc_libyuv", "xpra.codecs.csc_libyuv.colorspace_converter", "ColorspaceConverter")
-        add_codec_version("libyuv", "xpra.codecs.csc_libyuv.colorspace_converter")
-
+        load(*CSC_CODECS)
     if decoders:
         show += list(DECODER_CODECS)
-        codec_import_check("dec_pillow", "Pillow decoder", "xpra.codecs.pillow", "xpra.codecs.pillow.decode", "decode")
-        add_codec_version("dec_pillow", "xpra.codecs.pillow.decode")
-
-        codec_import_check("dec_webp", "webp decoder", "xpra.codecs.webp", "xpra.codecs.webp.decode", "decompress")
-        add_codec_version("dec_webp", "xpra.codecs.webp.decode")
-
-        codec_import_check("dec_jpeg", "JPEG decoder", "xpra.codecs.jpeg", "xpra.codecs.jpeg.decoder", "decoder")
-        add_codec_version("dec_jpeg", "xpra.codecs.jpeg.decoder")
-
-        codec_import_check("dec_vpx", "vpx decoder", "xpra.codecs.vpx", "xpra.codecs.vpx.decoder", "Decoder")
-        add_codec_version("vpx", "xpra.codecs.vpx.encoder")
-
-        codec_import_check("dec_avcodec2", "avcodec2 decoder", "xpra.codecs.dec_avcodec2", "xpra.codecs.dec_avcodec2.decoder", "Decoder")
-        add_codec_version("avcodec2", "xpra.codecs.dec_avcodec2.decoder")
+        load(*DECODER_CODECS)
+        if video:
+            show += list(DECODER_VIDEO_CODECS)
+            load(*DECODER_VIDEO_CODECS)
 
     log("done loading codecs")
     log("found:")
@@ -200,32 +211,31 @@ def load_codecs(encoders=True, decoders=True, csc=True):
 
 
 def get_codec_error(name):
-    assert loaded
     return codec_errors.get(name)
 
 def get_codec(name):
-    assert loaded
     return codecs.get(name)
 
 def get_codec_version(name):
-    assert loaded
     return codec_versions.get(name)
 
 def has_codec(name):
-    assert loaded
     return name in codecs
 
 
 CSC_CODECS = "csc_swscale", "csc_libyuv"
-ENCODER_CODECS = "enc_pillow", "enc_vpx", "enc_webp", "enc_x264", "enc_x265", "nvenc", "enc_ffmpeg", "enc_jpeg"
-DECODER_CODECS = "dec_pillow", "dec_vpx", "dec_webp", "dec_avcodec2", "dec_jpeg"
+ENCODER_CODECS = "enc_pillow", "enc_webp", "enc_jpeg"
+ENCODER_VIDEO_CODECS = "enc_vpx", "enc_x264", "enc_x265", "nvenc", "enc_ffmpeg"
+DECODER_CODECS = "dec_pillow", "dec_webp", "dec_jpeg"
+DECODER_VIDEO_CODECS = "dec_vpx", "dec_avcodec2"
 
-ALL_CODECS = tuple(set(CSC_CODECS + ENCODER_CODECS + DECODER_CODECS))
+ALL_CODECS = tuple(set(CSC_CODECS + ENCODER_CODECS + ENCODER_VIDEO_CODECS + DECODER_CODECS + DECODER_VIDEO_CODECS))
 
 
 def get_rgb_compression_options():
     from xpra.net import compression
     compressors = compression.get_enabled_compressors()
+    compressors = [x for x in compressors if x!="brotli"]
     RGB_COMP_OPTIONS  = ["Raw RGB"]
     if compressors:
         RGB_COMP_OPTIONS  += ["/".join(compressors)]
@@ -244,7 +254,6 @@ def get_encoding_name(encoding):
           "png/P"   : "PNG (8bpp colour)",
           "png/L"   : "PNG (8bpp grayscale)",
           "jpeg"    : "JPEG",
-          "jpeg2000": "JPEG 2000",
           "rgb"     : " + ".join(get_rgb_compression_options()) + " (24/32bpp)",
         }
     return ENCODINGS_TO_NAME.get(encoding, encoding)
@@ -252,6 +261,7 @@ def get_encoding_name(encoding):
 def get_encoding_help(encoding):
     from xpra.net import compression
     compressors = compression.get_enabled_compressors()
+    compressors = [x for x in compressors if x!="brotli"]
     return {
           "auto"    : "automatic mode (recommended)",
           "h264"    : "H.264 video codec",
@@ -264,8 +274,8 @@ def get_encoding_help(encoding):
           "png/L"   : "Portable Network Graphics (lossy, 8bpp grayscale)",
           "webp"    : "WebP compression (supports lossless and lossy modes)",
           "jpeg"    : "JPEG lossy compression",
-          "jpeg2000": "JPEG 2000 lossy compression (slow)",
-          "rgb"     : "Raw RGB pixels, lossless, compressed using %s (24bpp or 32bpp for transparency)" % (" or ".join(compressors)),
+          "rgb"     : "Raw RGB pixels, lossless,"
+                      +" compressed using %s (24bpp or 32bpp for transparency)" % (" or ".join(compressors)),
           }.get(encoding)
 
 
@@ -285,13 +295,15 @@ def encoding_help(encoding):
 
 def main():
     from xpra.platform import program_context
-    from xpra.log import enable_color
+    from xpra.log import enable_color, LOG_FORMAT, NOPREFIX_FORMAT
     from xpra.util import print_nested_dict, pver
     with program_context("Loader", "Encoding Info"):
-        enable_color()
         verbose = "-v" in sys.argv or "--verbose" in sys.argv
+        format_string = NOPREFIX_FORMAT
         if verbose:
+            format_string = LOG_FORMAT
             log.enable_debug()
+        enable_color(format_string)
 
         load_codecs()
         #not really a codec, but gets used by codecs, so include version info:
@@ -322,7 +334,7 @@ def main():
                         i = mod.get_info()
                         for k,v in sorted(i.items()):
                             print("                         %s = %s" % (k,v))
-                    except:
+                    except Exception:
                         pass
                 except Exception as e:
                     print("error getting extra information on %s: %s" % (name, e))
