@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2018-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -8,20 +8,25 @@ import sys
 import os.path
 import subprocess
 
-from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, import_pango, import_glib
+from xpra.gtk_common.gobject_compat import (
+    import_gtk, import_gdk, import_pango, import_glib,
+    register_os_signals,
+    )
+from xpra.gtk_common.gtk_util import (
+    gtk_main, set_tooltip_text, add_close_accel,
+    pixbuf_new_from_file, add_window_accel, imagebutton,
+    window_defaults, scaled_image, WIN_POS_CENTER,
+    )
+from xpra.platform.paths import get_icon_dir, get_xpra_command
+from xpra.os_util import OSX, WIN32, platform_name
+from xpra.log import Logger
+
+log = Logger("client", "util")
+
 gtk = import_gtk()
 gdk = import_gdk()
 pango = import_pango()
 glib = import_glib()
-glib.threads_init()
-
-from xpra.platform.paths import get_icon_dir, get_xpra_command
-from xpra.os_util import OSX, WIN32
-from xpra.gtk_common.gtk_util import gtk_main, set_tooltip_text, add_close_accel, pixbuf_new_from_file, add_window_accel, imagebutton, window_defaults, scaled_image, WIN_POS_CENTER
-
-from xpra.log import Logger
-log = Logger("client", "util")
-
 
 try:
     from xpra import client
@@ -45,7 +50,9 @@ except ImportError:
 
 
 def exec_command(cmd):
-    proc = subprocess.Popen(cmd)
+    env = os.environ.copy()
+    env["XPRA_WAIT_FOR_INPUT"] = "0"
+    proc = subprocess.Popen(cmd, env=env)
     log("exec_command(%s)=%s", cmd, proc)
     return proc
 
@@ -85,23 +92,32 @@ class GUI(gtk.Window):
         label_font = pango.FontDescription("sans 16")
         if has_client:
             icon = get_pixbuf("browse.png")
-            self.browse_button = imagebutton("Browse", icon, "Browse and connect to local sessions", clicked_callback=self.browse, icon_size=48, label_font=label_font)
+            self.browse_button = imagebutton("Browse", icon,
+                                             "Browse and connect to local sessions", clicked_callback=self.browse,
+                                             icon_size=48, label_font=label_font)
             self.widgets.append(self.browse_button)
             icon = get_pixbuf("connect.png")
-            self.connect_button = imagebutton("Connect", icon, "Connect to a session", clicked_callback=self.show_launcher, icon_size=48, label_font=label_font)
+            self.connect_button = imagebutton("Connect", icon,
+                                              "Connect to a session", clicked_callback=self.show_launcher,
+                                              icon_size=48, label_font=label_font)
             self.widgets.append(self.connect_button)
         if has_server:
             icon = get_pixbuf("server-connected.png")
-            self.shadow_button = imagebutton("Shadow", icon, "Start a shadow server", clicked_callback=self.start_shadow, icon_size=48, label_font=label_font)
+            self.shadow_button = imagebutton("Shadow", icon,
+                                             "Start a shadow server", clicked_callback=self.start_shadow,
+                                             icon_size=48, label_font=label_font)
             if not has_shadow:
                 set_tooltip_text(self.shadow_button, "This build of Xpra does not support starting sessions")
                 self.shadow_button.set_sensitive(False)
             self.widgets.append(self.shadow_button)
             icon = get_pixbuf("windows.png")
-            self.start_button = imagebutton("Start", icon, "Start a session", clicked_callback=self.start, icon_size=48, label_font=label_font)
+            self.start_button = imagebutton("Start", icon,
+                                            "Start a session", clicked_callback=self.start,
+                                            icon_size=48, label_font=label_font)
             #not all builds and platforms can start sessions:
             if OSX or WIN32:
-                set_tooltip_text(self.start_button, "Starting sessions is not supported on %s" % sys.platform)
+                set_tooltip_text(self.start_button,
+                                 "Starting sessions is not supported on %s" % platform_name(sys.platform))
                 self.start_button.set_sensitive(False)
             elif not has_server:
                 set_tooltip_text(self.start_button, "This build of Xpra does not support starting sessions")
@@ -130,9 +146,9 @@ class GUI(gtk.Window):
         log("do_quit()")
         gtk.main_quit()
 
-    def app_signal(self, signum, frame):
+    def app_signal(self, signum):
         self.exit_code = 128 + signum
-        log("app_signal(%s, %s) exit_code=%i", signum, frame, self.exit_code)
+        log("app_signal(%s) exit_code=%i", signum, self.exit_code)
         self.do_quit()
 
 
@@ -162,12 +178,14 @@ class GUI(gtk.Window):
             self.busy_cursor(self.shadow_button)
 
     def browse(self, *_args):
-        subcommand = "mdns-gui"
+        subcommand = "sessions"
         try:
-            from xpra.net import mdns
-            assert mdns
+            from xpra.net.mdns import get_listener_class
+            listener = get_listener_class()
+            if listener:
+                subcommand = "mdns-gui"
         except ImportError:
-            subcommand = "sessions"
+            pass
         cmd = get_xpra_command()+[subcommand]
         proc = exec_command(cmd)
         if proc.poll() is None:
@@ -179,7 +197,7 @@ class GUI(gtk.Window):
         if proc.poll() is None:
             self.busy_cursor(self.connect_button)
 
-    def start(self, *args):
+    def start(self, *_args):
         if not self.start_session:
             self.start_session = StartSession()
         self.start_session.show()
@@ -369,7 +387,7 @@ class StartSession(gtk.Window):
             submenu = self.categories[category]
             assert isinstance(submenu, Menu)
             for entry in submenu.getEntries():
-                #TODO: can we have more than 2 levels of submenus?
+                #can we have more than 2 levels of submenus?
                 if isinstance(entry, MenuEntry):
                     name = entry.DesktopEntry.getName()
                     self.commands[name] = entry.DesktopEntry
@@ -379,7 +397,7 @@ class StartSession(gtk.Window):
         if self.commands:
             self.command_combo.set_active(0)
         self.command_box.show()
-            
+
 
     def populate_command(self):
         log("populate_command()")
@@ -412,7 +430,7 @@ class StartSession(gtk.Window):
         else:
             self.desktop_entry = None
             self.run_btn.set_sensitive(False)
-            
+
 
     def seamless_toggled(self, *args):
         self.seamless = self.seamless_btn.get_active()
@@ -429,7 +447,10 @@ class StartSession(gtk.Window):
         self.hide()
         if xdg:
             if self.desktop_entry.getTryExec():
-                command = self.desktop_entry.findTryExec()
+                try:
+                    command = self.desktop_entry.findTryExec()
+                except Exception:
+                    command = self.desktop_entry.getTryExec()
             else:
                 command = self.desktop_entry.getExec()
         else:
@@ -459,9 +480,7 @@ def main():
         enable_color()
         init()
         gui = GUI()
-        import signal
-        signal.signal(signal.SIGINT, gui.app_signal)
-        signal.signal(signal.SIGTERM, gui.app_signal)
+        register_os_signals(gui.app_signal)
         ready()
         if OSX:
             from xpra.platform.darwin.gui import wait_for_open_handlers
