@@ -32,6 +32,7 @@ SOCKET_TIMEOUT = envint("XPRA_SOCKET_TIMEOUT", 20)
 SSL_PEEK = envbool("XPRA_SSL_PEEK", True)
 #this is more proper but would break the proxy server:
 SOCKET_SHUTDOWN = envbool("XPRA_SOCKET_SHUTDOWN", False)
+LOG_TIMEOUTS = envint("XPRA_LOG_TIMEOUTS", 1)
 
 #on some platforms (ie: OpenBSD), reading and writing from sockets
 #raises an IOError but we should continue if the error code is EINTR
@@ -70,16 +71,9 @@ PROTOCOL_STR = {}
 FAMILY_STR = {}
 for x in dir(socket):
     if x.startswith("AF_"):
-        try:
-            PROTOCOL_STR[getattr(socket, x)] = x
-        except:
-            pass
+        PROTOCOL_STR[getattr(socket, x)] = x
     if x.startswith("SOCK_"):
-        try:
-            FAMILY_STR[getattr(socket, x)] = x
-        except:
-            pass
-
+        FAMILY_STR[getattr(socket, x)] = x
 
 
 def set_continue_wait(v):
@@ -123,8 +117,9 @@ def untilConcludes(is_active_cb, can_retry_cb, f, *a, **kw):
             return f(*a, **kw)
         except Exception as e:
             retry = can_retry_cb(e)
-            log("untilConcludes(%s, %s, %s, %s, %s) %s, retry=%s",
-                is_active_cb, can_retry_cb, f, a, kw, e, retry, exc_info=True)
+            if LOG_TIMEOUTS>0:
+                log("untilConcludes(%s, %s, %s, %s, %s) %s, retry=%s",
+                    is_active_cb, can_retry_cb, f, a, kw, e, retry, exc_info=LOG_TIMEOUTS>=2)
             e = None
             if not retry:
                 raise
@@ -136,26 +131,29 @@ def untilConcludes(is_active_cb, can_retry_cb, f, *a, **kw):
 
 def pretty_socket(s):
     try:
+        if isinstance(s, str):
+            return s
         if len(s)==2:
             return "%s:%s" % (s[0], s[1])
-        assert len(s)==4
-        return csv(str(x) for x in s)
-    except:
-        return str(s)
+        if len(s)==4:
+            return csv(str(x) for x in s)
+    except (ValueError, TypeError):
+        pass
+    return str(s)
 
 
 class Connection(object):
-    def __init__(self, endpoint, socktype, info={}):
+    def __init__(self, endpoint, socktype, info=None):
         log("Connection%s", (endpoint, socktype, info))
         self.endpoint = endpoint
         try:
             assert isinstance(endpoint, (tuple, list))
             self.target = ":".join(str(x) for x in endpoint)
-        except:
+        except Exception:
             self.target = str(endpoint)
         self.socktype_wrapped = socktype
         self.socktype = socktype
-        self.info = info
+        self.info = info or {}
         self.input_bytecount = 0
         self.input_readcount = 0
         self.output_bytecount = 0
@@ -228,7 +226,7 @@ class Connection(object):
 # client.py relies on self.filename to locate the unix domain
 # socket (if it exists)
 class TwoFileConnection(Connection):
-    def __init__(self, writeable, readable, abort_test=None, target=None, socktype="", close_cb=None, info={}):
+    def __init__(self, writeable, readable, abort_test=None, target=None, socktype="", close_cb=None, info=None):
         Connection.__init__(self, target, socktype, info)
         self._writeable = writeable
         self._readable = readable
@@ -295,10 +293,10 @@ class TwoFileConnection(Connection):
 TCP_SOCKTYPES = ("tcp", "ssl", "ws", "wss")
 
 class SocketConnection(Connection):
-    def __init__(self, socket, local, remote, target, socktype, info={}):
-        log("SocketConnection%s", (socket, local, remote, target, socktype, info))
+    def __init__(self, sock, local, remote, target, socktype, info=None):
+        log("SocketConnection%s", (sock, local, remote, target, socktype, info))
         Connection.__init__(self, target, socktype, info)
-        self._socket = socket
+        self._socket = sock
         self.local = local
         self.remote = remote
         self.protocol_type = "socket"
@@ -356,12 +354,15 @@ class SocketConnection(Connection):
         except EOFError:
             log("%s.close()", s, exc_info=True)
         except IOError as e:
-            if isinstance(e, CLOSED_EXCEPTIONS):
+            if self.error_is_closed(e):
                 log("%s.close() already closed!", s)
             else:
                 raise
         log("%s.close() done", self)
 
+    def error_is_closed(self, e):
+        return isinstance(e, CLOSED_EXCEPTIONS)
+
     def __repr__(self):
         if self.remote:
             return "%s %s: %s <- %s" % (
@@ -609,14 +610,15 @@ def log_new_connection(conn, socket_info=""):
         conn, socket_info, type(conn), sock, sockname, address, peername)
     if peername:
         frominfo = pretty_socket(peername)
-        info_msg = "New %s connection received from %s" % (socktype, frominfo)
+        log.info("New %s connection received", socktype)
+        log.info(" from '%s'", frominfo)
         if socket_info:
-            info_msg += " on %s" % (pretty_socket(socket_info),)
+            log.info(" on %s", pretty_socket(socket_info))
     elif socktype=="unix-domain":
         frominfo = sockname
-        info_msg = "New %s connection received on %s" % (socktype, frominfo)
+        log.info("New %s connection received", socktype)
+        log.info(" on '%s'", frominfo)
     else:
-        info_msg = "New %s connection received"
+        log.info("New %s connection received")
         if socket_info:
-            info_msg += " on %s" % (pretty_socket(socket_info),)
-    log.info(info_msg)
+            log.info(" on %s", pretty_socket(socket_info))
