@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2013-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,7 +9,9 @@
 import socket
 import sys
 
+from xpra.os_util import WIN32, OSX, PYTHON3
 from xpra.log import Logger
+
 log = Logger("network", "util")
 
 
@@ -23,7 +25,7 @@ def import_netifaces():
             log("netifaces loaded sucessfully")
             _netifaces = netifaces
             netifaces_version = netifaces.version        #@UndefinedVariable
-        except:
+        except ImportError:
             _netifaces = False
             log.warn("Warning: the python netifaces package is missing")
     return _netifaces
@@ -89,7 +91,7 @@ def get_gateways():
                 continue
             gateways[AF_NAMES.get(family, family)] = gws
         return gateways
-    except:
+    except Exception:
         log("get_gateways() failed", exc_info=True)
         return {}
 
@@ -191,7 +193,7 @@ def get_iface(ip):
                 log.error("incorrect ip or mask: %s/%s", test_ip, mask)
             match = True
             try:
-                for i in [0,1,2,3]:
+                for i in (0,1,2,3):
                     mask_part = int(mask_parts[i])
                     ip_part = int(ip_parts[i]) & mask_part
                     test_ip_part = int(test_ip_parts[i]) & mask_part
@@ -210,8 +212,25 @@ def get_iface(ip):
 # http://code.activestate.com/recipes/442490/
 if_nametoindex = None
 if_indextoname = None
-from xpra.os_util import WIN32, OSX
-if not WIN32:
+
+if WIN32:
+    def int_if_nametoindex(iface):
+        #IPv6 addresses give us the interface as a string:
+        #fe80:....%11, so try to convert "11" into 11
+        try:
+            return int(iface)
+        except (TypeError, ValueError):
+            return None
+    if_nametoindex = int_if_nametoindex
+elif PYTHON3:
+    if_nametoindex = socket.if_nametoindex
+    def socket_if_indextoname(index):
+        if index<0:
+            return None
+        return socket.if_indextoname(index)
+    if_indextoname = socket_if_indextoname
+
+else:
     library = "libc.so.6"
     if OSX:
         library = "/usr/lib/libc.dylib"
@@ -237,19 +256,13 @@ if not WIN32:
         _libc.if_indextoname.argtypes = [c_uint, c_char_p]
         _libc.if_nametoindex.restype = c_uint
         _libc.if_nametoindex.argtypes = [c_char_p]
-        def if_nametoindex(interfaceName):
+        def libc_if_nametoindex(interfaceName):
             return _libc.if_nametoindex(create_string_buffer(interfaceName.encode()))
-        def if_indextoname(index):
+        def libc_if_indextoname(index):
             s = create_string_buffer(b'\000' * 256)
             return _libc.if_indextoname(c_uint(index), s)
-else:
-    def if_nametoindex(iface):
-        #IPv6 addresses give us the interface as a string:
-        #fe80:....%11, so try to convert "11" into 11
-        try:
-            return int(iface)
-        except:
-            return None
+        if_nametoindex = libc_if_nametoindex
+        if_indextoname = libc_if_indextoname
 
 
 net_sys_config = None
@@ -333,20 +346,22 @@ def get_net_config():
     return config
 
 
-def get_ssl_info():
+def get_ssl_info(show_constants=False):
     try:
         import ssl
     except ImportError as e:
         log("no ssl: %s", e)
         return {}
-    protocols = dict((k,int(getattr(ssl, k))) for k in dir(ssl) if k.startswith("PROTOCOL_"))
-    ops = dict((k,int(getattr(ssl, k))) for k in dir(ssl) if k.startswith("OP_"))
-    vers = dict((k,int(getattr(ssl, k))) for k in dir(ssl) if k.startswith("VERIFY_"))
-    info = {
-            "protocols"    : protocols,
-            "options"    : ops,
-            "verify"    : vers,
-            }
+    info = {}
+    if show_constants:
+        protocols = dict((k,int(getattr(ssl, k))) for k in dir(ssl) if k.startswith("PROTOCOL_"))
+        ops = dict((k,int(getattr(ssl, k))) for k in dir(ssl) if k.startswith("OP_"))
+        vers = dict((k,int(getattr(ssl, k))) for k in dir(ssl) if k.startswith("VERIFY_"))
+        info.update({
+                "protocols"    : protocols,
+                "options"    : ops,
+                "verify"    : vers,
+                })
     for k,name in {
                     "HAS_ALPN"                : "alpn",
                     "HAS_ECDH"                : "ecdh",
@@ -372,7 +387,7 @@ def get_ssl_info():
 def get_network_caps():
     try:
         from xpra.platform.features import MMAP_SUPPORTED
-    except:
+    except ImportError:
         MMAP_SUPPORTED = False
     from xpra.net.digest import get_digests
     from xpra.net.crypto import get_crypto_caps
@@ -380,7 +395,7 @@ def get_network_caps():
     from xpra.net.packet_encoding import get_enabled_encoders, get_packet_encoding_caps
     digests = get_digests()
     #"hmac" is the legacy name, "xor" and "des" should not be used for salt:
-    salt_digests = [x for x in digests if x not in ("hmac", "xor", "des")]
+    salt_digests = tuple(x for x in digests if x not in ("hmac", "xor", "des"))
     caps = {
                 "digest"                : digests,
                 "salt-digest"           : salt_digests,
@@ -449,7 +464,7 @@ def main():
                                 }[addr]
                             print(" * %s:     %s" % (stype, ip))
                             if POSIX:
-                                from xpra.server.socket_util import create_tcp_socket
+                                from xpra.net.socket_util import create_tcp_socket
                                 try:
                                     sock = create_tcp_socket(ip, 0)
                                     sockfd = sock.fileno()
@@ -478,9 +493,9 @@ def main():
                     s += bytestostr(x)
                     lastx = x
                 return s
-            if type(v)==bytes:
+            if isinstance(v, bytes):
                 v = bytestostr(v)
-            if type(v)==str and v.startswith("v"):
+            if isinstance(v, str) and v.startswith("v"):
                 return v[1:]
             return str(v)
 
@@ -488,12 +503,9 @@ def main():
         for gt,idefs in get_gateways().items():
             print("* %s" % gt)      #ie: "INET"
             for i, idef in enumerate(idefs):
-                try:
-                    if isinstance(idef, (list, tuple)):
-                        print(" [%i]           %s" % (i, csv(idef)))
-                        continue
-                except:
-                    print(" [%i]           %s" % (i, idef))
+                if isinstance(idef, (list, tuple)):
+                    print(" [%i]           %s" % (i, csv(idef)))
+                    continue
 
         print("")
         print("Protocol Capabilities:")
@@ -516,7 +528,7 @@ def main():
 
         print("")
         print("SSL:")
-        print_nested_dict(get_ssl_info())
+        print_nested_dict(get_ssl_info(True))
 
         try:
             from xpra.net.crypto import crypto_backend_init, get_crypto_caps
