@@ -75,10 +75,10 @@ def force_flush_queue(q):
         try:
             while q.qsize()>0:
                 q.read(False)
-        except:
+        except Exception:
             pass
         q.put_nowait(None)
-    except:
+    except Exception:
         pass
 
 
@@ -168,7 +168,7 @@ class Protocol(object):
         #initial value which may get increased by client/server after handshake:
         self.max_packet_size = 4*1024*1024
         self.abs_max_packet_size = 256*1024*1024
-        self.large_packets = [b"hello", b"window-metadata", b"sound-data", b"notify_show"]
+        self.large_packets = [b"hello", b"window-metadata", b"sound-data", b"notify_show", b"setting-change"]
         self.send_aliases = {}
         self.receive_aliases = {}
         self._log_stats = None          #None here means auto-detect
@@ -214,6 +214,10 @@ class Protocol(object):
         self.enable_encoder(self.encoder)
 
 
+    def is_closed(self):
+        return self._closed
+
+
     def wait_for_io_threads_exit(self, timeout=None):
         io_threads = [x for x in (self._read_thread, self._write_thread) if x is not None]
         for t in io_threads:
@@ -263,7 +267,8 @@ class Protocol(object):
         pass
 
     def parse_remote_caps(self, caps):
-        self.send_aliases = caps.dictget("aliases")
+        for k,v in caps.dictget("aliases", {}).items():
+            self.send_aliases[bytestostr(k)] = v
 
     def get_info(self, alias_info=True):
         info = {
@@ -277,7 +282,7 @@ class Protocol(object):
             info["compressor"] = compression.get_compressor_name(self._compress)
         e = self._encoder
         if e:
-            if self._encoder==self.noencode:
+            if self._encoder==self.noencode:        #pylint: disable=comparison-with-callable
                 info["encoder"] = "noencode"
             else:
                 info["encoder"] = packet_encoding.get_encoder_name(self._encoder)
@@ -288,7 +293,7 @@ class Protocol(object):
         if c:
             try:
                 info.update(self._conn.get_info())
-            except:
+            except Exception:
                 log.error("error collecting connection information on %s", self._conn, exc_info=True)
         #add stats to connection info:
         info.setdefault("input", {}).update({
@@ -338,7 +343,8 @@ class Protocol(object):
             log("send_now(%s ...) connection is closed already, not sending", packet[0])
             return
         log("send_now(%s ...)", packet[0])
-        assert self._get_packet_cb is None, "cannot use send_now when a packet source exists! (set to %s)" % self._get_packet_cb
+        if self._get_packet_cb:
+            raise Exception("cannot use send_now when a packet source exists! (set to %s)" % self._get_packet_cb)
         tmp_queue = [packet]
         def packet_cb():
             self._get_packet_cb = None
@@ -349,7 +355,7 @@ class Protocol(object):
         self._get_packet_cb = packet_cb
         self.source_has_more()
 
-    def source_has_more(self):
+    def source_has_more(self):      #pylint: disable=method-hidden
         shm = self._source_has_more
         if not shm or self._closed:
             return
@@ -424,7 +430,7 @@ class Protocol(object):
             else:
                 #the xpra packet header:
                 #(WebSocketProtocol may also add a websocket header too)
-                header = self.make_chunk_header(packet_type, proto_flags, level, index, actual_size)
+                header = self.make_chunk_header(packet_type, proto_flags, level, index, payload_size)
                 if actual_size<PACKET_JOIN_SIZE:
                     if not isinstance(data, JOIN_TYPES):
                         data = memoryview_to_bytes(data)
@@ -433,7 +439,7 @@ class Protocol(object):
                     items.append(header)
                     items.append(data)
         #WebSocket header may be added here:
-        frame_header = self.make_frame_header(packet_type, items)
+        frame_header = self.make_frame_header(packet_type, items)       #pylint: disable=assignment-from-none
         if frame_header:
             item0 = items[0]
             if len(item0)<PACKET_JOIN_SIZE:
@@ -594,9 +600,13 @@ class Protocol(object):
         #now the main packet (or what is left of it):
         packet_type = packet[0]
         self.output_stats[packet_type] = self.output_stats.get(packet_type, 0)+1
-        if USE_ALIASES and self.send_aliases and packet_type in self.send_aliases:
-            #replace the packet type with the alias:
-            packet[0] = self.send_aliases[packet_type]
+        if USE_ALIASES:
+            alias = self.send_aliases.get(packet_type)
+            if alias:
+                #replace the packet type with the alias:
+                packet[0] = alias
+            else:
+                log("packet type send alias not found for '%s'", packet_type)
         try:
             main_packet, proto_flags = self._encoder(packet)
         except Exception:
@@ -675,7 +685,7 @@ class Protocol(object):
         if start_cb:
             try:
                 start_cb(conn.output_bytecount)
-            except:
+            except Exception:
                 if not self._closed:
                     log.error("Error on write start callback %s", start_cb, exc_info=True)
         self.write_buffers(buf_data, fail_cb, synchronous)
@@ -686,7 +696,7 @@ class Protocol(object):
         if end_cb:
             try:
                 end_cb(self._conn.output_bytecount)
-            except:
+            except Exception:
                 if not self._closed:
                     log.error("Error on write end callback %s", end_cb, exc_info=True)
         return True
@@ -783,8 +793,7 @@ class Protocol(object):
             self.start_read_parser_thread()
         self._read_queue.put(data)
         #from now on, take shortcut:
-        if self._read_queue_put==self.read_queue_put:
-            self._read_queue_put = self._read_queue.put
+        self._read_queue_put = self._read_queue.put
 
     def start_read_parser_thread(self):
         self._read_parser_thread = start_thread(self._read_parse_thread_loop, "parse", daemon=True)
@@ -915,7 +924,7 @@ class Protocol(object):
 
                 #decrypt if needed:
                 if self.cipher_in:
-                    if not (protocol_flags & FLAGS_CIPHER):
+                    if not protocol_flags & FLAGS_CIPHER:
                         self.invalid("unencrypted packet dropped", data)
                         return
                     cryptolog("received %i %s encrypted bytes with %i padding",
@@ -925,7 +934,7 @@ class Protocol(object):
                         def debug_str(s):
                             try:
                                 return hexstr(bytearray(s))
-                            except:
+                            except Exception:
                                 return csv(tuple(s))
                         # pad byte value is number of padding bytes added
                         padtext = pad(self.cipher_in_padding, padding_size)
@@ -934,7 +943,8 @@ class Protocol(object):
                         else:
                             actual_padding = data[-padding_size:]
                             cryptolog.warn("Warning: %s decryption failed: invalid padding", self.cipher_in_name)
-                            cryptolog(" data does not end with %s padding bytes %s", self.cipher_in_padding, debug_str(padtext))
+                            cryptolog(" data does not end with %s padding bytes %s",
+                                      self.cipher_in_padding, debug_str(padtext))
                             cryptolog(" but with %s (%s)", debug_str(actual_padding), type(data))
                             cryptolog(" decrypted data: %s", debug_str(data[:128]))
                             self._internal_error("%s encryption padding error - wrong key?" % self.cipher_in_name)
@@ -1005,9 +1015,10 @@ class Protocol(object):
                     raw_packets = {}
 
                 packet_type = packet[0]
-                if self.receive_aliases and isinstance(packet_type, int) and packet_type in self.receive_aliases:
+                if self.receive_aliases and isinstance(packet_type, int):
                     packet_type = self.receive_aliases.get(packet_type)
-                    packet[0] = packet_type
+                    if packet_type:
+                        packet[0] = packet_type
                 self.input_stats[packet_type] = self.output_stats.get(packet_type, 0)+1
                 if LOG_RAW_PACKET_SIZE:
                     log("%s: %i bytes", packet_type, HEADER_SIZE + payload_size)
@@ -1017,7 +1028,7 @@ class Protocol(object):
                 self._process_packet_cb(self, packet)
                 packet = None
 
-    def flush_then_close(self, last_packet, done_callback=None):
+    def flush_then_close(self, last_packet, done_callback=None):    #pylint: disable=method-hidden
         """ Note: this is best effort only
             the packet may not get sent.
 
@@ -1027,6 +1038,10 @@ class Protocol(object):
             we wait again for the queue to flush,
             then no matter what, we close the connection and stop the threads.
         """
+        def closing_already(last_packet, done_callback=None):
+            log("flush_then_close%s had already been called, this new request has been ignored",
+                (last_packet, done_callback))
+        self.flush_then_close = closing_already
         log("flush_then_close(%s, %s) closed=%s", last_packet, done_callback, self._closed)
         def done():
             log("flush_then_close: done, callback=%s", done_callback)
@@ -1044,7 +1059,7 @@ class Protocol(object):
                     log("flush_then_close: queue still busy, closing without sending the last packet")
                     try:
                         self._write_lock.release()
-                    except:
+                    except Exception:
                         pass
                     self.close()
                     done()
@@ -1059,7 +1074,7 @@ class Protocol(object):
                     self.close()
                     try:
                         self._write_lock.release()
-                    except:
+                    except Exception:
                         pass
                     done()
                 def wait_for_packet_sent():
@@ -1076,7 +1091,9 @@ class Protocol(object):
                     if wait_for_packet_sent():
                         #check again every 100ms
                         self.timeout_add(100, wait_for_packet_sent)
-                self._add_chunks_to_queue(last_packet[0], chunks, start_send_cb=None, end_send_cb=packet_queued, synchronous=False, more=False)
+                self._add_chunks_to_queue(last_packet[0], chunks,
+                                          start_send_cb=None, end_send_cb=packet_queued,
+                                          synchronous=False, more=False)
                 #just in case wait_for_packet_sent never fires:
                 self.timeout_add(5*1000, close_and_release)
 
@@ -1128,7 +1145,7 @@ class Protocol(object):
                          std_unit(self.input_packetcount), std_unit_dec(c.input_bytecount),
                          std_unit(self.output_packetcount), std_unit_dec(c.output_bytecount)
                          )
-            except:
+            except Exception:
                 log.error("error closing %s", c, exc_info=True)
         self.terminate_queue_threads()
         self.idle_add(self.clean)
