@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2018-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -7,18 +7,23 @@ import sys
 import time
 import os
 import socket
-
 from subprocess import PIPE, Popen
 
-from xpra.log import Logger
-log = Logger("network", "ssh")
-
-from xpra.scripts.main import InitException, InitExit, shellquote
+from xpra.scripts.main import InitException, InitExit, shellquote, host_target_string
 from xpra.platform.paths import get_xpra_command, get_ssh_known_hosts_files
+from xpra.platform import get_username
+from xpra.scripts.config import TRUE_OPTIONS
 from xpra.net.bytestreams import SocketConnection, SOCKET_TIMEOUT, ConnectionClosedException
 from xpra.exit_codes import EXIT_SSH_KEY_FAILURE, EXIT_SSH_FAILURE
-from xpra.os_util import bytestostr, osexpand, monotonic_time, setsid, nomodule_context, umask_context, WIN32, OSX, POSIX
+from xpra.os_util import (
+    bytestostr, osexpand, monotonic_time, load_binary_file,
+    setsid, nomodule_context, umask_context,
+    is_WSL, WIN32, OSX, POSIX,
+    )
 from xpra.util import envint, envbool, nonl, engs
+from xpra.log import Logger
+
+log = Logger("network", "ssh")
 
 INITENV_COMMAND = os.environ.get("XPRA_INITENV_COMMAND", "xpra initenv")
 SSH_DEBUG = envbool("XPRA_SSH_DEBUG", False)
@@ -39,22 +44,6 @@ PASSWORD_RETRY = envint("XPRA_SSH_PASSWORD_RETRY", 2)
 assert PASSWORD_RETRY>=0
 
 
-def ssh_target_string(display_desc):
-    target = "ssh://"
-    username = display_desc.get("username")
-    if username:
-        target += "%s@" % username
-    host = display_desc.get("host")
-    target += host
-    ssh_port = display_desc.get("ssh-port")
-    if ssh_port:
-        target += ":%i" % ssh_port
-    display = display_desc.get("display")
-    if display and display.startswith(":"):
-        display = display[1:]
-    target += "/%s" % (display or "")
-    return target
-
 
 def keymd5(k):
     import binascii
@@ -79,6 +68,9 @@ def exec_dialog_subprocess(cmd):
             env["XPRA_LOG_TO_FILE"] = "0"
             kwargs["env"] = env
         proc = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE, **kwargs)
+        if is_WSL():
+            #WSL needs to wait before calling communicate?
+            proc.wait()
         stdout, stderr = proc.communicate()
         log("exec_dialog_subprocess(%s)", cmd)
         if stderr:
@@ -96,26 +88,28 @@ def dialog_pass(title="Password Input", prompt="enter password", icon=""):
     cmd = get_xpra_command()+["_pass", nonl(title), nonl(prompt), icon]
     return exec_dialog_subprocess(cmd)
 
-def dialog_confirm(title, prompt, qinfo="", icon="", buttons=[("OK", 1)]):
+def dialog_confirm(title, prompt, qinfo="", icon="", buttons=(("OK", 1),)):
     cmd = get_xpra_command()+["_dialog", nonl(title), nonl(prompt), nonl("\\n".join(qinfo)), icon]
     for label, code in buttons:
         cmd.append(nonl(label))
         cmd.append(str(code))
+    log("dialog_confirm%s", (title, prompt, qinfo, icon, buttons))
     return exec_dialog_subprocess(cmd)
 
-def confirm_key(info=[]):
+def confirm_key(info=()):
     if SKIP_UI:
         return False
     from xpra.platform.paths import get_icon_filename
     from xpra.os_util import use_tty
     if not use_tty():
-        icon = get_icon_filename("authentication", "png")
+        icon = get_icon_filename("authentication", "png") or ""
         prompt = "Are you sure you want to continue connecting?"
         code, out = dialog_confirm("Confirm Key", prompt, info, icon, buttons=[("yes", 200), ("NO", 201)])
         log.debug("dialog output: '%s', return code=%s", nonl(out), code)
         r = code==200
         log.info("host key %sconfirmed", ["not ", ""][r])
         return r
+    log("confirm_key(%s) will use stdin prompt", nonl(info))
     prompt = "Are you sure you want to continue connecting (yes/NO)? "
     sys.stderr.write(os.linesep.join(info)+os.linesep+prompt)
     v = sys.stdin.readline().rstrip(os.linesep)
@@ -127,7 +121,7 @@ def input_pass(prompt):
     from xpra.platform.paths import get_icon_filename
     from xpra.os_util import use_tty
     if not use_tty():
-        icon = get_icon_filename("authentication", "png")
+        icon = get_icon_filename("authentication", "png") or ""
         code, out = dialog_pass("Password Input", prompt, icon)
         log.debug("pass dialog output return code=%s", code)
         if code!=0:
@@ -139,8 +133,8 @@ def input_pass(prompt):
 
 class SSHSocketConnection(SocketConnection):
 
-    def __init__(self, ssh_channel, sock, target, info={}):
-        SocketConnection.__init__(self, ssh_channel, sock.getsockname(), sock.getpeername(), target, "ssh", info)
+    def __init__(self, ssh_channel, sock, sockname, peername, target, info=None):
+        SocketConnection.__init__(self, ssh_channel, sockname, peername, target, "ssh", info)
         self._raw_socket = sock
 
     def start_stderr_reader(self):
@@ -164,9 +158,13 @@ class SSHSocketConnection(SocketConnection):
                 log.warn(" %s", e)
 
     def peek(self, n):
+        if not self._raw_socket:
+            return None
         return self._raw_socket.recv(n, socket.MSG_PEEK)
 
     def get_socket_info(self):
+        if not self._raw_socket:
+            return {}
         return self.do_get_socket_info(self._raw_socket)
 
     def get_info(self):
@@ -180,27 +178,165 @@ class SSHSocketConnection(SocketConnection):
         return i
 
 
+class SSHProxyCommandConnection(SSHSocketConnection):
+    def __init__(self, ssh_channel, peername, target, info):
+        SSHSocketConnection.__init__(self, ssh_channel, None, None, peername, target, info)
+        self.process = None
+
+    def error_is_closed(self, e):
+        p = self.process
+        if p:
+            #if the process has terminated,
+            #then the connection must be closed:
+            if p[0].poll() is not None:
+                return True
+        return SSHSocketConnection.error_is_closed(self, e)
+
+    def get_socket_info(self):
+        p = self.process
+        if not p:
+            return {}
+        proc, _ssh, cmd = p
+        return {
+            "process" : {
+                "pid"       : proc.pid,
+                "returncode": proc.returncode,
+                "command"   : cmd,
+                }
+            }
+
+
 def ssh_paramiko_connect_to(display_desc):
     #plain socket attributes:
     dtype = display_desc["type"]
     host = display_desc["host"]
     port = display_desc.get("ssh-port", 22)
-    ipv6 = display_desc.get("ipv6", False)
-    from xpra.scripts.main import socket_connect
-    sock = socket_connect(dtype, host, port, ipv6)
     #ssh and command attributes:
-    username = display_desc.get("username")
-    if not username:
-        import getpass
-        username = getpass.getuser()
+    username = display_desc.get("username") or get_username()
+    if "proxy_host" in display_desc:
+        display_desc.setdefault("proxy_username", get_username())
     password = display_desc.get("password")
-    target = ssh_target_string(display_desc)
     remote_xpra = display_desc["remote_xpra"]
     proxy_command = display_desc["proxy_command"]       #ie: "_proxy_start"
     socket_dir = display_desc.get("socket_dir")
+    display = display_desc.get("display")
     display_as_args = display_desc["display_as_args"]   #ie: "--start=xterm :10"
+    keyfiles = None
+    socket_info = {
+            "host"  : host,
+            "port"  : port,
+            }
     with nogssapi_context():
-        return do_ssh_paramiko_connect_to(sock, host, port, username, password, proxy_command, remote_xpra, socket_dir, display_as_args, target)
+        from paramiko import SSHConfig, ProxyCommand
+        ssh_config = SSHConfig()
+        user_config_file = os.path.expanduser("~/.ssh/config")
+        sock = None
+        host_config = None
+        if os.path.exists(user_config_file):
+            with open(user_config_file) as f:
+                ssh_config.parse(f)
+            host_config = ssh_config.lookup(host)
+            if host_config:
+                host = host_config.get("hostname", host)
+                username = host_config.get("username", username)
+                port = host_config.get("port", port)
+                proxycommand = host_config.get("proxycommand")
+                keyfiles = host_config.get("identityfile")
+                if proxycommand:
+                    sock = ProxyCommand(proxycommand)
+                    from xpra.child_reaper import getChildReaper
+                    cmd = getattr(sock, "cmd", [])
+                    getChildReaper().add_process(sock.process, "paramiko-ssh-client", cmd, True, True)
+                    log("found proxycommand='%s' for host '%s'", proxycommand, host)
+                    from paramiko.client import SSHClient
+                    ssh_client = SSHClient()
+                    ssh_client.load_system_host_keys()
+                    ssh_client.connect(host, port, sock=sock)
+                    transport = ssh_client.get_transport()
+                    do_ssh_paramiko_connect_to(transport, host,
+                                               username, password,
+                                               host_config or ssh_config.lookup("*"),
+                                               keyfiles)
+                    chan = paramiko_run_remote_xpra(transport, proxy_command, remote_xpra, socket_dir, display_as_args)
+                    peername = (host, port)
+                    conn = SSHProxyCommandConnection(chan, peername, peername, socket_info)
+                    conn.target = host_target_string("ssh", username, host, port, display)
+                    conn.timeout = SOCKET_TIMEOUT
+                    conn.start_stderr_reader()
+                    conn.process = (sock.process, "ssh", cmd)
+                    from xpra.net import bytestreams
+                    from paramiko.ssh_exception import ProxyCommandFailure
+                    bytestreams.CLOSED_EXCEPTIONS = tuple(list(bytestreams.CLOSED_EXCEPTIONS)+[ProxyCommandFailure])
+                    return conn
+        from xpra.scripts.main import socket_connect
+        from paramiko.transport import Transport
+        from paramiko import SSHException
+        if "proxy_host" in display_desc:
+            proxy_host = display_desc["proxy_host"]
+            proxy_port = display_desc.get("proxy_port", 22)
+            proxy_username = display_desc.get("proxy_username", username)
+            proxy_password = display_desc.get("proxy_password", password)
+            sock = socket_connect(dtype, proxy_host, proxy_port)
+            middle_transport = Transport(sock)
+            middle_transport.use_compression(False)
+            try:
+                middle_transport.start_client()
+            except SSHException as e:
+                log("start_client()", exc_info=True)
+                raise InitException("SSH negotiation failed: %s" % e)
+            proxy_host_config = ssh_config.lookup(host)
+            do_ssh_paramiko_connect_to(middle_transport, proxy_host,
+                                       proxy_username, proxy_password,
+                                       proxy_host_config or ssh_config.lookup("*"),
+                                       keyfiles)
+            log("Opening proxy channel")
+            chan_to_middle = middle_transport.open_channel("direct-tcpip", (host, port), ('localhost', 0))
+
+            transport = Transport(chan_to_middle)
+            transport.use_compression(False)
+            try:
+                transport.start_client()
+            except SSHException as e:
+                log("start_client()", exc_info=True)
+                raise InitException("SSH negotiation failed: %s" % e)
+            do_ssh_paramiko_connect_to(transport, host,
+                                       username, password,
+                                       host_config or ssh_config.lookup("*"),
+                                       keyfiles)
+            chan = paramiko_run_remote_xpra(transport, proxy_command, remote_xpra, socket_dir, display_as_args)
+
+            peername = (host, port)
+            conn = SSHProxyCommandConnection(chan, peername, peername, socket_info)
+            conn.target = "%s via %s" % (
+                host_target_string("ssh", username, host, port, display),
+                host_target_string("ssh", proxy_username, proxy_host, proxy_port, None),
+                )
+            conn.timeout = SOCKET_TIMEOUT
+            conn.start_stderr_reader()
+            return conn
+
+        #plain TCP connection to the server,
+        #we open it then give the socket to paramiko:
+        sock = socket_connect(dtype, host, port)
+        sockname = sock.getsockname()
+        peername = sock.getpeername()
+        log("paramiko socket_connect: sockname=%s, peername=%s", sockname, peername)
+        transport = Transport(sock)
+        transport.use_compression(False)
+        try:
+            transport.start_client()
+        except SSHException as e:
+            log("start_client()", exc_info=True)
+            raise InitException("SSH negotiation failed: %s" % e)
+        do_ssh_paramiko_connect_to(transport, host, username, password,
+                                   host_config or ssh_config.lookup("*"),
+                                   keyfiles)
+        chan = paramiko_run_remote_xpra(transport, proxy_command, remote_xpra, socket_dir, display_as_args)
+        conn = SSHSocketConnection(chan, sock, sockname, peername, (host, port), socket_info)
+        conn.target = host_target_string("ssh", username, host, port, display)
+        conn.timeout = SOCKET_TIMEOUT
+        conn.start_stderr_reader()
+        return conn
 
 
 #workaround incompatibility between paramiko and gssapi:
@@ -210,17 +346,14 @@ class nogssapi_context(nomodule_context):
         nomodule_context.__init__(self, "gssapi")
 
 
-def do_ssh_paramiko_connect_to(sock, host, port, username, password, proxy_command, remote_xpra, socket_dir, display_as_args, target):
-    from paramiko import SSHException, Transport, Agent, RSAKey, PasswordRequiredException
+def do_ssh_paramiko_connect_to(transport, host, username, password, host_config=None, keyfiles=None):
+    from paramiko import SSHException, PasswordRequiredException
+    from paramiko.agent import Agent
     from paramiko.hostkeys import HostKeys
-    transport = Transport(sock)
-    transport.use_compression(False)
+    log("do_ssh_paramiko_connect_to%s", (transport, host, username, password, host_config, keyfiles))
     log("SSH transport %s", transport)
-    try:
-        transport.start_client()
-    except SSHException as e:
-        log("start_client()", exc_info=True)
-        raise InitException("SSH negotiation failed: %s" % e)
+    if not keyfiles:
+        keyfiles = [osexpand(os.path.join("~/", ".ssh", keyfile)) for keyfile in ("id_rsa", "id_dsa")]
 
     host_key = transport.get_remote_server_key()
     assert host_key, "no remote server key"
@@ -250,7 +383,33 @@ def do_ssh_paramiko_connect_to(sock, host, port, username, password, proxy_comma
             assert host_key
             log("%s host key '%s' OK for host '%s'", keyname(), keymd5(host_key), host)
         else:
-            if known_host_key:
+            dnscheck = ""
+            if host_config:
+                verifyhostkeydns = host_config.get("verifyhostkeydns")
+                if verifyhostkeydns and verifyhostkeydns.lower() in TRUE_OPTIONS:
+                    try:
+                        from xpra.net.sshfp import check_host_key
+                        dnscheck = check_host_key(host, host_key)
+                    except ImportError as e:
+                        log("verifyhostkeydns failed", exc_info=True)
+                        log.warn("Warning: cannot check SSHFP DNS records")
+                        log.warn(" %s", e)
+            log("dnscheck=%s", dnscheck)
+            def adddnscheckinfo(q):
+                if dnscheck is not True:
+                    if dnscheck:
+                        q += [
+                            "SSHFP validation failed:",
+                            dnscheck
+                            ]
+                    else:
+                        q += [
+                            "SSHFP validation failed"
+                            ]
+            if dnscheck is True:
+                #DNSSEC provided a matching record
+                log.info("found a valid SSHFP record for host %s", host)
+            elif known_host_key:
                 log.warn("Warning: SSH server key mismatch")
                 qinfo = [
 "WARNING: REMOTE HOST IDENTIFICATION HAS CHANGED!",
@@ -260,6 +419,7 @@ def do_ssh_paramiko_connect_to(sock, host, port, username, password, proxy_comma
 "The fingerprint for the %s key sent by the remote host is" % keyname(),
 keymd5(host_key),
 ]
+                adddnscheckinfo(qinfo)
                 if VERIFY_STRICT:
                     log.warn("Host key verification failed.")
                     #TODO: show alert with no option to accept key
@@ -281,12 +441,13 @@ keymd5(host_key),
                 if not keys:
                     log.warn("Warning: unknown SSH host")
                 else:
-                    log.warn("Warning: unknown %s SSH host key", keyname())                        
+                    log.warn("Warning: unknown %s SSH host key", keyname())
                 qinfo = [
                     "The authenticity of host '%s' can't be established." % (host,),
                     "%s key fingerprint is" % keyname(),
                     keymd5(host_key),
                     ]
+                adddnscheckinfo(qinfo)
                 if not confirm_key(qinfo):
                     transport.close()
                     raise InitExit(EXIT_SSH_KEY_FAILURE, "Unknown SSH host '%s'" % host)
@@ -339,27 +500,47 @@ keymd5(host_key),
                 log.info("agent authentication failed, tried %i key%s", len(agent_keys), engs(agent_keys))
 
     def auth_publickey():
-        log("trying public key authentication")
-        for keyfile in ("id_rsa", "id_dsa"):
-            keyfile_path = osexpand(os.path.join("~/", ".ssh", keyfile))
+        log("trying public key authentication using %s", keyfiles)
+        for keyfile_path in keyfiles:
             if not os.path.exists(keyfile_path):
                 log("no keyfile at '%s'", keyfile_path)
                 continue
+            log("trying '%s'", keyfile_path)
+            key_data = load_binary_file(keyfile_path)
+            if key_data and key_data.find(b"BEGIN OPENSSH PRIVATE KEY")>=0:
+                log.warn("Warning: private key '%s'", keyfile_path)
+                log.warn(" this file seems to be using OpenSSH's own format")
+                log.warn(" please convert it to something more standard (ie: PEM)")
+                log.warn(" so it can be used with the paramiko backend")
+                log.warn(" or switch to the OpenSSH backend with '--ssh=ssh'")
             key = None
-            try:
-                key = RSAKey.from_private_key_file(keyfile_path)
-            except PasswordRequiredException:
-                log("%s keyfile requires a passphrase", keyfile_path)
-                passphrase = input_pass("please enter the passphrase for %s:" % (keyfile_path,))
-                if passphrase:
-                    try:
-                        key = RSAKey.from_private_key_file(keyfile_path, passphrase)
-                    except SSHException as e:
-                        log("from_private_key_file", exc_info=True)
-                        log.info("cannot load key from file '%s':", keyfile_path)
-                        log.info(" %s", e)
+            import paramiko
+            for pkey_classname in ("RSA", "DSS", "ECDSA", "Ed25519"):
+                pkey_class = getattr(paramiko, "%sKey" % pkey_classname, None)
+                if pkey_class is None:
+                    log("no %s key type", pkey_classname)
+                    continue
+                log("trying to load as %s", pkey_classname)
+                try:
+                    key = pkey_class.from_private_key_file(keyfile_path)
+                    log.info("loaded %s private key from '%s'", pkey_classname, keyfile_path)
+                    break
+                except PasswordRequiredException as e:
+                    log("%s keyfile requires a passphrase; %s", keyfile_path, e)
+                    passphrase = input_pass("please enter the passphrase for %s:" % (keyfile_path,))
+                    if passphrase:
+                        try:
+                            key = pkey_class.from_private_key_file(keyfile_path, passphrase)
+                            log.info("loaded %s private key from '%s'", pkey_classname, keyfile_path)
+                        except SSHException as e:
+                            log("from_private_key_file", exc_info=True)
+                            log.info("cannot load key from file '%s':", keyfile_path)
+                            log.info(" %s", e)
+                    break
+                except Exception as e:
+                    log("auth_publickey() loading as %s", pkey_classname, exc_info=True)
             if key:
-                log("auth_publickey using %s: %s", keyfile_path, keymd5(key))
+                log("auth_publickey using %s as %s: %s", keyfile_path, pkey_classname, keymd5(key))
                 try:
                     transport.auth_publickey(username, key)
                 except SSHException as e:
@@ -369,12 +550,14 @@ keymd5(host_key),
                 else:
                     if transport.is_authenticated():
                         break
+            else:
+                log.error("Error: cannot load private key '%s'", keyfile_path)
 
     def auth_none():
         log("trying none authentication")
         try:
             transport.auth_none(username)
-        except SSHException as e:
+        except SSHException:
             log("auth_none()", exc_info=True)
 
     def auth_password():
@@ -383,7 +566,30 @@ keymd5(host_key),
             transport.auth_password(username, password)
         except SSHException as e:
             log("auth_password(..)", exc_info=True)
-            log.info("SSH password authentication failed: %s", e)
+            log.info("SSH password authentication failed:")
+            log.info(" %s", getattr(e, "message", e))
+
+    def auth_interactive():
+        log("trying interactive authentication")
+        class iauthhandler:
+            def __init__(self):
+                self.authcount = 0
+            def handlestuff(self, _title, _instructions, prompt_list):
+                p = []
+                for pent in prompt_list:
+                    if self.authcount==0 and password:
+                        p.append(password)
+                    else:
+                        p.append(input_pass(pent[0]))
+                    self.authcount += 1
+                return p
+        try:
+            myiauthhandler = iauthhandler()
+            transport.auth_interactive(username, myiauthhandler.handlestuff, "")
+        except SSHException as e:
+            log("auth_interactive(..)", exc_info=True)
+            log.info("SSH password authentication failed:")
+            log.info(" %s", getattr(e, "message", e))
 
     banner = transport.get_banner()
     if banner:
@@ -392,21 +598,27 @@ keymd5(host_key),
             log.info(" %s", x)
 
     log("starting authentication")
+    # per the RFC we probably should do none first always and read off the supported
+    # methods, however, the current code seems to work fine with OpenSSH
     if not transport.is_authenticated() and NONE_AUTH:
         auth_none()
 
-    if not transport.is_authenticated() and PASSWORD_AUTH and password:
-        auth_password()
-
     if not transport.is_authenticated() and AGENT_AUTH:
         auth_agent()
 
+    # Some people do two-factor using KEY_AUTH to kick things off, so this happens first
     if not transport.is_authenticated() and KEY_AUTH:
         auth_publickey()
 
+    if not transport.is_authenticated() and PASSWORD_AUTH:
+        auth_interactive()
+
+    if not transport.is_authenticated() and PASSWORD_AUTH and password:
+        auth_password()
+
     if not transport.is_authenticated() and PASSWORD_AUTH and not password:
         for _ in range(1+PASSWORD_RETRY):
-            password = input_pass("please enter the SSH password for %s@%s" % (username, host))
+            password = input_pass("please enter the SSH password for %s@%s:" % (username, host))
             if not password:
                 break
             auth_password()
@@ -415,9 +627,12 @@ keymd5(host_key),
 
     if not transport.is_authenticated():
         transport.close()
-        raise InitException("SSH Authentication failed")
+        raise InitException("SSH Authentication on %s failed" % host)
 
-    assert len(remote_xpra)>0
+
+def paramiko_run_remote_xpra(transport, xpra_proxy_command=None, remote_xpra=None, socket_dir=None, display_as_args=None):
+    from paramiko import SSHException
+    assert remote_xpra
     log("will try to run xpra from: %s", remote_xpra)
     for xpra_cmd in remote_xpra:
         try:
@@ -442,13 +657,13 @@ keymd5(host_key),
         chan.close()
         if r!=0:
             continue
-        cmd = xpra_cmd + " " + " ".join(shellquote(x) for x in proxy_command)
+        cmd = xpra_cmd + " " + " ".join(shellquote(x) for x in xpra_proxy_command)
         if socket_dir:
             cmd += " \"--socket-dir=%s\"" % socket_dir
         if display_as_args:
             cmd += " "
             cmd += " ".join(shellquote(x) for x in display_as_args)
-        log("cmd(%s, %s)=%s", proxy_command, display_as_args, cmd)
+        log("cmd(%s, %s)=%s", xpra_proxy_command, display_as_args, cmd)
 
         #see https://github.com/paramiko/paramiko/issues/175
         #WINDOW_SIZE = 2097152
@@ -462,16 +677,7 @@ keymd5(host_key),
         else:
             log("channel exec_command(%s)" % cmd)
             chan.exec_command(cmd)
-            info = {
-                "host"  : host,
-                "port"  : port,
-                }
-            conn = SSHSocketConnection(chan, sock, target, info)
-            conn.timeout = SOCKET_TIMEOUT
-            conn.start_stderr_reader()
-            child = None
-            conn.process = (child, "ssh", cmd)
-            return conn
+            return chan
     raise Exception("all SSH remote proxy commands have failed")
 
 
@@ -504,7 +710,7 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
         elif not display_desc.get("exit_ssh", False) and not OSX:
             kwargs["preexec_fn"] = setsid
         remote_xpra = display_desc["remote_xpra"]
-        assert len(remote_xpra)>0
+        assert remote_xpra
         socket_dir = display_desc.get("socket_dir")
         proxy_command = display_desc["proxy_command"]       #ie: "_proxy_start"
         display_as_args = display_desc["display_as_args"]   #ie: "--start=xterm :10"
@@ -533,7 +739,7 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
         #non-string arguments can make Popen choke,
         #instead of lazily converting everything to a string, we validate the command:
         for x in cmd:
-            if type(x)!=str:
+            if not isinstance(x, str):
                 raise InitException("argument is not a string: %s (%s), found in command: %s" % (x, type(x), cmd))
         password = display_desc.get("password")
         if password and not display_desc.get("is_putty", False):
@@ -558,7 +764,8 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
             log.info("executing ssh command: %s" % (" ".join("\"%s\"" % x for x in cmd)))
         child = Popen(cmd, stdin=PIPE, stdout=PIPE, **kwargs)
     except OSError as e:
-        raise InitExit(EXIT_SSH_FAILURE, "Error running ssh command '%s': %s" % (" ".join("\"%s\"" % x for x in cmd), e))
+        raise InitExit(EXIT_SSH_FAILURE,
+                       "Error running ssh command '%s': %s" % (" ".join("\"%s\"" % x for x in cmd), e))
     def abort_test(action):
         """ if ssh dies, we don't need to try to read/write from its sockets """
         e = child.poll()
@@ -623,13 +830,19 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
                 child.terminate()
         except Exception as e:
             print("error trying to stop ssh tunnel process: %s" % e)
+    host = display_desc["host"]
+    port = display_desc.get("ssh-port", 22)
+    username = display_desc.get("username")
+    display = display_desc.get("display")
     info = {
-        "host"  : display_desc["host"],
-        "port"  : display_desc.get("ssh-port", 22),
+        "host"  : host,
+        "port"  : port,
         }
     from xpra.net.bytestreams import TwoFileConnection
-    target = ssh_target_string(display_desc)
-    conn = TwoFileConnection(child.stdin, child.stdout, abort_test, target=target, socktype="ssh", close_cb=stop_tunnel, info=info)
+    conn = TwoFileConnection(child.stdin, child.stdout,
+                             abort_test, target=(host, port),
+                             socktype="ssh", close_cb=stop_tunnel, info=info)
+    conn.endpoint = host_target_string("ssh", username, host, port, display)
     conn.timeout = 0            #taken care of by abort_test
     conn.process = (child, "ssh", cmd)
     return conn
