@@ -1,25 +1,26 @@
 # This file is part of Xpra.
-# Copyright (C) 2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import socket
 import struct
 import random
-try:
-    import errno
-    EMSGSIZE = errno.EMSGSIZE
-except ImportError:
-    EMSGSIZE = None
-
-from xpra.log import Logger
-log = Logger("network", "protocol", "udp")
 
 from xpra.os_util import LINUX, monotonic_time, memoryview_to_bytes
 from xpra.util import envint, repr_ellipsized
 from xpra.make_thread import start_thread
 from xpra.net.protocol import Protocol, READ_BUFFER_SIZE
 from xpra.net.bytestreams import SocketConnection, can_retry
+from xpra.log import Logger
+
+try:
+    import errno
+    EMSGSIZE = errno.EMSGSIZE
+except ImportError:
+    EMSGSIZE = None
+
+log = Logger("network", "protocol", "udp")
 
 DROP_PCT = envint("XPRA_UDP_DROP_PCT", 0)
 DROP_FIRST = envint("XPRA_UDP_DROP_FIRST", 0)
@@ -103,7 +104,7 @@ class UDPListener(object):
             try:
                 log("Protocol.close() calling %s", s.close)
                 s.close()
-            except:
+            except (OSError, IOError):
                 log.error("error closing %s", s, exc_info=True)
             self._socket = None
         log("UDPListener.close() done")
@@ -119,13 +120,13 @@ class UDPProtocol(Protocol):
         "udp-control" packets are used to synchronize both ends.
     """
 
-    def __init__(self, *args):
+    def __init__(self, *args, **kwargs):
         Protocol.__init__(self, *args)
         self.mtu = 0
         self.last_sequence = -1     #the most recent packet sequence we processed in full
         self.highest_sequence = -1
         self.jitter = 20            #20ms
-        self.uuid = 0
+        self.uuid = kwargs.get("uuid", 0)
         self.fail_cb = {}
         self.resend_cache = {}
         self.pending_packets = {}
@@ -238,13 +239,13 @@ class UDPProtocol(Protocol):
             for x in done:
                 try:
                     del self.fail_cb[x]
-                except:
+                except KeyError:
                     pass
             done = [x for x in self.resend_cache.keys() if x<=last_seq]
             for x in done:
                 try:
                     del self.resend_cache[x]
-                except:
+                except KeyError:
                     pass
         #next we can forget about sequence numbers that have been cancelled:
         #we don't need to request a re-send, and we can skip over them:
@@ -254,7 +255,7 @@ class UDPProtocol(Protocol):
                     self.can_skip.add(seqno)
                 try:
                     del self.pending_packets[seqno]
-                except:
+                except KeyError:
                     pass
             #we may now be able to move forward a bit:
             if self.pending_packets and (self.last_sequence+1) in self.can_skip:
@@ -268,12 +269,13 @@ class UDPProtocol(Protocol):
                 #hope for the best, and tell the other end to stop asking:
                 self.cancel.add(seqno)
                 continue
-            if len(missing_chunks)==0:
+            if not missing_chunks:
                 #the other end only knows it is missing the seqno,
                 #not how many chunks are missing, so send them all
                 missing_chunks = resend_cache.keys()
             if fail_cb_seq:
-                log("fail_cb[%i]=%s, missing_chunks=%s, len(resend_cache)=%i", seqno, repr_ellipsized(str(fail_cb_seq)), missing_chunks, len(resend_cache))
+                log("fail_cb[%i]=%s, missing_chunks=%s, len(resend_cache)=%i",
+                    seqno, repr_ellipsized(str(fail_cb_seq)), missing_chunks, len(resend_cache))
                 #we have a fail callback for this packet,
                 #we have to decide if we send the missing chunks or use the callback,
                 #resend if the other end is missing less than 25% of the chunks:
@@ -282,11 +284,11 @@ class UDPProtocol(Protocol):
                     #too many are missing, forget about it
                     try:
                         del self.resend_cache[seqno]
-                    except:
+                    except KeyError:
                         pass
                     try:
                         del self.fail_cb[seqno]
-                    except:
+                    except KeyError:
                         pass
                     self.cancel.add(seqno)
                     fail_cb_seq()
@@ -357,13 +359,15 @@ class UDPProtocol(Protocol):
                     return
             if any(x is None for x in ip.chunks):
                 #one of the chunks is still missing
-                log("process_udp_data: sequence %i, got chunk %i but still missing: %s", seqno, chunk, [i for i,x in enumerate(ip.chunks) if x is None])
+                log("process_udp_data: sequence %i, got chunk %i but still missing: %s",
+                    seqno, chunk, [i for i,x in enumerate(ip.chunks) if x is None])
                 self.schedule_control(self.jitter)
                 return
             #all the data is here!
             del self.pending_packets[seqno]
             data = b"".join(ip.chunks)
-        log("process_udp_data: adding packet sequence %5i to read queue (got final chunk %i, synchronous=%s)", seqno, chunk, synchronous!=0)
+        log("process_udp_data: adding packet sequence %5i to read queue (got final chunk %i, synchronous=%s)",
+            seqno, chunk, synchronous!=0)
         if seqno==self.last_sequence+1:
             self.last_sequence = seqno
         else:
@@ -415,7 +419,7 @@ class UDPProtocol(Protocol):
         log("_send_async(%s, %s)", packet, sync)
         chunks = self.encode(packet)
         if len(chunks)>1:
-            return Protocol.send_now(packet)
+            return Protocol.send_now(self, packet)
         proto_flags,index,level,data = chunks[0]
         from xpra.net.header import pack_header
         payload_size = len(data)
@@ -461,7 +465,8 @@ class UDPProtocol(Protocol):
         chunks = l // maxpayload
         if l % maxpayload > 0:
             chunks += 1
-        log("UDP.write_buf(%s, %i bytes, %s, %s) seq=%i, mtu=%s, maxpayload=%i, chunks=%i, data=%s", con, l, fail_cb, synchronous, seqno, mtu, maxpayload, chunks, repr_ellipsized(data))
+        log("UDP.write_buf(%s, %i bytes, %s, %s) seq=%i, mtu=%s, maxpayload=%i, chunks=%i, data=%s",
+            con, l, fail_cb, synchronous, seqno, mtu, maxpayload, chunks, repr_ellipsized(data))
         chunk = 0
         offset = 0
         if fail_cb:
@@ -507,6 +512,9 @@ class UDPServerProtocol(UDPProtocol):
 
 class UDPClientProtocol(UDPProtocol):
 
+    def __init__(self, *args, **kwargs):
+        UDPProtocol.__init__(self, *args, uuid = random.randint(0, 2**64-1))
+
     def con_write(self, data, fail_cb):
         """ After successfully writing some data, update the mtu value """
         r = UDPProtocol.con_write(self, data, fail_cb)
@@ -555,7 +563,7 @@ class UDPSocketConnection(SocketConnection):
 
     def close(self):
         """
-            don't close the socket, we're don't own it
+            don't close the socket, we don't own it
         """
         pass
 
