@@ -1,12 +1,18 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 import sys
+import binascii
+
+from xpra.platform import platform_import
+from xpra.os_util import bytestostr
+from xpra.log import Logger
+
 
 _init_done = False
 def init():
@@ -31,6 +37,13 @@ def do_ready():
     pass
 
 
+def use_stdin():
+    stdin = sys.stdin
+    return stdin and stdin.isatty()
+
+def get_clipboard_native_class():
+    return None
+
 #defaults:
 def get_native_tray_menu_helper_class():
     #classes that generate menus for xpra's system tray
@@ -72,12 +85,11 @@ def get_icon_size():
         dpi = 96
     if dpi > 144:
         return 48
-    elif dpi > 120:
+    if dpi > 120:
         return 32
-    elif dpi > 96:
+    if dpi > 96:
         return 24
-    else:
-        return 16
+    return 16
 
 def get_antialias_info():
     return {}
@@ -87,55 +99,62 @@ def get_display_icc_info():
     return {}
 
 def get_icc_info():
+    return default_get_icc_info()
+
+def default_get_icc_info():
     ENV_ICC_DATA = os.environ.get("XPRA_ICC_DATA")
     if ENV_ICC_DATA:
-        import binascii
         return {
-                "source"    : "environment-override",
-                "data"      : binascii.unhexlify(ENV_ICC_DATA),
-                }
-    from xpra.os_util import get_util_logger, bytestostr
-    log = get_util_logger()
+            "source"    : "environment-override",
+            "data"      : binascii.unhexlify(ENV_ICC_DATA),
+            }
+    return get_pillow_icc_info()
+
+def get_pillow_icc_info():
+    screenlog = Logger("screen")
     info = {}
     try:
         from PIL import ImageCms
-        from PIL.ImageCms import get_display_profile, getDefaultIntent
+        from PIL.ImageCms import get_display_profile
         INTENT_STR = {}
         for x in ("PERCEPTUAL", "RELATIVE_COLORIMETRIC", "SATURATION", "ABSOLUTE_COLORIMETRIC"):
             v = getattr(ImageCms, "INTENT_%s" % x, None)
             if v:
                 INTENT_STR[v] = x.lower().replace("_", "-")
-        log("get_icc_info() intents=%s", INTENT_STR)
-        def getDefaultIntentStr(_p):
-            return INTENT_STR.get(getDefaultIntent(_p), "unknown")
-        def getData(_p):
-            return _p.tobytes()
+        screenlog("get_icc_info() intents=%s", INTENT_STR)
         p = get_display_profile()
-        log("get_icc_info() display_profile=%s", p)
+        screenlog("get_icc_info() display_profile=%s", p)
         if p:
-            for (k, fn) in {
-                            "name"          : "getProfileName",
-                            "info"          : "getProfileInfo",
-                            "copyright"     : "getProfileCopyright",
-                            "manufacturer"  : "getProfileManufacturer",
-                            "model"         : "getProfileModel",
-                            "description"   : "getProfileDescription",
-                            "default-intent": "getDefaultIntentStr",
-                            "data"          : "getData",
-                            }.items():
+            def getDefaultIntentStr(v):
+                return INTENT_STR.get(v, "unknown")
+            def getData(v):
+                return v.tobytes()
+            for (k, fn, conv) in (
+                ("name",            "getProfileName",           None),
+                ("info",            "getProfileInfo",           None),
+                ("copyright",       "getProfileCopyright",      None),
+                ("manufacturer",    "getProfileManufacturer",   None),
+                ("model",           "getProfileModel",          None),
+                ("description",     "getProfileDescription",    None),
+                ("default-intent",  "getDefaultIntent",         getDefaultIntentStr),
+                ("data",            "getData",                  getData),
+                ):
                 m = getattr(ImageCms, fn, None)
-                if not m:
-                    log("%s lacks %s", ImageCms, fn)
+                if m is None:
+                    screenlog("%s lacks %s", ImageCms, fn)
                     continue
                 try:
                     v = m(p)
+                    if conv:
+                        v = conv(v)
                     info[k] = bytestostr(v).rstrip("\n\r")
                 except Exception as e:
-                    log("ICC profile error on %s using %s: %s", k, fn, e)
+                    screenlog("get_icc_info()", exc_info=True)
+                    screenlog("ICC profile error on %s using %s: %s", k, fn, e)
     except Exception as e:
-        log("get_icc_info()", exc_info=True)
-        log.warn("Warning: cannot query ICC profiles:")
-        log.warn(" %s", e)
+        screenlog("get_icc_info()", exc_info=True)
+        screenlog.warn("Warning: cannot query ICC profiles:")
+        screenlog.warn(" %s", e)
     return info
 
 
@@ -171,6 +190,12 @@ def get_fixed_cursor_size():
 def get_cursor_size():
     return -1
 
+def get_window_min_size():
+    return 0, 0
+
+def get_window_max_size():
+    return 2**15-1, 2**15-1
+
 def get_window_frame_size(_x, _y, _w, _h):
     return None
 
@@ -178,29 +203,26 @@ def get_window_frame_sizes():
     return {}
 
 
-def add_window_hooks(window):
+def add_window_hooks(_window):
     pass
 
-def remove_window_hooks(window):
+def remove_window_hooks(_window):
     pass
 
 
-def show_desktop(show):
+def show_desktop(_show):
     pass
 
-def set_fullscreen_monitors(window, fsm, source_indication=0):
+def set_fullscreen_monitors(_window, _fsm, _source_indication=0):
     pass
 
-def set_shaded(window, shaded):
+def set_shaded(_window, _shaded):
     pass
 
 
 def gl_check():
     return None     #no problem
 
-def get_menu_support_function():
-    return None
-
 def get_wm_name():
     return None
 
@@ -213,11 +235,12 @@ def get_info_base():
     def fname(v):
         try:
             return v.__name__
-        except:
+        except AttributeError:
             return str(v)
     def fnames(l):
         return [fname(x) for x in l]
     return {
+            "native-clipboard"              : fname(get_clipboard_native_class()),
             "native_tray_menu_helper"       : fname(get_native_tray_menu_helper_class()),
             "native_trays"                  : fnames(get_native_tray_classes()),
             "native_system_trays"           : fnames(get_native_system_tray_classes()),
@@ -251,16 +274,16 @@ def get_info_base():
 get_info = get_info_base
 
 
-from xpra.platform import platform_import
 platform_import(globals(), "gui", False,
                 "do_ready",
                 "do_init",
                 "gl_check",
+                "use_stdin",
                 "get_wm_name",
                 "show_desktop", "set_fullscreen_monitors", "set_shaded",
                 "ClientExtras",
                 "take_screenshot",
-                "get_menu_support_function",
+                "get_clipboard_native_class",
                 "get_native_tray_menu_helper_class",
                 "get_native_tray_classes",
                 "get_native_system_tray_classes",
@@ -270,6 +293,7 @@ platform_import(globals(), "gui", False,
                 "get_number_of_desktops", "get_desktop_names",
                 "get_antialias_info", "get_icc_info", "get_display_icc_info", "get_xdpi", "get_ydpi",
                 "get_icon_size",
+                "get_window_min_size", "get_window_max_size",
                 "get_mouse_config",
                 "get_double_click_time", "get_double_click_distance",
                 "get_fixed_cursor_size", "get_cursor_size", "get_window_frame_sizes",
@@ -294,11 +318,8 @@ def main():
 
         #naughty, but how else can I hook this up?
         if POSIX and not OSX:
-            try:
-                from xpra.x11.bindings.posix_display_source import init_posix_display_source    #@UnresolvedImport
-                init_posix_display_source()
-            except:
-                pass    #maybe running on OSX? hope for the best..
+            from xpra.x11.bindings.posix_display_source import init_posix_display_source    #@UnresolvedImport
+            init_posix_display_source()
         i = get_info()
         print_nested_dict(i, hex_keys=("data", "icc-data", "icc-profile"))
 
