@@ -1,15 +1,17 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2014-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2014-2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import sys, os
+import os
+import sys
 
 #default implementation uses pycups
 from xpra.util import envbool, print_nested_dict
 from xpra.os_util import WIN32, PYTHON3
 from xpra.log import Logger
+
 log = Logger("printing")
 
 RAW_MODE = envbool("XPRA_PRINTER_RAW", False)
@@ -24,7 +26,7 @@ def err(*args, **kwargs):
 def get_printers():
     return {}
 
-def get_printer_attributes(name):
+def get_printer_attributes(_name):
     return []
 
 def get_default_printer():
@@ -33,10 +35,10 @@ def get_default_printer():
 def print_files(printer, filenames, title, options):
     raise Exception("no print implementation available")
 
-def printing_finished(printpid):
+def printing_finished(_printpid):
     return True
 
-def init_printing(printers_modified_callback=None):
+def init_printing(printers_modified_callback=None):     #pylint: disable=unused-argument
     pass
 
 def cleanup_printing():
@@ -70,7 +72,7 @@ def get_mimetypes():
 
 
 def get_info():
-    return default_get_info
+    return default_get_info()
 
 def default_get_info():
     return {
@@ -86,11 +88,20 @@ from xpra.platform import platform_import
 if not WIN32:
     #pycups is not available on win32
     try:
-        from xpra.platform.pycups_printing import get_printers, print_files, printing_finished, init_printing, cleanup_printing, get_info
+        from xpra.platform.pycups_printing import (
+            get_printers,
+            print_files,
+            printing_finished,
+            init_printing,
+            cleanup_printing,
+            get_info,
+            )
         assert get_printers and print_files and printing_finished and init_printing, cleanup_printing
     except Exception as e:
+        log("cannot load pycups", exc_info=True)
         err("Error: printing disabled:")
         err(" %s", e)
+        del e
 
 platform_import(globals(), "printing", False,
                 "init_printing",
@@ -110,11 +121,11 @@ def main():
         enable_debug_for("printing")
         try:
             sys.argv.remove("-v")
-        except:
+        except ValueError:
             pass
         try:
             sys.argv.remove("--verbose")
-        except:
+        except ValueError:
             pass
 
     from xpra.util import nonl, pver
@@ -123,7 +134,7 @@ def main():
         try:
             for pk,pv in d.items():
                 try:
-                    if type(pv)==unicode:
+                    if isinstance(pv, unicode):
                         sv = pv.encode("utf8")
                     else:
                         sv = nonl(pver(pv))
@@ -163,7 +174,7 @@ def main():
             dump_info(get_info())
             return 0
         printers = get_printers()
-        if len(printers)==0:
+        if not printers:
             print("Cannot print: no printers found")
             return 1
         if len(sys.argv)==2:
@@ -171,7 +182,7 @@ def main():
             if not os.path.exists(filename):
                 print("Cannot print file '%s': file does not exist" % filename)
                 return 1
-            printer = get_default_printer()
+            printer = get_default_printer()     #pylint: disable=assignment-from-none
             if not printer:
                 printer = printers.keys()[0]
                 if len(printers)>1:
