@@ -1,12 +1,24 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2017-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2017-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from ctypes import WinDLL, POINTER, WINFUNCTYPE, GetLastError, Structure, c_ulong, c_ushort, c_ubyte, c_int, c_long, c_void_p, c_size_t, c_char, byref, sizeof
-from ctypes.wintypes import HWND, DWORD, WPARAM, LPARAM, HDC, HMONITOR, HMODULE, SHORT, ATOM, RECT, POINT, MAX_PATH, WCHAR, BYTE
-from ctypes.wintypes import HANDLE, LPCWSTR, UINT, INT, BOOL, WORD, HGDIOBJ, LONG, LPVOID, HBITMAP, LPCSTR, LPWSTR, HWINSTA, HINSTANCE, HMENU
+from ctypes import (
+    WinDLL, POINTER, WINFUNCTYPE,
+    GetLastError, Structure,
+    c_ulong, c_ushort, c_ubyte, c_int, c_long, c_void_p, c_size_t, c_char,
+    byref, sizeof,
+    )
+from ctypes.wintypes import (
+    HWND, DWORD, WPARAM, LPARAM, HDC, HMONITOR, HMODULE,
+    SHORT, ATOM, RECT, POINT, MAX_PATH, WCHAR, BYTE,
+    )
+from ctypes.wintypes import (
+    HANDLE, LPSTR, LPCWSTR, UINT, INT, BOOL, WORD, HGDIOBJ,
+    LONG, LPVOID, HBITMAP, LPCSTR, LPWSTR, HWINSTA,
+    HINSTANCE, HMENU, ULONG,
+    )
 #imported from this module but not used here:
 assert GetLastError
 
@@ -128,12 +140,20 @@ class MONITORINFOEX(Structure):
         ('szDevice', WCHAR * CCHDEVICENAME),
         ]
 
+class SECURITY_ATTRIBUTES(Structure):
+    _fields_ = [
+        ("nLength",                 c_int),
+        ("lpSecurityDescriptor",    c_void_p),
+        ("bInheritHandle",          c_int),
+        ]
+LPSECURITY_ATTRIBUTES = POINTER(SECURITY_ATTRIBUTES)
+
 def GetMonitorInfo(hmonitor):
     info = MONITORINFOEX()
     info.szDevice = ""
     info.cbSize = sizeof(MONITORINFOEX)
     if not GetMonitorInfoW(hmonitor, byref(info)):
-        raise WindowsError()
+        raise WindowsError()    #@UndefinedVariable
     monitor = info.rcMonitor.left, info.rcMonitor.top, info.rcMonitor.right, info.rcMonitor.bottom
     work = info.rcWork.left, info.rcWork.top, info.rcWork.right, info.rcWork.bottom
     return  {
@@ -144,15 +164,20 @@ def GetMonitorInfo(hmonitor):
         }
 
 kernel32 = WinDLL("kernel32", use_last_error=True)
+LocalFree = kernel32.LocalFree
+FormatMessageW = kernel32.FormatMessageW
 SetConsoleTitleA = kernel32.SetConsoleTitleA
 SetConsoleTitleA.restype = INT
-SetConsoleTitleA.argtypes = [LPCTSTR] 
+SetConsoleTitleA.argtypes = [LPCTSTR]
 GetConsoleScreenBufferInfo = kernel32.GetConsoleScreenBufferInfo
 GetModuleHandleA = kernel32.GetModuleHandleA
 GetModuleHandleA.restype = HMODULE
 GetModuleHandleW = kernel32.GetModuleHandleW
 GetModuleHandleW.restype = HMODULE
+ConsoleCtrlHandler = WINFUNCTYPE(BOOL, DWORD)
 SetConsoleCtrlHandler = kernel32.SetConsoleCtrlHandler
+SetConsoleCtrlHandler.argtypes = (ConsoleCtrlHandler, BOOL)
+SetConsoleCtrlHandler.restype = BOOL
 GetComputerNameW = kernel32.GetComputerNameW
 GetComputerNameW.restype = BOOL
 GetComputerNameW.argtypes = [LPWSTR, LPDWORD]
@@ -171,6 +196,29 @@ CloseHandle.restype = BOOL
 GetProductInfo = kernel32.GetProductInfo
 GetProductInfo.argtypes = [DWORD, DWORD, DWORD, DWORD, PDWORD]
 GetProductInfo.restype  = BOOL
+GetStdHandle = WINFUNCTYPE(HANDLE, DWORD)(("GetStdHandle", kernel32))
+HGLOBAL = HANDLE
+GlobalAlloc = kernel32.GlobalAlloc
+GlobalAlloc.restype = HGLOBAL
+GlobalAlloc.argtypes = [UINT, c_size_t]
+GlobalFree = kernel32.GlobalFree
+GlobalFree.restype = HGLOBAL
+GlobalFree.argtypes = [HGLOBAL]
+GlobalLock = kernel32.GlobalLock
+GlobalLock.restype = LPVOID
+GlobalLock.argtypes = [HGLOBAL]
+GlobalUnlock = kernel32.GlobalUnlock
+GlobalUnlock.restype = BOOL
+GlobalUnlock.argtypes = [HGLOBAL]
+WideCharToMultiByte = kernel32.WideCharToMultiByte
+WideCharToMultiByte.restype = c_int
+WideCharToMultiByte.argtypes = [UINT, DWORD, LPCWSTR, c_int, c_void_p, c_int, LPCSTR, POINTER(BOOL)]
+MultiByteToWideChar = kernel32.MultiByteToWideChar
+MultiByteToWideChar.restype = c_int
+MultiByteToWideChar.argtypes =  [UINT, DWORD, LPCSTR, c_int, LPWSTR, c_int]
+WTSGetActiveConsoleSessionId = kernel32.WTSGetActiveConsoleSessionId
+WTSGetActiveConsoleSessionId.restype = DWORD
+WTSGetActiveConsoleSessionId.argtypes = []
 
 user32 = WinDLL("user32", use_last_error=True)
 RegisterClassExA = user32.RegisterClassExA
@@ -186,9 +234,13 @@ UnregisterClassW.restype = BOOL
 UnregisterClassW.argtypes = [LPCWSTR, HINSTANCE]
 CreateWindowExA = user32.CreateWindowExA
 CreateWindowExA.restype = HWND
+#CreateWindowExA.argtypes = [DWORD, ATOM, LPCTSTR, DWORD, c_int, c_int, c_int, c_int, HWND, HMENU, HINSTANCE, LPVOID]
+CreateWindowExW = user32.CreateWindowExW
+CreateWindowExW.restype = HWND
+CreateWindowExW.argtypes = [DWORD, ATOM, LPCWSTR, DWORD, c_int, c_int, c_int, c_int, HWND, HMENU, HINSTANCE, LPVOID]
 DestroyWindow = user32.DestroyWindow
 DestroyWindow.restype = BOOL
-DestroyWindow.argtypes = [HWND] 
+DestroyWindow.argtypes = [HWND]
 DefWindowProcA = user32.DefWindowProcA
 DefWindowProcW = user32.DefWindowProcW
 MessageBoxA = user32.MessageBoxA
@@ -220,7 +272,7 @@ PostMessageA = user32.PostMessageA
 FindWindowA = user32.FindWindowA
 GetWindowRect = user32.GetWindowRect
 GetDoubleClickTime = user32.GetDoubleClickTime
-EnumDisplayMonitors = user32.EnumDisplayMonitors
+_EnumDisplayMonitors = user32.EnumDisplayMonitors
 MonitorFromWindow = user32.MonitorFromWindow
 MonitorFromWindow.restype = HMONITOR
 GetMonitorInfoW = user32.GetMonitorInfoW
@@ -312,12 +364,18 @@ OpenWindowStationW = user32.OpenWindowStationW
 OpenWindowStationW.restype = HWINSTA
 ACCESS_MASK = DWORD
 OpenWindowStationW.argtypes = [LPWSTR, BOOL, ACCESS_MASK]
+OpenWindowStationA = user32.OpenWindowStationA
+OpenWindowStationA.restype = HWINSTA
+OpenWindowStationA.argtypes = [LPCSTR, BOOL, ACCESS_MASK]
 GetProcessWindowStation = user32.GetProcessWindowStation
 GetProcessWindowStation.restype = HWINSTA
 GetProcessWindowStation.argtypes = []
 SetProcessWindowStation = user32.SetProcessWindowStation
 SetProcessWindowStation.restype = BOOL
 SetProcessWindowStation.argtypes = [HWINSTA]
+CreateWindowStationA = user32.CreateWindowStationA
+CreateWindowStationA.restype = HWINSTA
+CreateWindowStationA.argtypes = [LPCSTR, DWORD, ACCESS_MASK, LPSECURITY_ATTRIBUTES]
 CloseWindowStation = user32.CloseWindowStation
 CloseWindowStation.restype = BOOL
 CloseWindowStation.argtypes = [HWINSTA]
@@ -330,7 +388,7 @@ CloseDesktop.restype = BOOL
 CloseDesktop.argtypes = [HDESK]
 OpenInputDesktop = user32.OpenInputDesktop
 OpenInputDesktop.restype = HDESK
-OpenInputDesktop.argtypes = [DWORD, BOOL, ACCESS_MASK] 
+OpenInputDesktop.argtypes = [DWORD, BOOL, ACCESS_MASK]
 GetUserObjectInformationA = user32.GetUserObjectInformationA
 GetUserObjectInformationA.restype = BOOL
 GetUserObjectInformationA.argtypes = [HANDLE, INT, LPVOID, DWORD, LPDWORD]
@@ -340,6 +398,108 @@ CreatePopupMenu.argtypes = []
 AppendMenu = user32.AppendMenuW
 AppendMenu.restype = BOOL
 AppendMenu.argtypes = [HMENU, UINT, UINT, LPCWSTR]
+#clipboard functions:
+class COPYDATASTRUCT(Structure):
+    _fields_ = [
+        ("dwData",          POINTER(ULONG)),
+        ("cbData",          DWORD),
+        ("lpData",          LPVOID),
+    ]
+OpenClipboard = user32.OpenClipboard
+OpenClipboard.restype = BOOL
+OpenClipboard.argtypes = [HWND]
+CloseClipboard = user32.CloseClipboard
+CloseClipboard.restype = BOOL
+CloseClipboard.argtypes = []
+EmptyClipboard = user32.EmptyClipboard
+EmptyClipboard.restype = BOOL
+EmptyClipboard.argtypes = []
+SetClipboardData = user32.SetClipboardData
+SetClipboardData.restype = HANDLE
+SetClipboardData.argtypes = [UINT, HANDLE]
+RegisterClipboardFormatA = user32.RegisterClipboardFormatA
+RegisterClipboardFormatA.restype = UINT
+RegisterClipboardFormatA.argtypes = [LPCSTR]
+GetClipboardFormatNameA = user32.GetClipboardFormatNameA
+GetClipboardFormatNameA.restype = c_int
+GetClipboardFormatNameA.argtypes = [UINT, LPCTSTR, c_int]
+RegisterClipboardFormatA = user32.RegisterClipboardFormatA
+RegisterClipboardFormatA.restype = UINT
+RegisterClipboardFormatA.argtypes = [LPCSTR]
+GetClipboardData = user32.GetClipboardData
+GetClipboardData.restype = HANDLE
+GetClipboardData.argtypes = [UINT]
+GetClipboardSequenceNumber = user32.GetClipboardSequenceNumber
+GetClipboardSequenceNumber.restype = DWORD
+GetClipboardSequenceNumber.argtypes = []
+GetClipboardOwner = user32.GetClipboardOwner
+GetClipboardOwner.restype = HWND
+GetClipboardOwner.argtypes = []
+GetOpenClipboardWindow = user32.GetOpenClipboardWindow
+GetOpenClipboardWindow.restype = HWND
+GetOpenClipboardWindow.argtypes = []
+AddClipboardFormatListener = user32.AddClipboardFormatListener
+AddClipboardFormatListener.restype = BOOL
+AddClipboardFormatListener.argtypes = [HWND]
+RemoveClipboardFormatListener = user32.RemoveClipboardFormatListener
+RemoveClipboardFormatListener.restype = BOOL
+RemoveClipboardFormatListener.argtypes = [HWND]
+GetUpdatedClipboardFormats = user32.GetUpdatedClipboardFormats
+GetUpdatedClipboardFormats.restype = BOOL
+GetUpdatedClipboardFormats.argtypes = [POINTER(UINT), UINT, POINTER(UINT)]
+IsClipboardFormatAvailable = user32.IsClipboardFormatAvailable
+IsClipboardFormatAvailable.restype = BOOL
+IsClipboardFormatAvailable.argtypes = [UINT]
+SetClipboardViewer = user32.SetClipboardViewer
+SetClipboardViewer.restype = HWND
+SetClipboardViewer.argtypes = [HWND]
+GetClipboardFormatNameA = user32.GetClipboardFormatNameA
+GetClipboardFormatNameA.restype = c_int
+GetClipboardFormatNameA.argtypes = [UINT, LPCSTR, c_int]
+GetClipboardFormatNameW = user32.GetClipboardFormatNameW
+GetClipboardFormatNameW.restype = c_int
+GetClipboardFormatNameW.argtypes = [UINT, LPWSTR, c_int]
+EnumClipboardFormats = user32.EnumClipboardFormats
+EnumClipboardFormats.restype = UINT
+EnumClipboardFormats.argtypes = [UINT]
+CountClipboardFormats = user32.CountClipboardFormats
+CountClipboardFormats.restype = c_int
+CountClipboardFormats.argtypes = []
+
+class PROCESS_INFORMATION(Structure):
+    _fields_ = (
+        ('_hProcess',   HANDLE),
+        ('_hThread',    HANDLE),
+        ('dwProcessId', DWORD),
+        ('dwThreadId',  DWORD),
+        )
+PPROCESS_INFORMATION = POINTER(PROCESS_INFORMATION)
+class STARTUPINFOA(Structure):
+    _fields_ = (
+        ('cb',              DWORD),
+        ('lpReserved',      LPWSTR),
+        ('lpDesktop',       LPWSTR),
+        ('lpTitle',         LPWSTR),
+        ('dwX',             DWORD),
+        ('dwY',             DWORD),
+        ('dwXSize',         DWORD),
+        ('dwYSize',         DWORD),
+        ('dwXCountChars',   DWORD),
+        ('dwYCountChars',   DWORD),
+        ('dwFillAttribute', DWORD),
+        ('dwFlags',         DWORD),
+        ('wShowWindow',     WORD),
+        ('cbReserved2',     WORD),
+        ('lpReserved2',     c_void_p),
+        ('hStdInput',       HANDLE),
+        ('hStdOutput',      HANDLE),
+        ('hStdError',       HANDLE),
+        )
+PSTARTUPINFOA = POINTER(STARTUPINFOA)
+advapi32 = WinDLL("advapi32")
+CreateProcessAsUserA = advapi32.CreateProcessAsUserA
+CreateProcessAsUserA.restype = BOOL
+CreateProcessAsUserA.argtypes = [HANDLE, LPCSTR, LPSTR, LPSECURITY_ATTRIBUTES, LPSECURITY_ATTRIBUTES, BOOL, DWORD, LPVOID, LPCSTR, LPCSTR, PSTARTUPINFOA, PPROCESS_INFORMATION]
 
 gdi32 = WinDLL("gdi32", use_last_error=True)
 CreateCompatibleDC = gdi32.CreateCompatibleDC
@@ -408,9 +568,8 @@ GetObjectA.restype = INT
 
 #wrap EnumDisplayMonitors to hide the callback function:
 MonitorEnumProc = WINFUNCTYPE(BOOL, HMONITOR, HDC, POINTER(RECT), LPARAM)
-EnumDisplayMonitors.argtypes = [HDC, POINTER(RECT), MonitorEnumProc, LPARAM]
-EnumDisplayMonitors.restype = BOOL
-_EnumDisplayMonitors = EnumDisplayMonitors
+_EnumDisplayMonitors.argtypes = [HDC, POINTER(RECT), MonitorEnumProc, LPARAM]
+_EnumDisplayMonitors.restype = BOOL
 def EnumDisplayMonitors():
     results = []
     def _callback(monitor, _dc, _rect, _data):
@@ -465,3 +624,69 @@ class GUID(Structure):
 
 IID = GUID
 REFIID = POINTER(IID)
+
+
+ERROR_BROKEN_PIPE = 109
+ERROR_PIPE_NOT_CONNECTED = 233
+ERROR_MORE_DATA = 234
+ERROR_BROKEN_PIPE = 109
+ERROR_NO_DATA = 232
+ERROR_HANDLE_EOF = 38
+ERROR_IO_INCOMPLETE = 996
+ERROR_IO_PENDING = 997
+ERROR_MORE_DATA = 234
+ERROR_CANCELLED = 1223
+ERROR_ACCESS_DENIED = 5
+ERROR_INVALID_HANDLE = 6
+ERROR_OPERATION_ABORTED = 995
+ERROR_INVALID_PARAMETER = 87
+ERROR_SUCCESS = 0
+ERROR_COUNTER_TIMEOUT = 1121
+ERROR_PIPE_BUSY = 231
+
+IO_ERROR_STR = {
+    ERROR_PIPE_NOT_CONNECTED    : "PIPE_NOT_CONNECTED",
+    ERROR_MORE_DATA             : "MORE_DATA",
+    ERROR_BROKEN_PIPE           : "BROKEN_PIPE",
+    ERROR_NO_DATA               : "NO_DATA",
+    ERROR_HANDLE_EOF            : "HANDLE_EOF",
+    ERROR_IO_INCOMPLETE         : "IO_INCOMPLETE",
+    ERROR_IO_PENDING            : "IO_PENDING",
+    ERROR_MORE_DATA             : "MORE_DATA",
+    ERROR_CANCELLED             : "CANCELLED",
+    ERROR_ACCESS_DENIED         : "ACCESS_DENIED",
+    ERROR_INVALID_HANDLE        : "INVALID_HANDLE",
+    ERROR_OPERATION_ABORTED     : "OPERATION_ABORTED",
+    ERROR_INVALID_PARAMETER     : "INVALID_PARAMETER",
+    ERROR_SUCCESS               : "SUCCESS",
+    ERROR_COUNTER_TIMEOUT       : "COUNTER_TIMEOUT",
+    ERROR_PIPE_BUSY             : "PIPE_BUSY",
+    }
+
+#https://gist.github.com/EBNull/6135237
+LANG_NEUTRAL = 0x00
+SUBLANG_NEUTRAL = 0x00
+SUBLANG_DEFAULT = 0x01
+
+LANG_ENGLISH = 0x09
+SUBLANG_ENGLISH_US = 0x01
+
+def MAKELANGID(primary, sublang):
+    return (primary & 0xFF) | (sublang & 0xFF) << 16
+
+LCID_ENGLISH = MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US)
+LCID_DEFAULT = MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT)
+LCID_NEUTRAL = MAKELANGID(LANG_NEUTRAL, SUBLANG_NEUTRAL)
+
+def FormatMessageSystem(message_id, langid=LCID_ENGLISH):
+    from xpra.platform.win32.constants import FORMAT_MESSAGE_ALLOCATE_BUFFER, FORMAT_MESSAGE_FROM_SYSTEM, FORMAT_MESSAGE_IGNORE_INSERTS
+    sys_flag = FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS
+    bufptr = LPWSTR()
+    chars = kernel32.FormatMessageW(sys_flag, None, message_id, langid, byref(bufptr), 0, None)
+    if not chars:
+        chars = FormatMessageW(sys_flag, None, message_id, LCID_NEUTRAL, byref(bufptr), 0, None)
+        if not chars:
+            return str(message_id)
+    val = bufptr.value[:chars]
+    LocalFree(bufptr)
+    return val
